
package org.netbeans.modules.asm.hyperlink;

import java.io.IOException;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JEditorPane;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import org.netbeans.lib.editor.hyperlink.spi.HyperlinkProvider;
import org.netbeans.modules.editor.NbEditorUtilities;
import org.openide.cookies.EditCookie;
import org.openide.cookies.EditorCookie;
import org.openide.cookies.OpenCookie;
import org.openide.loaders.DataObject;
import org.openide.windows.TopComponent;

import org.netbeans.modules.asm.model.lang.LabelInstance;
import org.netbeans.modules.asm.model.lang.AsmOffsetable;
import org.netbeans.modules.asm.model.AsmModelAccessor;
import org.netbeans.modules.asm.model.AsmModel;
import org.netbeans.modules.asm.model.lang.OperandInstance;
import org.netbeans.modules.asm.model.lang.operand.LabelOperand;

public class AsmHyperlinkProvider implements HyperlinkProvider {

    private Document lastDocument;
    private AsmOffsetable lastFrom;
    private AsmOffsetable lastTo;    
      
    
    //ToDo: rewrite :-)
    
    public boolean isHyperlinkPoint(Document doc, int offset) {
        DataObject obj = NbEditorUtilities.getDataObject(doc);
        
       
        AsmModel model = (AsmModel) obj.getCookie(AsmModel.class);
        
        if (model == null) {
            return false;
        }
        
        String text;        
        lastDocument = null; 
        lastFrom = lastTo = null;
        
        try {
            text = doc.getText(0, doc.getLength());
        } catch (BadLocationException ex) {           
            return false;
        }
        
        AsmModelAccessor acc = model.parse(new StringReader(text));                
        List<AsmOffsetable> elements = acc.getResult();
                
        List<LabelInstance> labels = new LinkedList<LabelInstance>();
        LabelOperand source = null;
        
        for (AsmOffsetable el: elements) {
            if (el.getStartOffset() <= offset && el.getEndOffset() >= offset) {
                if(el instanceof OperandInstance)  {
                    OperandInstance op = (OperandInstance) el;
                    if (op.getOperand() instanceof LabelOperand) {
                        source = (LabelOperand) op.getOperand(); 
                        lastFrom = el;
                    }                    
                }              
            }
            
            else if (el instanceof LabelInstance) {
                labels.add((LabelInstance) el);
            }
        }  
        
        if (source == null) {
            return false;
        }
        
        for (LabelInstance label: labels) {
            if (label.getName().equals(source.getName())) {
                if (source.getDirection() == LabelOperand.Direction.BIDIRECTIONAL || 
                     (source.getDirection() == LabelOperand.Direction.BACK &&
                      label.getEndOffset() < lastFrom.getStartOffset())
                   ) {
                    lastDocument = doc;
                    lastTo = label;                
                }
            }
        }
        
        if (lastTo != null) {
            return true;
        }
        
        return false;
    }

    public int[] getHyperlinkSpan(Document doc, int offset) {
        if(doc != lastDocument || lastFrom == null) {
            return null;
        }
        
        return new int[] { lastFrom.getStartOffset(), 
                           lastFrom.getEndOffset() 
                         };                 
    }

    public void performClickAction(Document doc, int offset) {
        
        if(doc != lastDocument || lastTo == null) {
            return;
        }
        
        DataObject ob = NbEditorUtilities.getDataObject(doc);
        int position = lastTo.getStartOffset();
        
        if (!openFileInEditor(ob)) {
            return;
        }
        
        EditorCookie ed = (EditorCookie) ob.getCookie(EditorCookie.class);
        if (ed != null) {
            try {
                ed.openDocument();
            } catch (IOException ex) {
                return;
            }
            JEditorPane pane = ed.getOpenedPanes()[0];
            pane.setSelectionStart(position);
            pane.setSelectionEnd(position);
            
            TopComponent tc = (TopComponent)
                    SwingUtilities.getAncestorOfClass(TopComponent.class,
                                                      pane);
            if (tc != null) { 
                tc.requestActive();
            }
        }
    }

                    
    private boolean openFileInEditor(DataObject ob) {
        EditCookie ck = (EditCookie) ob.getCookie(EditCookie.class);
        if (ck != null) {
            ck.edit();
            return true;
        }
        OpenCookie oc = (OpenCookie) ob.getCookie(OpenCookie.class);
        if (oc != null) {
            oc.open();
            return true;
        }
        return false;
    }
    
}
