/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** 
 * Represents global UC notification
 */
public class GlobalNotif extends Object {

    private String notifText = null;
    private String notifURL = null;
    public static final String GLOBAL_NOTIF_FILE = "global_notif_file";
    public static final String GLOBAL_NOTIF_URL = "global_notif_url";

    /**
     * Creates Module object from info.xml file
     * 
     */
    public GlobalNotif() {
        notifURL = UpdateCenter.getNotifPath();
        File file = new File(notifURL, "notif.txt");
        String notifFile;
        if (file.canRead()) {
            notifFile = file.getAbsolutePath();
        } else {
            notifFile = null;
        }
        System.out.println("Global notif file " + notifFile);
        if (notifFile == null) {
            return;
        }
        try {
            String one_line;
            StringBuffer notif_text = new StringBuffer();

            BufferedReader in = new BufferedReader(new FileReader(notifFile));
            while ((one_line = in.readLine()) != null) {
                notif_text.append(one_line);
                notif_text.append("\n");
            }
            in.close();
            this.notifText = notif_text.toString();
        } catch (IOException e) {
            throw new IllegalStateException("Failure. Can't read file ", e);
        }
    }

    public void toCatalogElement(Element parElement) {
        if (notifText == null) {
            return;
        }
        Document resDoc = parElement.getOwnerDocument();
        Element notif_element = resDoc.createElement("notification");
        notif_element.setAttribute("notification_url", notifURL);
        parElement.appendChild(notif_element);
        CDATASection notif_element_text = resDoc.createCDATASection(notifText);
        notif_element.appendChild(notif_element_text);
    }
}
