/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;

import java.util.ArrayList;
import org.w3c.dom.*;

/** 
 * Represents Autoupdate Module Catalog
 */
public class ModulesCatalog {

    private List module_groups = new ArrayList();
    private List modules = new ArrayList();
    private Set all_groups = new HashSet();
    private Set all_modules = new HashSet();
    private Set live_modules = new HashSet();
    private Set staged_modules = new HashSet();
    static boolean staging = false;
    static String language = null;

    /**
     * Creates Module Catalog object
     */
    public ModulesCatalog(File nbmsDir) {
        loadFromNBMs(nbmsDir);
    }

    /**
     * Sets staging flag
     */
    public void setStagingFlag(boolean flag) {
        staging = flag;
    }

    /**
     * Sets catalog language to String languagae
     */
    public void setLanguage(String language) {
        ModulesCatalog.language = language;
    }

    /**
     * Returns catalog language
     */
    public String getLanguage() {
        return (language);
    }

    /**
     * Returns Iterator for list of modules
     */
    public Iterator getModulesIterator() {
        return all_modules.iterator();
    }

    public List<Module> getModules() {
        return Collections.unmodifiableList(new ArrayList(all_modules));
    }

    /**
     * Returns Iterator for list of groups
     */
    public Iterator getGroupsIterator() {
        return (all_groups.iterator());
    }

    /**
     * Converts Catalog Document to XML format
     */
    public Document toCatalogDocument() {
        Document result_doc = XMLutils.newXML();
        Element updates_element = result_doc.createElement("module_updates");
        SimpleDateFormat formatter = new SimpleDateFormat("ss/mm/HH/dd/MM/yyyy");
        updates_element.setAttribute("timestamp", formatter.format(new Date()));
        result_doc.appendChild(updates_element);

        GlobalNotif global_notif = new GlobalNotif();
        global_notif.toCatalogElement(updates_element);

        toCatalogElement(updates_element);

        return (result_doc);
    }

    /**
     * Stores Module Catalog as <module_updates> element
     */
    public void toCatalogElementNb(Element updatesElement) {
        Document result_doc = updatesElement.getOwnerDocument();

        Module one_module;
        for (Iterator i = modules.listIterator(); i.hasNext();) {
            one_module = (Module) i.next();
            one_module.toCatalogElement(updatesElement);
        }

        ModuleGroup one_module_group;
        for (Iterator i = module_groups.listIterator(); i.hasNext();) {
            one_module_group = (ModuleGroup) i.next();
            one_module_group.toCatalogElement(updatesElement, true);
        }

        Licenses.toElement(result_doc, updatesElement);
    }

    /**
     * Stores Module Catalog as <module_updates> element
     */
    public void toCatalogElement(Element updatesElement) {
        Document result_doc = updatesElement.getOwnerDocument();

        Module one_module;
        for (Iterator i = modules.listIterator(); i.hasNext();) {
            one_module = (Module) i.next();
            one_module.toCatalogElement(updatesElement);
        }

        ModuleGroup one_module_group;
        for (Iterator i = module_groups.listIterator(); i.hasNext();) {
            one_module_group = (ModuleGroup) i.next();
            one_module_group.toCatalogElement(updatesElement, true);
        }

        Licenses.toElement(result_doc, updatesElement);
    }

    /**
     * Loading Module Catalog from directory structure with NBM files 
     */
    private void loadFromNBMs(File nbmsDir) {
        File one_file;
        ModuleGroup new_group;
        Module new_module;

        System.out.println();
        System.out.println("Loading modules from NBMs from path " + nbmsDir + ":");
        System.out.println("/");
        AppSetup.init();
        for (Iterator i = new NBMiterator(nbmsDir); i.hasNext();) {
            one_file = (File) i.next();
            if (one_file.isDirectory()) {
                new_group = new ModuleGroup(one_file, null);
                module_groups.add(new_group);
                all_modules.addAll(new_group.getAllModules());
            } else {
                NBMfile nbm_file = new NBMfile(one_file);
                new_module = new Module(nbm_file, null);
                modules.add(new_module);
                all_modules.add(new_module);
            }
        }
        initModuleList();
        removeUpdatedModules();
        printAll();
    }

    /**
     * Initializing list of all modules
     */
    private void initModuleList() {
        Module one_module;
        ModuleGroup parent_group;

        for (Iterator i = all_modules.iterator(); i.hasNext();) {
            one_module = (Module) i.next();
            parent_group = one_module.getParentGroup();
            if (parent_group.isStaging()) {
                staged_modules.add(one_module);
            } else {
                live_modules.add(one_module);
            }
        }
    }

    /**
     * Removes live modules if there was staged a new version
     */
    private void removeUpdatedModules() {
        Module one_live_module;
        Module one_staged_module;
        String live_module_name;
        String staged_module_name;

        for (Iterator i = live_modules.iterator(); i.hasNext();) {
            one_live_module = (Module) i.next();
            live_module_name = one_live_module.getName();

            for (Iterator j = staged_modules.iterator(); j.hasNext();) {
                one_staged_module = (Module) j.next();
                staged_module_name = one_staged_module.getName();

                if (live_module_name.compareTo(staged_module_name) == 0) {
                    one_live_module.setUpdated();
                }
            }
        }
    }

    /**
     * Recursivly Loads Map all_groups
     */
    private void addToAllModules(List levelGroups, List levelModules) {
        ModuleGroup one_group;

        for (Iterator i = levelGroups.listIterator(); i.hasNext();) {
            one_group = (ModuleGroup) i.next();
            addToAllModules(one_group.getModuleGroups(), one_group.getModules());
            all_groups.add(one_group);
        }
        all_modules.addAll(levelModules);
    }

    /**
     * For debug - prints list of ordered modules
     */
    private void printAll() {
        Module one_module;

        System.out.println("All modules:");

        for (Iterator i = all_modules.iterator(); i.hasNext();) {
            one_module = (Module) i.next();
            System.out.println("module ->" + one_module.getName());
        }

    }
}
    