/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.jar.*;
import java.util.List;
import java.util.ArrayList;

import java.util.Enumeration;

/** Creates 
 *
 * @author  rnovak
 * @version 1.0
 */
public class NBMfile {

    public static final String INFO_PATH = "Info/info.xml";
    public static final String INFO_LOCALE_PATH_BASE = "Info/locale/info_";
    private File nbm_file;
    public String language = "en";
    public String os = null;
    private JarFile jar_file;

    /**
     * Creates NBMfile object from NBM file
     */
    public NBMfile(File nbmFile) {
        this.nbm_file = nbmFile;

        setLanguage();


        try {
            jar_file = new JarFile(nbmFile);
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    /**
     * Returns input stream to info.xml file
     */
    public InputStream getInfoXMLStream() {
        InputStream inpStream = null;

        try {
            inpStream = jar_file.getInputStream(jar_file.getJarEntry(INFO_PATH));
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException e) {
            System.out.println(e);
        }
        return (inpStream);
    }

    /**
     * Returns list of supported languages
     */
    public List getSupportedLangs() {
        List supported_langs = new ArrayList();
        //String infoxml_file_base = "Info/locale/info_";
        Enumeration all_files = null;

        all_files = jar_file.entries();

        String one_file_name;
        while (all_files.hasMoreElements()) {
            JarEntry one_file = (JarEntry) all_files.nextElement();
            one_file_name = one_file.getName();

            if (one_file_name.startsWith(INFO_LOCALE_PATH_BASE)) {
                int code_begin = INFO_LOCALE_PATH_BASE.length();
                int code_end = one_file_name.indexOf(".xml");

                String lang_code = one_file_name.substring(code_begin, code_end);
                supported_langs.add(lang_code);
            }
        }

        return (supported_langs);
    }

    /**
     * Returns input stream to info_{lang_code}.xml file
     */
    public InputStream getInfoXMLStream(String langCode) {
        InputStream inpStream = null;
        String infoxml_file = INFO_LOCALE_PATH_BASE + langCode + ".xml";

        if (jar_file.getEntry(infoxml_file) == null) {
            return (null);
        }

        try {
            inpStream = jar_file.getInputStream(jar_file.getJarEntry(infoxml_file));
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException e) {
            System.out.println(e);
        }
        return (inpStream);
    }

    /**
     * Returns input stream to <module_name>.xml file
     */
    public InputStream getModuleConfigStream(String moduleCodeName) {
        InputStream inpStream = null;

        moduleCodeName = moduleCodeName.replace(".", "-");
        String config_file = "netbeans/config/Modules/" + moduleCodeName + ".xml";

        try {
            jar_file = new JarFile(nbm_file);
            JarEntry file_in_jar = jar_file.getJarEntry(config_file);
            if (file_in_jar != null) {
                inpStream = jar_file.getInputStream(jar_file.getJarEntry(config_file));
            }
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException e) {
            System.out.println(e);
        }
        return (inpStream);
    }

    /**
     * Returns size of NBM
     */
    public long getSize() {
        return (nbm_file.length());
    }

    /**
     * Returns name of NBM
     */
    public String getName() {
        return (nbm_file.getName());
    }

    /**
     * Closes NBM file
     */
    public void close() {
        try {
            jar_file.close();
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    /**
     * Sets NBM language - nbm name contains it
     */
    private void setLanguage() {
        String nbm_name = nbm_file.getName();

        int dash = nbm_name.lastIndexOf("_");
        int dot = nbm_name.lastIndexOf(".nbm");
        if (dash > 0 && dot > 0) {
            language = nbm_name.substring(dash + 1, dot);
        }
    }
}
    