/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml.web;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.io.IOException;
import org.netbeans.auxml.Module;
import org.springframework.web.servlet.ModelAndView;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.util.Map;
import org.netbeans.auxml.AuManager;
import org.netbeans.auxml.ModuleGroup;
import org.netbeans.auxml.UpdateCenter;
import org.netbeans.auxml.Version;
import org.netbeans.auxml.service.ModulesActionForm;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.view.RedirectView;
import static org.netbeans.auxml.web.Config.getManager;
import org.springframework.validation.BindException;


/**
 * 
 * @author mschovanek@netbeans.org
 */
public class ModulesController extends SimpleFormController {
    
    Map<String, List<Module>> staged;
    Map<String, List<Module>> published;

    protected final Log LOG = LogFactory.getLog(getClass());
    
    @Override
    public ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws ServletException, IOException {
        ModulesActionForm uc = (ModulesActionForm) command;
        
        getManager().applyChanges(uc.getVersion(), uc.getType(), request.getParameterMap());
                        
        ModelAndView model;
        model = new ModelAndView(new RedirectView(getSuccessView()));
        Map map = model.getModel();
        map.put("version", uc.getType());
        map.put("type", uc.getType());
        
        return model;
    }
    

    @Override
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, Exception {
        AuManager mgr = getManager();
        Version version;
        String verName = request.getParameter("version");
        if (verName != null && !verName.trim().equals("")) {
            version = mgr.getVersion(verName);
        } else {
            version = Collections.max(mgr.getVersions());
        }
        UpdateCenter uc;
        String type = request.getParameter("type");
        if (type == null || type.trim().equals("")) {
            type = "stable";
        }
        uc = version.getUpdateCenter(type);
        staged = new HashMap<String, List<Module>>();
        published = new HashMap<String, List<Module>>();
        Map<String, List<Module>> groupsMap;
        for (Module module : uc.getModules()) {
            if (module.isStaging()) {
                groupsMap = staged;
            } else {
                groupsMap = published;
            }
            String groupName = module.getGroupPath();
            List<Module> list;
            if (groupsMap.containsKey(groupName)) {
                list = groupsMap.get(groupName);
            } else {
                list = new ArrayList<Module>();
                groupsMap.put(groupName, list);
            }
            list.add(module);
        }
        ModelAndView model = super.handleRequest(request, response);
        Map map = model.getModel();
        map.put("version", uc.getVersion());
        map.put("type", uc.getType());
        map.put("groups", staged);
        map.put("stagedGroups", staged);
        map.put("publishedGroups", published);
        return model;
    }
    
    @Override
    protected Object formBackingObject(HttpServletRequest request) throws ServletException {
        ModulesActionForm actions = new ModulesActionForm(request.getParameter("type"),request.getParameter("version"));
        return actions;
    }
    
}