 /*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml.web;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.netbeans.auxml.UpdateCenter;
import org.netbeans.auxml.Version;
import org.netbeans.auxml.service.SelectUCForm;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.CancellableFormController;
import org.springframework.web.servlet.view.RedirectView;
import static org.netbeans.auxml.web.Config.getManager;

/**
 *
 * @author mschovanek@netbeans.org
 */
public class SelectUCController extends CancellableFormController {
    
    private String sel_version = null;
    private String reloadForm = null;

    /** Logger for this class and subclasses */
    protected final Log LOG = LogFactory.getLog(getClass());
    
    public String getSel_version() {
        return(sel_version);
    }

    public void setSel_version(String sel_version) {
        this.sel_version = sel_version;
    }
    
    public String getReloadForm() {
        return(reloadForm);
    }

    public void setReloadForm(String reloadForm) {
        this.reloadForm = reloadForm;
    }
    
    @Override
    public ModelAndView onSubmit(Object command) throws ServletException, IOException {
        SelectUCForm uc = (SelectUCForm) command;
        ModelAndView model;
        
        sel_version = uc.getVersion();
        String type = uc.getType();
        if ( type.equals("type") ) {
//            model = new ModelAndView(new RedirectView("selectuc.htm"));
            model = new ModelAndView(new RedirectView(getReloadForm()));
        }
        else {
            model = new ModelAndView(new RedirectView(getSuccessView()));
            Map map = model.getModel();
            map.put("version", sel_version);
            map.put("type", type);
        }

        return (model);
    }
    
    @Override
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, Exception {
        ModelAndView model = super.handleRequest(request, response);
        Map map = model.getModel();
        
        List<Version> versions = getManager().getVersions();
        List<String> verNames = new ArrayList<String>(versions.size());
        List<String> typeSet = new ArrayList<String>();
        
        Version sel_one_version = versions.get(0);
        String one_verName;
        for (Version one_version : versions) {
            one_verName = one_version.getName();
            verNames.add(one_verName);
            if (one_verName.equals(sel_version))
                sel_one_version = one_version;
        }
        
        for (UpdateCenter uc : sel_one_version.getUpdateCenters()) {
                typeSet.add(uc.getType());
        }
        
        map.put("versions", verNames);
        map.put("types", typeSet);        
        
        return model;
     }

    @Override
    protected Object formBackingObject(HttpServletRequest request) throws ServletException {
        SelectUCForm uc = new SelectUCForm(sel_version,"type");
        return uc;
    }
    
}