
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage.annotations;

import org.netbeans.modules.coverage.utils.LineCoverageStates;

/**
 * This class uses enum to select annotation type.
 * Three different objects (CoverageAnnotationComplete, CoverageAnnotationPartial, CoverageAnnotationZero) might be used instead.
 *<b>
 * To create this object use:<br>
 * AbstractCoverageAnnotation annotation = new CoverageAnnotation( LineCoverageStates enum type );
 * 
 * @author Andrey Korostelev
 */
public class CoverageAnnotation extends AbstractCoverageAnnotation {
    
    
    //private static List annotations = new ArrayList();
    private LineCoverageStates state;

    /**
     * Creates coverage annotation
     * @param state Stat of the annotated line (covered/partial)
     * @param projectName Project the annotation belongs to
     * @param className Class the annotation belongs to
     * @param lineNum Number of line to be annotated
     */
    public CoverageAnnotation(LineCoverageStates state, String projectName, String className, Integer lineNum){
        super(projectName, className, lineNum);
        this.state = state;
    }
   

    /**
     * Getter for the annotation's state type
     * @return Annotation type
     */
    public String getAnnotationType() {
        return state.getType();
    }

    /**
     * Returns annotation short description: this is state's description + annotation
     *message
     * @return Short description
     */
    public String getShortDescription() {
        if (getMessage() != null)
            return state.getDescription()+". "+getMessage();
        else
            return state.getDescription();
    }
    
}
