/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.data.local;

import java.util.Hashtable;
import org.netbeans.modules.coverage.data.*;
import org.netbeans.modules.coverage.utils.CountRate;
import org.netbeans.modules.coverage.utils.LineCoverageStates;

/**
 * Class coverage data implementation to handle emma coverage data files
 * @author mikk@netbeans.org
 */
public class ClassCoverageData implements IClassCoverageData{
    private static final double CLASS_COVERED_LIMIT = 0.5;
    
    private String className;
    private String packageName;
    private int totalLines;
    private Hashtable<Integer, LineCoverageStates> lineCoverageHash = new Hashtable<Integer, LineCoverageStates>();

    public ClassCoverageData(String className, String packageName) {
        this.className = className;
        this.packageName = packageName;
    }
    
    /**
     * 
     */
    public Hashtable<Integer, LineCoverageStates> getCoveredLines(int firstLineNum, int lastLineNum) {
        Hashtable<Integer, LineCoverageStates> lines = new Hashtable<Integer, LineCoverageStates>();
        for(int i=firstLineNum; i<lastLineNum+1;i++){
            LineCoverageStates covVal = lineCoverageHash.get(i);
            if (covVal!=null){
                lines.put(i, covVal);
            }
        }
        return lines;
    }
    
    public Hashtable<Integer, LineCoverageStates> getAllCoveredLines() {
        return lineCoverageHash;
    }
    
    public LineCoverageStates getLineCoverage(int lineNum) {
        LineCoverageStates covVal = lineCoverageHash.get(lineNum);
        if (covVal!=null){
            return covVal;
        }else{
            return LineCoverageStates.LINE_COVERAGE_ZERO;
        }
    }
    
    /**
     * Used by report generator to add coverage info to the object
     * @param lineNum Line number to add info for
     * @param state Line coverage state
     */
    void putLineCoverage(int lineNum, LineCoverageStates state){
        lineCoverageHash.put(lineNum, state);
    }
    
    public void setTotalLines(int totalLines) {
        this.totalLines = totalLines;
    }

    public String getName() {
        return className;
    }

    public CountRate getOverallLineCoverage() {
        return new CountRate(totalLines, lineCoverageHash.size());
    }

    public boolean isCovered() {
        return getOverallLineCoverage().getRate() >= CLASS_COVERED_LIMIT;
    }
    
    
}
