/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage.data.local;

import java.util.HashMap;
import java.util.Map;
import org.netbeans.modules.coverage.data.IPackageCoverageData;
import org.netbeans.modules.coverage.utils.CountRate;

/**
 *
 * @author Lyadzhin
 */
public class PackageCoverageData implements IPackageCoverageData {
    private static final double CLASS_COVERAGE_LIMIT = 0.5;
    
    private final String packageName;
    private final Map<String, ClassCoverageData> classCoverageMap = new HashMap<String, ClassCoverageData>();

    public PackageCoverageData(String packageName) {
        this.packageName = packageName;
    }

    public String getName() {
        return packageName;
    }

    public CountRate getOverallClassCoverage() {
        int totalCovered = 0;
        for (ClassCoverageData data : classCoverageMap.values()) {
            if (data.isCovered()) {
                totalCovered++;
            }
        }
        return new CountRate(classCoverageMap.size(), totalCovered);
    }

    public CountRate getOverallLineCoverage() {
        int totalLines = 0;
        int totalCovered = 0;
        for (ClassCoverageData classCoverageData : classCoverageMap.values()) {
            final CountRate lineCoverage = classCoverageData.getOverallLineCoverage();
            totalLines += lineCoverage.getTotalCount();
            totalCovered += lineCoverage.getSelectedCount();
        }
        return new CountRate(totalLines, totalCovered);
    }

    public boolean isCovered() {
        return getOverallClassCoverage().getRate() >= CLASS_COVERAGE_LIMIT;
    }
    
    void putClassCoverage(String className, ClassCoverageData coverageData) {
        classCoverageMap.put(className, coverageData);
    }
}
