/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.instrumentation;

import org.openide.filesystems.FileObject;

/**
 *
 * @author apepin
 */
public class XmlBuilder extends TextFileBuilder{
    
    /** Creates a new instance of XmlBuilder */
    public XmlBuilder(FileObject file) {
        super(file);
        load();
    }
    
    public void addTarget(String targetContent){
        //find line to insert
        //let it be the last line
        int line = findStringStartsWith("</project>");
        //System.out.println("text will be added at line "+line);
        appendText(targetContent, line);
    }
    
    public void addTargetsFromFile(FileObject fo){
        int line = findStringStartsWith("</project>");
        //System.out.println("text will be added at line "+line);
        appendText(new XmlBuilder(fo), line);
    }
    
    public void removeTarget(String targetName){
        //find starting target line
        String startTag = "<target name=\""+targetName+"\"";
        int start = findStringStartsWith(startTag);
        if(start>=0){
            //find ending target line 
            int end = start;
            String text = getLine(start);
            int beginIndex = text.indexOf(startTag);
            if(text.indexOf("/>", beginIndex) < 0)
                end = findLineStartsWith("</target>", start);
            if(end>=0){
                //remove target lines
                for(int line=end; line>=start; line--)
                    removeLine(line);
            }
        }
    }
    
    public void removeTargets(String... targets){
        for(String target: targets)
            removeTarget(target);
    }

    public String getElementText(String elName) {
        //simple implementation needs improving
        String text = null;
        if(elName != null){
            String startTag = "<"+elName;
            int start = findStringStartsWith(startTag);
            if(start >= 0){
                String element = getLine(start);
                int index = element.indexOf(startTag);
                int textStart = element.indexOf(">", index)+1;
                if(textStart > 0){
                    int textEnd = element.indexOf("</"+elName+">", textStart);
                    if(textEnd > 0)
                        text = element.substring(textStart, textEnd);
                }
            }
        }
        //System.out.println("element "+elName+" text is: "+text);
        return text;
    }
    
}
