/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.utils;

/**
 * Enum that represents possible line coverage states
 * @author  Andrey Korostelev
 */
public enum LineCoverageStates {
    /**
     * Line not covered
     */
    LINE_COVERAGE_ZERO(0),
    /**
     * Line is partially covered
     */
    LINE_COVERAGE_PARTIAL(1),
    /**
     * Line is covered completely
     */
    LINE_COVERAGE_COMPLETE(2);
    
    private int digitalValue;
    private LineCoverageStates(int newDigitalValue){
        digitalValue = newDigitalValue;
    }
    
    /**
     * Checks if current coverage state is more than suggested 'toCheck'
     * @param toCheck State to compare with
     * @return Comparison result
     */
    public boolean isMoreThan(LineCoverageStates toCheck){
        if (toCheck == null) 
            return true;
        switch (this){
            case LINE_COVERAGE_ZERO: return false;
            case LINE_COVERAGE_PARTIAL: return toCheck.equals(LINE_COVERAGE_ZERO);
            case LINE_COVERAGE_COMPLETE: return true;
            default : return false;
        }
    }
    
    /**
     * Returns state textual description
     * @return State description
     */
    public String getDescription(){
        switch (this){
            case LINE_COVERAGE_ZERO: return "Zero coverage";
            case LINE_COVERAGE_PARTIAL: return "Partial coverage";
            case LINE_COVERAGE_COMPLETE: return "Complete coverage";
            default : return "unknown info about coverage";
        }
            
    }

    /**
     * Returns state textual representation. Each state has  some String constant associated
     * @return Textual representation
     */
    public String getType(){
        switch (this){
            case LINE_COVERAGE_ZERO: return "annotation_zero";
            case LINE_COVERAGE_PARTIAL: return "annotation_partial";
            case LINE_COVERAGE_COMPLETE: return "annotation_complete";
            default : return "annotation_unknown";
        }
            
    }
}
