
package org.netbeans.jsstubsgenerator;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Logger;
import org.mozilla.javascript.CompilerEnvirons;
import org.mozilla.javascript.ErrorReporter;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.ScriptOrFnNode;

/**
 *
 * @author tor, Martin Fousek <marfous@netbeans.org>
 */
public final class Parser {

    private static final Logger LOG = Logger.getLogger(Parser.class.getName());
    private AtomicBoolean wasSourceOk = new AtomicBoolean(true);

    public boolean wasSourceOk() {
        return wasSourceOk.get();
    }

    public void parse(String buffer, String filename, int lineno) {
        CompilerEnvirons compilerEnv = new CompilerEnvirons();
        ErrorReporter errorReporter =
            new ErrorReporter() {
                @Override
                public void error(String message, String sourceName, int line,
                           String lineSource, int lineOffset) {
                    LOG.severe("ERROR: " + message + ", sourceName=" + sourceName + ", line=" + line +", lineSource=" + lineSource);
                    wasSourceOk.set(false);
                }

                @Override
                public EvaluatorException runtimeError(String message, String sourceName,
                                                int line, String lineSource,
                                                int lineOffset) {
                    LOG.severe("ERROR: " + message + ", sourceName=" + sourceName + ", line=" + line +", lineSource=" + lineSource);
                    wasSourceOk.set(false);
                    return null;
                }

                @Override
                public void warning(String message, String sourceName, int line,
                             String lineSource, int lineOffset) {
                    LOG.warning("WARNING: " + message + ", sourceName=" + sourceName + ", line=" + line +", lineSource=" + lineSource);
                }
            };

        // XXX What do I set here: compilerEnv.setReservedKeywordAsIdentifier();

        org.mozilla.javascript.Context ctx = new org.mozilla.javascript.Context();
        compilerEnv.initFromContext(ctx);

        compilerEnv.setErrorReporter(errorReporter);
        compilerEnv.setGeneratingSource(false);
        compilerEnv.setGenerateDebugInfo(false);
        // We have a quickfix which lets you turn these warnings off or turn them
        // to errors
        compilerEnv.setReservedKeywordAsIdentifier(true);

        // The parser is NOT used for parsing here, but the Rhino scanner
        // calls into the parser for error messages. So we register our own error
        // handler for the parser and pass it into the tokenizer to handle errors.
        org.mozilla.javascript.Parser parser;
        parser = new org.mozilla.javascript.Parser(compilerEnv, errorReporter);

//        String fileName = "";
//
//        if ((context.file != null) && (context.file.getFileObject() != null)) {
//            fileName = context.file.getFileObject().getNameExt();
//        }

        try {
            ScriptOrFnNode root = parser.parse(buffer, filename, lineno);
        } catch (RuntimeException re) {
            //notifyError(context, message, sourceName, line, lineSource, lineOffset, sanitizing, Severity.WARNING, "", null);
            // XXX TODO - record this somehow
            wasSourceOk.set(false);
        }
    }
}
