
package org.netbeans.jsstubsgenerator.core;

import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.Return;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 * Generates properties and functions which aren't mentioned in the specs, but are commonly used in browsers.
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class NoSpecCoreGenerator {

    public static Map<String, StubObject> updateStubs(Map<String, StubObject> allStubs) {
        updateRegExpObject(allStubs.get("RegExp"));

        return allStubs;
    }

    public static void updateRegExpObject(StubObject so) {
        assert so != null;

        // issue #215923 - Missing "compile()" in completion for RegExp
        if (so.getAttribute("compile") == null) {
            StubObject compile = new StubObject(Origin.CORE, "compile", so.getName());
            compile.setPrune(true);
            compile.setReturns(new Return("RegExp", ""));
            compile.setSummary("The compile() method is used to compile a regular expression during execution of a script. The compile() method can also be used to change and recompile a regular expression.");
            compile.setSyntax("regExp.compile(regexp, modifier)");
            compile.setType(Type.METHOD);
            compile.addParameter(new Parameter("regexp", "RegExp", "A regular expression"));
            compile.addParameter(new Parameter("modifier", "String", "Specifies the type of matching. \"g\" for a global match, \"i\" for a case-insensitive match and \"gi\" for a global, case-insensitive match"));
            so.putAttribute(compile);
        }
    }
}
