/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class ConsoleInstanceGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject consoleInstance = new StubObject(Origin.DOM, "Console", StubObject.GLOBAL);
        consoleInstance.setPrune(true);
        consoleInstance.setType(Type.OBJECT);
        consoleInstance.setSummary("The console object provides access to the browser's debugging console. The specifics of how it works vary from browser to browser, but there is a de facto set of features that are typically provided.");

        // methods definition
        String[][] methods = new String[][]{
            {"dir", "undefined", "Displays an interactive list of the properties of the specified JavaScript object. The output is presented as a hierarchical listing with disclosure triangles that let you see the contents of child objects."},
            {"error", "undefined", "Outputs an error message to the Web Console."},
            {"group", "undefined", "Creates a new inline group in the Web Console log. This indents all following output by an additional level, until console.groupEnd() is called."},
            {"groupCollapsed", "undefined", "Creates a new inline group in the Web Console. Unlike console.group() , however, the new group is created collapsed. The user will need to use the disclosure button next to it to expand it, revealing the entries created in the group. Call console.groupEnd() to back out to the parent group."},
            {"groupEnd", "undefined", "Exits the current inline group in the Web Console. See Using groups in the console in the console documentation for details and examples."},
            {"info", "undefined", "Outputs an informational message to the Web Console. In Firefox, a small \"i\" icon is displayed next to these items in the Web Console's log."},
            {"log", "undefined", "Outputs a message to the Web Console."},
            {"time", "undefined", "Starts a timer you can use to track how long an operation takes. You give each timer a unique name, and may have up to 10,000 timers running on a given page. When you call console.timeEnd() with the same name, the browser will output the time, in milliseconds, that elapsed since the timer was started."},
            {"timeEnd", "undefined", "Stops a timer that was previously started by calling console.time() ."},
            {"trace", "undefined", "Outputs a stack trace to the Web Console."},
            {"warn", "undefined", "Outputs a warning message to the Web Console."},
        };
        List<StubObject> generatedMethods = generateMethods(methods);
        for (StubObject instanceMethod : generatedMethods) {
            consoleInstance.putAttribute(instanceMethod);
        }

        allStubs.put("Console", consoleInstance);
    }

    private static List<StubObject> generateMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "Console", true);
        for (StubObject method : methodStubs) {
            if ("dir".equals(method.getName())) {
                method.setSyntax("console.dir(object);");
                method.addParameter(new Parameter("object", "Object", "A JavaScript object whose properties should be output."));
            } else if ("error".equals(method.getName())) {
                method.setSyntax("console.error(obj1 [, obj2, ..., objN);<br>console.error(msg [, subst1, ..., substN);");
                method.addParameter(new Parameter("obj1 ... objN", "Object", "A list of JavaScript objects to output. The string representations of each of these objects are appended together in the order listed and output."));
                method.addParameter(new Parameter("msg", "String", "A JavaScript string containing zero or more substitution strings."));
                method.addParameter(new Parameter("subst1 ... substN", "String", "JavaScript objects with which to replace substitution strings within msg. This gives you additional control over the format of the output."));
            } else if ("info".equals(method.getName())) {
                method.setSyntax("console.info(obj1 [, obj2, ..., objN);<br>console.info(msg [, subst1, ..., substN);");
                method.addParameter(new Parameter("obj1 ... objN", "Object", "A list of JavaScript objects to output. The string representations of each of these objects are appended together in the order listed and output."));
                method.addParameter(new Parameter("msg", "String", "A JavaScript string containing zero or more substitution strings."));
                method.addParameter(new Parameter("subst1 ... substN", "String", "JavaScript objects with which to replace substitution strings within msg. This gives you additional control over the format of the output."));
            } else if ("log".equals(method.getName())) {
                method.setSyntax("console.log(obj1 [, obj2, ..., objN);<br>console.log(msg [, subst1, ..., substN);");
                method.addParameter(new Parameter("obj1 ... objN", "Object", "A list of JavaScript objects to output. The string representations of each of these objects are appended together in the order listed and output."));
                method.addParameter(new Parameter("msg", "String", "A JavaScript string containing zero or more substitution strings."));
                method.addParameter(new Parameter("subst1 ... substN", "String", "JavaScript objects with which to replace substitution strings within msg. This gives you additional control over the format of the output."));
            } else if ("time".equals(method.getName())) {
                method.setSyntax("console.time(timerName);");
                method.addParameter(new Parameter("timerName", "String", "The name to give the new timer. This will identify the timer; use the same name when calling console.timeEnd() to stop the timer and get the time output to the console."));
            } else if ("timeEnd".equals(method.getName())) {
                method.setSyntax("console.timeEnd(timerName);");
                method.addParameter(new Parameter("timerName", "String", "The name of the timer to stop. Once stopped, the elapsed time is automatically displayed in the Web Console."));
            } else if ("warn".equals(method.getName())) {
                method.setSyntax("console.warn(obj1 [, obj2, ..., objN);<br>console.warn(msg [, subst1, ..., substN);");
                method.addParameter(new Parameter("obj1 ... objN", "Object", "A list of JavaScript objects to output. The string representations of each of these objects are appended together in the order listed and output."));
                method.addParameter(new Parameter("msg", "String", "A JavaScript string containing zero or more substitution strings."));
                method.addParameter(new Parameter("subst1 ... substN", "String", "JavaScript objects with which to replace substitution strings within msg. This gives you additional control over the format of the output."));
            } else {
                method.setSyntax("console." + method.getName() + "()");
            }
        }
        return methodStubs;
    }

}
