/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class WindowGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject windowInterface = new StubObject(Origin.DOM, "Window", StubObject.GLOBAL);
        windowInterface.setPrune(true);
        windowInterface.setType(Type.OBJECT);
        windowInterface.setSuperClass("AbstractView");
        windowInterface.setSummary("This specification defines four features of the window object. It describes the use of the Window object as a browsing context: a container that displays a document, or a series of documents in sequence. It describes interfaces for navigation, the ability to go to a new document in the same browsing context. It describes interfaces for embedding, a feature also known as compound document by reference, where one document includes another separate document by external reference. And it specifies methods for using timers.");

        // attributes definition
        String[][] attributes = new String[][]{
            {"window", "Window", "The value of the window attribute MUST be the same Window object that has the attribute: the attribute is a self-reference."},
            {"self", "Window", "The value of the self attribute MUST be the same Window object that has the attribute: the attribute is a self-reference. Consequently, the value of the self attribute MUST be the same object as the window attribute."},
            {"location", "Location", "The value of the location attribute MUST be the Location object that represents the window's current location."},
            {"name", "DOMString", "An attribute containing a unique name used to refer to this Window object. "},
            {"parent", "Window", "An attribute containing a reference to Window object that contains this object."},
            {"top", "Window", "An attribute containing a reference to the topmost Window object in the hierarchy that contains this object."},
            {"frameElement", "Element", ""}
        };
        List<StubObject> generatedInstanceProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "Window", false);
        for (StubObject instanceProperty : generatedInstanceProperties) {
            windowInterface.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"setTimeout", "Number", "This method calls the function once after a specified number of milliseconds elapses, until canceled by a call to clearTimeout. The methods returns a timerID which may be used in a subsequent call to clearTimeout to cancel the interval."},
            {"setInterval", "Number", "This method calls the function every time a specified number of milliseconds elapses, until canceled by a call to clearInterval. The methods returns an intervalID which may be used in a subsequent call to clearInterval to cancel the interval."},
            {"clearTimeout", "undefined", "Cancels a timeout that was set with the setTimeout method."},
            {"clearInterval", "undefined", "Cancels an interval that was set with the setTimeout method."},
        };
        List<StubObject> generatedInstanceMethods = generateInstanceMethods(methods);
        for (StubObject instanceMethod : generatedInstanceMethods) {
            windowInterface.putAttribute(instanceMethod);
        }

        allStubs.put("Window", windowInterface);
    }

    private static List<StubObject> generateInstanceMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "Window", false);
        for (StubObject method : methodStubs) {
            if ("setTimeout".equals(method.getName()) || "setInterval".equals(method.getName())) {
                method.addParameter(new Parameter("listener", "Function", "Function to be called."));
                method.addParameter(new Parameter("milliseconds", "Number", "interval"));
            } else if ("clearTimeout".equals(method.getName())) {
                method.addParameter(new Parameter("timerID", "Number", "Identified of the timer."));
            } else if ("clearInterval".equals(method.getName())) {
                method.addParameter(new Parameter("intervalID", "Number", "Identified of the interval."));
            }
        }
        return methodStubs;
    }

}
