/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.Return;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class WindowInstanceGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject windowObject = new StubObject(Origin.DOM, "window", StubObject.GLOBAL);
        windowObject.setPrune(true);
        windowObject.setType(Type.OBJECT);
        windowObject.setSuperClass("Window");
        windowObject.setInstance(true);
        windowObject.setSummary("The window object represents the window itself. The document property of a window points to the DOM document loaded in that window. A window for a given document can be obtained using the document.defaultView property.");

        // properties definition
        String[][] attributes = new String[][]{
            {"applicationCache", "Object", "Returns a reference to the application cache object for the window."},
            {"closed", "Boolean", "This property indicates whether the referenced window is closed or not. "},
            {"Components", "Components", "The entry point to many XPCOM features. Some properties, e.g. classes, are only available to sufficiently privileged code."},
            {"console", "Console", "The console object provides access to the browser's debugging console. The specifics of how it works vary from browser to browser, but there is a de facto set of features that are typically provided."},
            {"content", "Window", "Returns a Window object for the primary content window. This is useful in XUL windows that have a <browser> (or tabbrowser or <iframe>) with type=\"content-primary\" attribute on it - the most famous example is Firefox main window, browser.xul. In such cases, content returns a reference to the Window object for the document currently displayed in the browser. It is a shortcut for browserRef.contentWindow. In unprivileged content (webpages), content is normally equivalent to top (except in the case of a webpage loaded in a sidebar, content still refers to the Window of the currently selected tab)."},
            {"controllers", "XULControllers", "Returns the XUL controllers of the chrome window."},
            {"crypto", "Crypto", "The crypto API allows web pages access to certain cryptographic related services. These services are a balance between the functionality web pages need, and the requirement to protect users from malicious web sites."},
            {"defaultStatus", "String", "Gets/sets the status bar text for the given window."},
            {"dialogArguments", "Object", "Property dialogArguments returns parameters of method showModalDialog()."},
            {"document", "document", "Returns a reference to the document contained in the window."},
            {"frames", "Array", "Returns the window itself, which is an array-like object, listing the direct sub-frames of the current window."},
            {"fullScreen", "Boolean", "This property indicates whether the window is displayed in full screen mode or not. It is only reliable in Gecko 1.9 (Firefox 3) and later, see the Notes below. "},
            {"history", "History", "Returns a reference to the History object, which provides an interface for manipulating the browser session history (pages visited in the tab or frame that the current page is loaded in)."},
            {"innerHeight", "Number", "Height (in pixels) of the browser window viewport including, if rendered, the horizontal scrollbar."},
            {"innerWidth", "Number", "Width (in pixels) of the browser window viewport including, if rendered, the vertical scrollbar."},
            {"length", "Number", "Returns the number of frames (either frame or iframe elements) in the window. "},
            {"locationbar", "Object", "Returns the locationbar object, whose visibility can be checked."},
            {"localStorage", "Storage", "localStorage is the same as sessionStorage with same same-origin rules applied but it is persistent."},
            {"menubar", "Object", "Returns the menubar object, whose visibility can be checked."},
            {"navigator", "Navigator", "Returns a reference to the navigator object, which can be queried for information about the application running the script."},
            {"opener", "Window", "Returns a reference to the window that opened this current window."},
            {"outerHeight", "Number", "window.outerHeight gets the height in pixels of the whole browser window. It represents the height of the whole browser window including sidebar (if expanded), window chrome and window resizing borders/handles."},
            {"outerWidth", "Number", "window.outerWidth gets the width of the outside of the browser window. It represents the width of the whole browser window including sidebar (if expanded), window chrome and window resizing borders/handles."},
            {"pageXOffset", "Number", "Returns the number of pixels that the document has already been scrolled horizontally."},
            {"pageYOffset", "Number", "Returns the number of pixels that the document has already been scrolled horizontally."},
            {"performance", "Object", "Provides a hosting area for performance related attributes."},
            {"personalbar", "Object", "Returns the personalbar object, whose visibility can be toggled in the window."},
            {"screen", "Screen", "Returns a reference to the screen object associated with the window."},
            {"screenLeft", "Number", "Returns the horizontal coordinate of the window relative to the screen."},
            {"screenTop", "Number", "Returns the vertical coordinate of the window relative to the screen."},
            {"screenX", "Number", "Returns the horizontal coordinate of the window relative to the screen."},
            {"screenY", "Number", "Returns the vertical coordinate of the window relative to the screen."},
            {"scrollbars", "Object", "Returns the scrollbars object, whose visibility can be toggled in the window."},
            {"scrollMaxX", "Number", "Returns the maximum number of pixels that the document can be scrolled horizontally."},
            {"scrollMaxY", "Number", "Returns the maximum number of pixels that the document can be scrolled vertically."},
            {"scrollX", "Number", "Returns the number of pixels that the document has already been scrolled horizontally."},
            {"scrollY", "Number", "Returns the number of pixels that the document has already been scrolled horizontally."},
            {"sessionStorage", "Storage", "Gets a global object (sessionStorage) that maintains a storage area that's available for the duration of the page session. A page session lasts for as long as the browser is open and survives over page reloads and restores. Opening a page in a new tab or window will cause a new session to be initiated."},
            {"sidebar", "Object", "Returns a reference to the window object of the sidebar."},
            {"status", "String", "Sets the text in the status bar at the bottom of the browser or returns the previously set text. This property does not work in default configuration of Firefox and some other browsers: setting window.status has no effect on the text displayed in the status bar. To allow scripts change the the status bar text, the user must set the dom.disable_window_status_change preference to false in the about:config screen."},
            {"statusbar", "Object", "Returns the statusbar object, whose visibility can be checked."},
            {"toolbar", "Object", "Returns the toolbar object, whose visibility can be toggled in the window."},
        };
        // events definition
        String[][] events = new String[][]{
            {"onabort", "Function", "An event handler property for abort events on the window."},
            {"onbeforeunload", "Function", "An event handler property for before-unload events on the window."},
            {"onblur", "Function", "An event handler property for blur events on the window."},
            {"onchange", "Function", "An event handler property for change events on the window."},
            {"onclick", "Function", "An event handler property for click events on the window."},
            {"onclose", "Function", "An event handler property for handling the window close event."},
            {"oncontextmenu", "Function", "An event handler property for right-click events on the window."},
            {"ondragdrop", "Function", "An event handler property for drag and drop events on the window."},
            {"onerror", "Function", "An event handler property for errors raised on the window."},
            {"onfocus", "Function", "An event handler property for focus events on the window."},
            {"onhashchange", "Function", "An event handler property for hash change events on the window; called when the part of the URL after the hash mark (\"#\") changes."},
            {"onkeydown", "Function", "An event handler property for keydown events on the window."},
            {"onkeypress", "Function", "An event handler property for keypress events on the window."},
            {"onkeyup", "Function", "An event handler property for keyup events on the window."},
            {"onload", "Function", "An event handler property for window loading."},
            {"onmousedown", "Function", "An event handler property for mousedown events on the window."},
            {"onmousemove", "Function", "An event handler property for mousemove events on the window."},
            {"onmouseout", "Function", "An event handler property for mouseout events on the window."},
            {"onmouseover", "Function", "An event handler property for mouseover events on the window."},
            {"onmouseup", "Function", "An event handler property for mouseup events on the window."},
            {"onpaint", "Function", "An event handler property for paint events on the window."},
            {"onpopstate", "Function", "An event handler property for popstate events, which are fired when navigating to a session history entry representing a state object."},
            {"onreset", "Function", "An event handler property for reset events on the window."},
            {"onresize", "Function", "An event handler property for window resizing."},
            {"onscroll", "Function", "An event handler property for window scrolling."},
            {"onselect", "Function", "An event handler property for window selection."},
            {"onsubmit", "Function", "An event handler property for submits on window forms."},
            {"onunload", "Function", "An event handler property for unload events on the window."},
            {"onpageshow", "Function", "An event handler property for pageshow events on the window."},
            {"onpagehide", "Function", "An event handler property for pagehide events on the window."}
        };
        List<StubObject> generatedProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "window", true);
        generatedProperties.addAll(GeneratorUtils.generateProperties(Origin.DOM, events, "window", true));
        for (StubObject instanceProperty : generatedProperties) {
            windowObject.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"addEventListener", "undefined", "addEventListener() registers a single event listener on a single target. The event target may be a single element in a document, the document itself, a window, or an XMLHttpRequest. To register more than one event listener for the target, call addEventListener() for the same target but with different event types or capture parameters."},
            {"alert", "undefined", "Display an alert dialog with the specified content and an OK button."},
            {"atob", "String", "Decodes a string of data which has been encoded using base-64 encoding. For use with Unicode or UTF-8 strings, see this note at window.btoa."},
            {"back", "undefined", "Returns the window to the previous item in the history. "},
            {"blur", "undefined", "Shifts focus away from the window. "},
            {"btoa", "undefined", "Creates a base-64 encoded ASCII string from a string of binary data. Please note that this is not suitable for raw Unicode strings! See Unicode section below."},
            {"clearImmediate", "undefined", "This method clears the action specified by window.setImmediate ."},
            {"clearInterval", "undefined", "Cancels repeated action which was set up using setInterval()."},
            {"clearTimeout", "undefined", "Clears the delay set by window.setTimeout()."},
            {"close", "undefined", "Closes the current window, or a referenced window."},
            {"confirm", "Boolean", "Displays a modal dialog with a message and two buttons, OK and Cancel."},
            {"dump", "undefined", "Prints messages to the (native) console."},
            {"escape", "String", "Encodes a string, replacing certain characters with a hexadecimal escape sequence."},
            {"find", "Boolean", "Finds a string in a window."},
            {"focus", "undefined", "Makes a request to bring the window to the front. It may fail due to user settings and the window isn't guaranteed to be frontmost before this method returns."},
            {"forward", "undefined", "Moves the window one document forward in the history. "},
            {"getAttention", "undefined", "Attempts to get the user's attention. How this happens varies based on OS and window manager. "},
            {"getComputedStyle", "CSSStyleDeclaration", "getComputedStyle() gives the final used values of all the CSS properties of an element."},
            {"getSelection", "Selection", "Returns a selection object representing the range of text selected by the user. "},
            {"matchMedia", "MediaQueryList", "Returns a new MediaQueryList object representing the parsed results of the specified media query string."},
            {"moveBy", "undefined", "Moves the current window by a specified amount."},
            {"moveTo", "undefined", "Moves the window to the specified coordinates."},
            {"open", "Window", "Creates a new secondary browser window and loads the referenced resource."},
            {"openDialog", "Window", "window.openDialog is an extension to window.open. It behaves the same, except that it can optionally take one or more parameters past windowFeatures, and windowFeatures itself is treated a little differently. <br>The optional parameters, if present, will be bundled up in a JavaScript Array object and added to the newly created window as a property named window.arguments. They may be referenced in the JavaScript of the window at any time, including during the execution of a load handler. These parameters may be used, then, to pass arguments to and from the dialog window. Note that the call to openDialog() returns immediately. If you want the call to block until the user has closed the dialog, supply modal as a windowFeatures parameter. Note that this also means the user won't be able to interact with the opener window until he closes the modal dialog."},
            {"postMessage", "undefined", "window.postMessage, when called, causes a MessageEvent to be dispatched at the target window when any pending script that must be executed completes (e.g. remaining event handlers if window.postMessage is called from an event handler, previously-set pending timeouts, etc.). The MessageEvent has the type message, a data property which is set to the value of the first argument provided to window.postMessage, an origin property corresponding to the origin of the main document in the window calling window.postMessage at the time window.postMessage was called, and a source property which is the window from which window.postMessage is called. (Other standard properties of events are present with their expected values.)"},
            {"print", "undefined", "Opens the Print Dialog to print the current document."},
            {"prompt", "String|null", "Displays a dialog with a message prompting the user to input some text."},
            {"removeEventListener", "undefined", "removeEventListener() allows the removal of event listeners from the event target"},
            {"resizeBy", "undefined", "Resizes the current window by a certain amount."},
            {"resizeTo", "undefined", "Dynamically resizes window."},
            {"scroll", "undefined", "Scrolls the window to a particular place in the document."},
            {"scrollBy", "undefined", "Scrolls the document in the window by the given amount."},
            {"scrollByLines", "undefined", "Scrolls the document by the given number of lines. "},
            {"scrollByPages", "undefined", "Scrolls the current document by the specified number of pages. "},
            {"scrollTo", "undefined", "Scrolls to a particular set of coordinates in the document. "},
            {"setCursor", "undefined", "Changes the cursor for the current window."},
            {"setImmediate", "Number", "This method is used to break-up long running operations and run a callback function immediately after the browser has completed other operations such as events and display updates. "},
            {"setInterval", "Number", "Calls a function repeatedly, with a fixed time delay between each call to that function."},
            {"setTimeout", "Number", "Executes a code snippet or a function after specified delay."},
            {"showModalDialog", "Number", "Creates and displays a modal dialog box containing a specified HTML document."},
            {"sizeToContent", "undefined", "Sizes the window according to its content. The DOM content should be loaded when this function is called, for example after the DOMContentLoaded event."},
            {"stop", "undefined", "This method stops window loading."},
            {"unescape", "String", "Decodes a value that has been encoded in hexadecimal (e.g., a cookie)."},
            {"updateCommands", "undefined", "Updates the state of commands of the current chrome window (UI)."},
            {"Image", "HTMLImageElement", "Used for creating an object representing an HTML img. Implements HTMLImageElement. The constructor accepts two optional parameters: width and height."},
            {"Option", "HTMLOptionElement", "Used for creating an object representing an HTML option. Implements HTMLOptionElement."},
            {"Worker", "Worker", "Used for creating a Web worker"}
        };
        List<StubObject> generatedMethods = generateMethods(methods);
        for (StubObject instanceMethod : generatedMethods) {
            windowObject.putAttribute(instanceMethod);
        }

        allStubs.put("window", windowObject);
    }

    private static List<StubObject> generateMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "window", true);
        for (StubObject method : methodStubs) {
            if ("addEventListener".equals(method.getName())) {
                method.setSyntax("target.addEventListener(type, listener[, useCapture])");
                method.addParameter(new Parameter("type", "String", "A string representing the event type to listen for."));
                method.addParameter(new Parameter("listener", "Function", "The object that receives a notification when an event of the specified type occurs. This must be an object implementing the EventListener interface, or simply a JavaScript function."));
                method.addParameter(new Parameter("useCapture", "Boolean", "If true, useCapture indicates that the user wishes to initiate capture. After initiating capture, all events of the specified type will be dispatched to the registered listener before being dispatched to any EventTarget beneath it in the DOM tree. Events which are bubbling upward through the tree will not trigger a listener designated to use capture. See DOM Level 3 Events for a detailed explanation. Note that this parameter is not optional in all browser versions. If not specified, useCapture is false.", true));
                method.addParameter(new Parameter("aWantsUntrusted", "Boolean", "If true, the event can be triggered by untrusted content. See Interaction between privileged and non-privileged pages."));
            } else if ("alert".equals(method.getName())) {
                method.addParameter(new Parameter("message", "String", "message is a string of text you want to display in the alert dialog, or, alternatively, an object that is converted into a string and displayed."));
            } else if ("atob".equals(method.getName())) {
                method.setSyntax("var decodedData = window.atob(encodedData);");
                method.addParameter(new Parameter("encodedData", "String", ""));
            } else if ("btoa".equals(method.getName())) {
                method.setSyntax("var encodedData = window.btoa(stringToEncode);");
                method.addParameter(new Parameter("stringToEncode", "String", ""));
            } else if ("clearImmediate".equals(method.getName())) {
                method.setSyntax("window.clearImmediate(immediateID)");
                method.addParameter(new Parameter("immediateID", "Number", "immediateID is a ID returned by window.setImmediate ."));
            } else if ("clearInterval".equals(method.getName())) {
                method.setSyntax("window.clearInterval(intervalID)");
                method.addParameter(new Parameter("intervalID", "Number", "intervalID is the identifier of the repeated action you want to cancel. This ID is returned from setInterval()."));
            } else if ("clearTimeout".equals(method.getName())) {
                method.setSyntax("window.clearTimeout(timeoutID)");
                method.addParameter(new Parameter("timeoutID", "Number", "timeoutID is the ID of the timeout you wish to clear, as returned by window.setTimeout()."));
            } else if ("confirm".equals(method.getName())) {
                method.setSyntax("result = window.confirm(message);");
                method.addParameter(new Parameter("message", "String", "message is the string to be displayed in the dialog. "));
            } else if ("dump".equals(method.getName())) {
                method.setSyntax("dump(message);");
                method.addParameter(new Parameter("message", "String", "message is the string message to log."));
            } else if ("escape".equals(method.getName())) {
                method.setSyntax("escaped = escape(regular);");
                method.addParameter(new Parameter("message", "String", "escaped is the encoded string."));
            } else if ("find".equals(method.getName())) {
                method.setSyntax("window.find(aString, aCaseSensitive, aBackwards, aWrapAround, aWholeWord, aSearchInFrames, aShowDialog);");
                method.addParameter(new Parameter("aString", "String", "The text string for which to search."));
                method.addParameter(new Parameter("aCaseSensitive", "Boolean", "If true, specifies a case-sensitive search."));
                method.addParameter(new Parameter("aBackwards", "Boolean", "If true, specifies a backward search."));
                method.addParameter(new Parameter("aWrapAround", "Boolean", "If true, specifies a wrap around search."));
                method.addParameter(new Parameter("aWholeWord", "Boolean", "If true, specifies a whole word search."));
                method.addParameter(new Parameter("aSearchInFrames", "Boolean", "If true, specifies a search in frames."));
                method.addParameter(new Parameter("aShowDialog", "Boolean", "If true, specifies a show Dialog."));
            } else if ("getComputedStyle".equals(method.getName())) {
                method.setSyntax("var style = window.getComputedStyle(element[, pseudoElt]);");
                method.addParameter(new Parameter("element", "Element", "The Element for which to get the computed style."));
                method.addParameter(new Parameter("pseudoElt", "String", "A string specifying the pseudo-element to match. Must be null (or not specified) for regular elements."));
            } else if ("getSelection".equals(method.getName())) {
                method.setSyntax("selection = window.getSelection() ;");
            } else if ("matchMedia".equals(method.getName())) {
                method.setSyntax("mql = window.matchMedia(mediaQueryString)");
                method.addParameter(new Parameter("mediaQueryString", "String", "mediaQueryString is a string representing the media query for which to return a new MediaQueryList object"));
            } else if ("moveBy".equals(method.getName())) {
                method.setSyntax("window.moveBy(deltaX, deltaY)");
                method.addParameter(new Parameter("deltaX", "Number", "deltaX is the amount of pixels to move the window horizontally"));
                method.addParameter(new Parameter("deltaY", "Number", "deltaY is the amount of pixels to move the window vertically"));
            } else if ("moveTo".equals(method.getName())) {
                method.setSyntax("window.moveTo(x, y)");
                method.addParameter(new Parameter("x", "Number", "x is the horizontal coordinate to be moved to"));
                method.addParameter(new Parameter("y", "Number", "y is the vertical coordinate to be moved to"));
            } else if ("open".equals(method.getName())) {
                method.setSyntax("var windowObjectReference = window.open(strUrl, strWindowName[, strWindowFeatures]);");
                method.setReturns(new Return("Window", "A reference to the newly created window. If the call failed, it will be null. The reference can be used to access properties and methods of the new window provided it complies with Same origin policy security requirements."));
                method.addParameter(new Parameter("strUrl", "String", "The URL to be loaded in the newly opened window.  strUrl can be an HTML document on the web, image file or any resource supported by the browser."));
                method.addParameter(new Parameter("strWindowName", "String", "A string name for the new window. The name can be used as the target of links and forms using the target attribute of an <a> or <form> element. The name should not contain any blank space. Note that strWindowName does not specify the title of the new window."));
                method.addParameter(new Parameter("strWindowFeatures", "String", "Optional parameter listing the features (size, position, scrollbars, etc.) of the new window. The string must not contain any blank space, each feature name and value must be separated by a comma."));
            } else if ("openDialog".equals(method.getName())) {
                method.setSyntax("newWindow = openDialog(url, name, features, arg1, arg2, ...)");
                method.setReturns(new Return("Window", "The opened window"));
                method.addParameter(new Parameter("url", "String", "The URL to be loaded in the newly opened window."));
                method.addParameter(new Parameter("name", "String", "The window name (optional). See window.open description for detailed information."));
                method.addParameter(new Parameter("features", "String", "See window.open description for description."));
                method.addParameter(new Parameter("arg1, arg2, ...", "Object", "The arguments to be passed to the new window (optional)."));
            } else if ("postMessage".equals(method.getName())) {
                method.setSyntax("otherWindow.postMessage(message, targetOrigin);");
                method.addParameter(new Parameter("message", "String", "Data to be sent to the other window."));
                method.addParameter(new Parameter("targetOrigin", "String", "Specifies what the origin of otherWindow must be for the event to be dispatched, either as the literal string \"*\" (indicating no preference) or as a URI. If at the time the event is scheduled to be dispatched the scheme, hostname, or port of otherWindow's document does not match that provided in targetOrigin, the event will not be dispatched; only if all three match will the event be dispatched. This mechanism provides control over where messages are sent; for example, if postMessage were used to transmit a password, it would be absolutely critical that this argument be a URI whose origin is the same as the intended receiver of the message containing the password, to prevent interception of the password by a malicious third party. Always provide a specific targetOrigin, not *, if you know where the other window's document should be located. Failing to provide a specific target discloses the data you send to any interested malicious site."));
            } else if ("prompt".equals(method.getName())) {
                method.setSyntax("result = window.prompt(text, value);");
                method.setReturns(new Return("String|null", "result is a string containing the text entered by the user, or the value null"));
                method.addParameter(new Parameter("text", "String", "text is a string of text to display to the user. This parameter are optional and can be omitted if there is nothing to show in the prompt window."));
                method.addParameter(new Parameter("value", "String", "value is a string containing the default value displayed in the text input field. It is an optional parameter. Note that in Internet Explorer 7 and 8, if you do not provide this parameter, the string \"undefined\" is the default value."));
            } else if ("removeEventListener".equals(method.getName())) {
                method.setSyntax("element.removeEventListener(type, listener, useCapture)");
                method.addParameter(new Parameter("type", "String", "A string representing the event type being removed."));
                method.addParameter(new Parameter("listener", "Function", "The listener parameter indicates the EventListener function to be removed."));
                method.addParameter(new Parameter("useCapture", "Boolean", "Specifies whether the EventListener being removed was registered as a capturing listener or not. If a listener was registered twice, one with capture and one without, each must be removed separately. Removal of a capturing listener does not affect a non-capturing version of the same listener, and vice versa.", true));
            } else if ("resizeBy".equals(method.getName())) {
                method.setSyntax("window.resizeBy(xDelta, yDelta)");
                method.addParameter(new Parameter("xDelta", "Number", "xDelta is the number of pixels to grow the window horizontally"));
                method.addParameter(new Parameter("yDelta", "Number", "yDelta is the number of pixels to grow the window vertically"));
            } else if ("resizeTo".equals(method.getName())) {
                method.setSyntax("window.resizeTo(iWidth, iHeight)");
                method.addParameter(new Parameter("iWidth", "Number", "iWidth is an integer representing the new outerWidth in pixels (including scroll bars, title bars, etc)"));
                method.addParameter(new Parameter("iHeight", "Number", "iHeight is an integer value representing the new outerHeight in pixels (including scroll bars, title bars, etc)"));
            } else if ("scroll".equals(method.getName())) {
                method.setSyntax("window.scroll(x-coord, y-coord)");
                method.addParameter(new Parameter("x-coord", "Number", "x-coord is the pixel along the horizontal axis of the document that you want displayed in the upper left"));
                method.addParameter(new Parameter("y-coord", "Number", "y-coord is the pixel along the vertical axis of the document that you want displayed in the upper left"));
            } else if ("scrollBy".equals(method.getName())) {
                method.setSyntax("window.scrollBy(X, Y);");
                method.addParameter(new Parameter("X", "Number", "X is the offset in pixels to scroll horizontally"));
                method.addParameter(new Parameter("Y", "Number", "Y is the offset in pixels to scroll vertically"));
            } else if ("scrollByLines".equals(method.getName())) {
                method.setSyntax("window.scrollByLines(lines)");
                method.addParameter(new Parameter("lines", "Number", "Lines is the number of lines to scroll the document by. Lines may be a positive or negative integer."));
            } else if ("scrollByPages".equals(method.getName())) {
                method.setSyntax("window.scrollByPages(pages)");
                method.addParameter(new Parameter("pages", "Number", "Pages is the number of pages to scroll. Pages may be a positive or negative integer."));
            } else if ("scrollTo".equals(method.getName())) {
                method.setSyntax("window.scrollTo(x-coord, y-coord)");
                method.addParameter(new Parameter("x-coord", "Number", "x-coord is the pixel along the horizontal axis of the document that you want displayed in the upper left"));
                method.addParameter(new Parameter("y-coord", "Number", "y-coord is the pixel along the vertical axis of the document that you want displayed in the upper left"));
            } else if ("setImmediate".equals(method.getName())) {
                method.setSyntax("var immediateID = setImmediate(func, [param1, param2, ...]);<br>var immediateID = setImmediate(func);");
                method.setReturns(new Return("immediateID", "immediateID is the ID of the immediate which can be used later with window.clearImmediate"));
                method.addParameter(new Parameter("func", "Function", "func is the function you wish to call"));
                method.addParameter(new Parameter("param1, param2, ...", "Object", "All parameters will be passed directly to your function."));
            } else if ("setInterval".equals(method.getName())) {
                method.setSyntax("var intervalID = window.setInterval(func, delay[, param1, param2, ...]);<br>var intervalID = window.setInterval(code, delay);");
                method.setReturns(new Return("Number", "intervalID is a unique interval ID you can pass to clearInterval()"));
                method.addParameter(new Parameter("func", "Function", "func is the function you want to be called repeatedly"));
                method.addParameter(new Parameter("code", "String", "code in the alternate syntax, is a string of code you want to be executed repeatedly. (Using this syntax is not recommended for the same reasons as using eval())"));
                method.addParameter(new Parameter("delay", "Number", "delay is the number of milliseconds (thousandths of a second) that the setInterval() function should wait before each call to func. As with setTimeout, there is a minimum delay enforced"));
                method.addParameter(new Parameter("param1, param2, ...", "Object", "All parameters will be passed directly to your function."));
            } else if ("setTimeout".equals(method.getName())) {
                method.setSyntax("var timeoutID = window.setTimeout(func, delay, [param1, param2, ...]);<br>var timeoutID = window.setTimeout(code, delay);");
                method.setReturns(new Return("Number", "timeoutID is the numerical ID of the timeout, which can be used later with window.clearTimeout"));
                method.addParameter(new Parameter("func", "Function", "func is the function you want to execute after delay milliseconds"));
                method.addParameter(new Parameter("code", "String", "code in the alternate syntax, is a string of code you want to execute after delay milliseconds. (Using this syntax is not_recommended for the same reasons as using eval())"));
                method.addParameter(new Parameter("delay", "Number", "delay is the number of milliseconds (thousandths of a second) that the function call should be delayed by. The actual delay may be longer, see Notes below"));
                method.addParameter(new Parameter("param1, param2, ...", "Object", "All parameters will be passed directly to your function."));
            } else if ("showModalDialog".equals(method.getName())) {
                method.setSyntax("returnVal = window.showModalDialog(uri[, arguments][, options]);");
                method.setReturns(new Return("Number", "returnVal is a variant, indicating the returnValue property as set by the window of the document specified by uri"));
                method.addParameter(new Parameter("url", "String", "uri is the URI of the document to display in the dialog box"));
                method.addParameter(new Parameter("arguments", "String", "arguments is an optional variant that contains values that should be passed to the dialog box; these are made available in the window object's window.dialogArguments property", true));
                method.addParameter(new Parameter("options", "String", "options an optional string that specifies window ornamentation for the dialog box, using one or more semicolon delimited values", true));
            } else if ("unescape".equals(method.getName())) {
                method.setSyntax("regular = window.unescape(escaped)");
                method.setReturns(new Return("String", "Regular is the decoded string."));
                method.addParameter(new Parameter("escaped", "String", "escaped is the encoded string"));
            } else if ("updateCommands".equals(method.getName())) {
                method.setSyntax("window.updateCommands(\"sCommandName\")");
                method.addParameter(new Parameter("sCommandName", "String", "sCommandName is a particular string which describes what kind of update event this is (e.g. whether we are in bold right now)"));
            } else if ("Image".equals(method.getName())) {
                method.setSyntax("image = new window.Image(width, height)");
                method.addParameter(new Parameter("width", "Number", null, true));
                method.addParameter(new Parameter("height", "Number", null, true));
            } else if ("Option".equals(method.getName())) {
                method.setSyntax("option = new window.Option(text, value, defaultSelected, selected)");
                method.addParameter(new Parameter("text", "String", "the displayed text for the option", true));
                method.addParameter(new Parameter("value", "String", "the form value", true));
                method.addParameter(new Parameter("defaultSelected", "Boolean", "a boolean which does not actually select the option, but is used to indicate the state of the defaultSelected property (of relevance for any future checks to check whether the option is meant to be selected by default)", true));
                method.addParameter(new Parameter("selected", "Boolean", "a boolean to indicate whether the option will be \"selected\"; if omitted, it will not be selected", true));
            } else if ("Worker".equals(method.getName())) {
                method.setSyntax("worker = new window.Worker(uri)");
                method.addParameter(new Parameter("url", "String", "The URI passed as parameter of the Worker constructor must obey the same-origin policy."));
            } else {
                method.setSyntax("window." + method.getName() + "()");
            }
        }
        return methodStubs;
    }

}
