/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.server.services;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.netbeans.server.uihandler.api.Issue;

/**
 *
 * @author Jindrich Sedek
 */
class IssueDBTask implements BugzillaDBConnector.DBTask<Issue> {

    private int id;

    IssueDBTask(Integer id) {
        this.id = id;
    }

    public Issue process(Connection conn) throws SQLException {
        Issue issue = new Issue(id);
        Statement st = conn.createStatement();
        ResultSet rs = st.executeQuery("SELECT b.bug_status, b.priority, b.resolution, p.name as product, c.name as component, " + "b.target_milestone, b.cf_bug_type, b.status_whiteboard, b.short_desc, b.reporter, b.cf_autoreporter_id " + "FROM bugs b, products p, components c " + "WHERE p.id = b.product_id AND c.id = b.component_id AND b.bug_id = " + id);
        if (rs.next()) {
            issue.setIssueStatus(rs.getString("bug_status"));
            issue.setPriority(rs.getString("priority"));
            issue.setResolution(rs.getString("resolution"));
            issue.setComponent(rs.getString("product"));
            issue.setSubcomponent(rs.getString("component"));
            issue.setTargetMilestone(rs.getString("target_milestone"));
            issue.setIssueType(rs.getString("cf_bug_type"));
            issue.setStatusWhiteboard(rs.getString("status_whiteboard"));
            issue.setShortDesc(rs.getString("short_desc"));
            issue.setReporter(rs.getString("reporter"));
            String cf_autoreporter = rs.getString("cf_autoreporter_id");
            try{
                int i = Integer.parseInt(cf_autoreporter);
                issue.setReporterSubmitId(i);
            }catch (NumberFormatException nfe){
                // let the value to be null
            }
        }
        rs.close();
        
        StringBuilder kwSb = new StringBuilder("");
        rs = st.executeQuery("SELECT name FROM keyworddefs as kd,keywords as k WHERE k.keywordid = kd.id and bug_id=" + id);
        while (rs.next()){
            kwSb.append(rs.getString("name"));
            kwSb.append(",");
        }
        // return null or comma separated list
        String keywords = kwSb.toString();
        if(keywords.equals("")){
            keywords = null;
        } else {
            keywords = kwSb.substring(0, kwSb.length()-1);
        }
        issue.setKeywords(keywords);
        
        rs = st.executeQuery("SELECT dupe_of FROM duplicates WHERE dupe = " + id);
        if (rs.next()){
            issue.setDuplicateOf(rs.getInt(1));
        }
        rs.close();
        
        rs = st.executeQuery("SELECT MAX(bug_when) AS max_when FROM bugs_activity a, fielddefs f WHERE a.fieldid = f.id AND f.name = 'resolution' AND a.bug_id = " + id);
        Date lastResolution = null;
        if (rs.next()) {
            lastResolution = rs.getDate("max_when");
        }
        issue.setLastResolutionChange(lastResolution);
        rs.close();

        List<String> ccUsers = new ArrayList<String>();
        rs = st.executeQuery("SELECT login_name FROM profiles p, cc WHERE cc.who = p.userid AND cc.bug_id = " + id);
        while (rs.next()) {
            String user = rs.getString("login_name");
            ccUsers.add(user);
        }
        issue.setCcUsers(ccUsers);
        rs.close();
        st.close();
        return issue;
    }
}
