/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.uihandler.logsubmitter;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.zip.GZIPInputStream;
import org.netbeans.lib.uihandler.LogRecords;

/**
 *
 * @author Jindrich Sedek
 */
public class Main {

    public static void main(String[] args) throws Exception {
        if (args.length < 2) {
            System.out.println("Set correct arguments: <TARGET SERVER URL> <SUBMITTED LOGFILE NAME> <SUBMITTER ID>");
            return;
        }
        URL serverUrl = new URL(args[0]);
        File submitFile = new File(args[1]);
        if (args.length > 2){
            System.out.println(args[2]);
            Uploader.id = args[2];
        }
        InputStream fis = new FileInputStream(submitFile);
        GZIPInputStream gis = new GZIPInputStream(fis);
        SubmitterHandler handl = new SubmitterHandler();
        LogRecords.scan(gis, handl);
        URL resultURL = Uploader.uLogs(serverUrl, handl.getRecords());
        assert resultURL != null;
        System.out.println(resultURL);
    }

    private static class SubmitterHandler extends Handler {

        private List<LogRecord> records = new ArrayList<LogRecord>();
        static final String USER_CONFIGURATION = "UI_USER_CONFIGURATION"; // NOI18N


        public SubmitterHandler() {
        }

        @Override
        public void publish(LogRecord record) {
            if (USER_CONFIGURATION.equals(record.getMessage())) {
                Object[] params = record.getParameters();
                if (params != null && params.length >= 4) {
                    for (int i = 3; i < params.length; i++) {
                        params[i] = "text";//test params
                    }
                    record.setParameters(params);
                }
            }
            records.add(record);
        }

        @Override
        public void flush() {
        }

        @Override
        public void close() throws SecurityException {
        }

        List<LogRecord> getRecords() {
            return records;
        }
    }
}












