/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.server.uihandler;

import org.netbeans.server.uihandler.api.Issue;
import org.netbeans.modules.exceptions.utils.BugReporterFactory;
import org.netbeans.modules.exceptions.utils.LogReporter;
import org.netbeans.junit.MockServices;
import org.netbeans.junit.NbTestCase;
import org.netbeans.modules.exceptions.entity.Nbuser;
import org.netbeans.modules.exceptions.entity.Report;
import static org.junit.Assert.*;

/**
 *
 * @author Jindrich Sedek
 */
public class IZInsertionTest extends NbTestCase {

    public IZInsertionTest(String name) {
        super(name);
    }

    public void testIssue150245() throws Exception {
        MockServices.setServices(LogReporter.class);
        IZInsertion.IssueZillaInsertionFilter<org.netbeans.modules.exceptions.entity.Exceptions> filter = new IZInsertion.AddToCCFilter<org.netbeans.modules.exceptions.entity.Exceptions>();

        org.netbeans.modules.exceptions.entity.Exceptions exc = new org.netbeans.modules.exceptions.entity.Exceptions(1);
        Report report = new Report(1);
        report.setIssueId(10);
        exc.setReportId(report);
        Nbuser user = new Nbuser(1);
        user.setName("jindra");
        exc.setNbuserId(user);
        new IZInsertion(exc, null);

        Issue iss = BugReporterFactory.getDefaultReporter().getIssue(10);
        iss.setReporter("tester");
        assertEquals(IZInsertion.InsertionResult.ADDTOCC, filter.isInsertable(null, exc));
        iss.setReporter("jindra");
        assertEquals(IZInsertion.InsertionResult.NONE, filter.isInsertable(null, exc));
    }
    
     public void testDirectComponentFilter() throws Exception {
        MockServices.setServices(LogReporter.class);
        IZInsertion.DirectComponentFilter<org.netbeans.modules.exceptions.entity.Exceptions> filter = new IZInsertion.DirectComponentFilter<org.netbeans.modules.exceptions.entity.Exceptions>();
        
        org.netbeans.modules.exceptions.entity.Exceptions exc = new org.netbeans.modules.exceptions.entity.Exceptions(1);
        Report report = new Report(1);
        exc.setReportId(report);
        report.setComponent("cnd");
        Nbuser user = new Nbuser(1);
        user.setName("tomas");
        exc.setNbuserId(user);
        //will insert only if not guest && if component == cnd && if !inBugzilla
        assertEquals(IZInsertion.InsertionResult.INSERT, filter.isInsertable(null, exc));

        report.setComponent("ide");
        assertEquals(IZInsertion.InsertionResult.NONE, filter.isInsertable(null, exc));
        report.setComponent("cnd");
        
        report.setIssueId(123456);
        assertEquals(IZInsertion.InsertionResult.NONE, filter.isInsertable(null, exc));
        report.setIssueId(null);
        
        user.setName("GUEST");
        assertEquals(IZInsertion.InsertionResult.NONE, filter.isInsertable(null, exc));
     }
}
