/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.mdrant;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;

import javax.jmi.reflect.*;

import org.apache.tools.ant.BuildException;

import org.netbeans.api.mdr.*;
import org.netbeans.lib.jmi.mapping.FileStreamFactory;
import org.netbeans.lib.jmi.mapping.JMIMapperImpl;

/** Creates JMI maping in form of source files.
 *
 * @author  Petr Hrebejk
 */
public class MapJava extends MdrTask.Sub {

    private File dir; 
    private File headerFile;
    private String extent; 
    
    /** Creates a new instance of PrintExtentsNames */
    public MapJava() {
    }
    
    public void execute() throws Exception {
        
        if ( dir == null ) {
            throw new BuildException( "Destination directory has to be sppecified. Use the \"dir\" attribute" );
        }
        
        if ( extent == null ) {
            throw new BuildException( "Extent to map has to be sppecified. Use the \"extent\" attribute" );
        }
        
        JMIStreamFactory streamFactory = new FileStreamFactory( dir );
        
        getMapper().generate( streamFactory, getRepository().getExtent( extent ) );
        
    }
    
    // ANT task attributes -----------------------------------------------------
    
    public void setDir( File dir ) {
      this.dir = dir;
    }
    
    public void setExtent( String extent ) {
        this.extent = extent;
    }
    
    public void setHeaderFile(File headerFile) {
        this.headerFile = headerFile;
    }
    
    // Protected methods -------------------------------------------------------
    
    protected JMIMapper getMapper() throws IOException {
        JMIMapperImpl result = new JMIMapperImpl();
        if (headerFile != null) {
            BufferedReader reader = new BufferedReader(new FileReader(headerFile));
            StringWriter header = new StringWriter();
            int ch;
            while ((ch = reader.read()) != -1) {
                header.write(ch);
            }
            reader.close();
            header.close();
            result.setHeader(header.toString());
        }
        return result;
    }
    
    
}
