/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdrant;

import java.io.*;
import java.util.*;
import org.openide.ErrorManager;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;

/**
 * Logger which forwards MDR notifications to Ant.
 *
 * @author John V. Sichi
 */
public class MdrantLogger extends ErrorManager
{
    private static Task task;

    public MdrantLogger()
    {
    }

    static void setTask(Task newTask)
    {
        task = newTask;
    }

    // implement ErrorManager
    public Throwable attachAnnotations(
        Throwable t, Annotation[] arr)
    {
        return t;
    }
        
    // implement ErrorManager
    public Annotation[] findAnnotations(Throwable t)
    {
        return null;
    }
        
    // implement ErrorManager
    public Throwable annotate(
        Throwable t, int severity,
        String message, String localizedMessage,
        Throwable stackTrace, java.util.Date date)
    {
        if (task != null) {
            task.log(message, convertSeverity(severity));
        }
        return t;
    }
        
    // implement ErrorManager
    public void notify(int severity, Throwable t)
    {
        if (task != null) {
            task.log(t.toString(), convertSeverity(severity));
        }
    }
        
    // implement ErrorManager
    public void log(int severity, String s)
    {
        if (task != null) {
            task.log(s, convertSeverity(severity));
        }
    }
        
    // implement ErrorManager
    public ErrorManager getInstance(String name)
    {
        return this;
    }
    
    private static int convertSeverity(int severity)
    {
        switch(severity) {
        case INFORMATIONAL:
            return Project.MSG_VERBOSE;
        case WARNING:
            return Project.MSG_WARN;
        case USER:
            return Project.MSG_INFO;
        case EXCEPTION:
            return Project.MSG_ERR;
        case ERROR:
            return Project.MSG_ERR;
        default:
            return Project.MSG_DEBUG;
        }
    }
}

// End MdrantLogger.java
