/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.lib.jmi.query;

import java.util.Iterator;
import java.util.NoSuchElementException;

/** Alters the objects in the query to other objects.
 *
 * @author Petr Hrebejk
 */
public abstract class AlterQuery implements Query {

    private final Query QUERY;
    private final Alternator ALTERNATOR;

    AlterQuery(Query query) {
        this.QUERY = query;
        this.ALTERNATOR = null;
    }

    AlterQuery(Query query, Alternator alternator ) {
        this.QUERY = query;
        this.ALTERNATOR = alternator;
    }

    public Iterator iterator() {
        return new AlterIterator();
    }
    
    public boolean contains( Object object ) {
        return QUERY.contains( alterBack( object ) );
    }
    
    protected Object alter( Object object ) {
        if ( ALTERNATOR == null ) {
            return object;
        }
        else {
            return ALTERNATOR.alter( object );
        }
    }
    
    protected Object alterBack( Object object ) {
        if ( ALTERNATOR == null ) {
            return object;
        }
        else {
            return ALTERNATOR.alterBack( object );
        }
    }
    
    // Inner classes -----------------------------------------------------------
    
    public interface Alternator {
        
        public Object alter( Object object );
        
        public Object alterBack( Object object );
    }
    
    private class AlterIterator extends QueryIterator.Delegate {
        
        /** The object which will be returned as next */
        private Object next = null;
        
        AlterIterator() {
            super ( QUERY.iterator() );
        }
        
        public boolean hasNext() {
            return ITERATOR.hasNext();
        }
        
        public Object next() {
            
            Object next = ITERATOR.next();    
            return alter( next );
        }
    }
    
}
