/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.lib.jmi.xmi;

import org.netbeans.api.xmi.*;
import org.netbeans.api.xmi.sax.*;
import org.xml.sax.*;
import org.xml.sax.helpers.LocatorImpl;

import java.util.*;
import java.io.*;
import javax.jmi.reflect.RefPackage;

public class Producer extends XMIProducer {

    private XMIOutputConfig config;
    private Collection objects = null;
    private RefPackage extent = null;
    private String xmiVersion = null;

    private DTDHandler dtdHandler = null;
    private ContentHandler contentHandler = null;
    private EntityResolver entityResolver = null;
    private ErrorHandler errorHandler = null;
    
    private ContentHandler defaultWriter = new DefaultWriter ();
    
    // init .....................................................................
    
    public Producer () {
        config = new OutputConfig ();
    }
    
    public Producer (XMIOutputConfig config) {
        this.config = config;
    }
    
    // XMIProducer implementation ...............................................
    
    public void setSource(Collection objects) {
        this.objects = objects;
        extent = null;
    }

    public void setSource(RefPackage extent) {
        this.extent = extent;
        objects = null;
    }

    public Object getSource() {
        if (objects == null)
            return extent;
        else
            return objects;        
    }

    public void setXmiVersion(String xmiVersion) {
        this.xmiVersion = xmiVersion;
    }

    public String getXmiVersion() {
        return xmiVersion;
    }

    public XMIOutputConfig getConfiguration() {
        return config;
    }

    // XMLReader implementation .................................................
    
    public ContentHandler getContentHandler() {
        return contentHandler;
    }

    public DTDHandler getDTDHandler() {
        return dtdHandler;
    }

    public EntityResolver getEntityResolver() {
        return entityResolver;
    }

    public ErrorHandler getErrorHandler() {
        return errorHandler;
    }

    public boolean getFeature(String name) throws SAXNotSupportedException {
        throw new SAXNotSupportedException (null);
    }

    public Object getProperty(String name) throws SAXNotSupportedException {
        throw new SAXNotSupportedException (null);
    }

    public void parse(InputSource input) throws IOException, SAXException {
        parse(input.getSystemId());
    }

    public void parse(String systemId) throws IOException, SAXException {
        ContentHandler handler = (contentHandler != null) ? contentHandler : defaultWriter;
        LocatorImpl locator = new LocatorImpl ();
        locator.setSystemId (systemId);
        handler.setDocumentLocator (locator);        
        if (extent != null) {
            getWriter ().write (contentHandler, systemId, extent, xmiVersion);
        } else if (objects != null) {
            getWriter ().write (contentHandler, systemId, objects, xmiVersion);
        } else {
            throw new SAXException ("Source not specified.");
        }
    }
   
    public void setContentHandler(ContentHandler handler) {
        contentHandler = handler;
    }

    public void setDTDHandler(DTDHandler handler) {
        dtdHandler = handler;
    }

    public void setEntityResolver(EntityResolver resolver) {
        entityResolver = resolver;
    }

    public void setErrorHandler(ErrorHandler handler) {
        errorHandler = handler;
    }

    public void setFeature(String name, boolean value) throws SAXNotSupportedException {
        throw new SAXNotSupportedException (null);
    }

    public void setProperty(String name, Object value) throws SAXNotSupportedException {
        throw new SAXNotSupportedException (null);
    }
    
    private WriterBase getWriter () {
        if ((xmiVersion != null) && xmiVersion.equals (DelegatingWriter.XMI_VERSION_20))
            return new XMI20Writer (config);
        else
            return new WriterBase (config);
    }
    
}
