/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is Forte for Java, Community Edition. The Initial
 * Developer of the Original Software is Sun Microsystems, Inc. Portions
 * Copyright 1997-2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.mdrxml.looks;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.io.IOException;
import org.openide.nodes.*;
import org.openide.util.NbBundle;
import org.netbeans.api.looks.*;
import org.netbeans.spi.looks.*;
import xmlmodel.AttributeNode;

/**
 *
 * @author  Tomas Zezula
 */
public class AttributeNodeLook extends NodeLook {

    private static final String ICON = "org/netbeans/modules/mdrxml/resources/attributeNode";

    private static class ValueProperty extends PropertySupport.ReadWrite {
        
        private AttributeNode node;
        
        public ValueProperty (AttributeNode node) {
            super (NbBundle.getMessage(AttributeNodeLook.class,"TXT_Value"),String.class,NbBundle.getMessage(AttributeNodeLook.class,"TXT_Value"),NbBundle.getMessage(AttributeNodeLook.class,"TIP_Value"));
            this.node = node;
        }
        
        public Object getValue () {
            if (this.node != null)
                return this.node.getValue();
            else
                return null;
        }
        
        public void setValue (Object value) {
            if (value instanceof String && this.node != null)
                node.setValue ((String)value);
        }
    }
    
    /** Creates a new instance of AttributeNodeLook */
    public AttributeNodeLook() {
        super (NbBundle.getMessage (AttributeNodeLook.class, "TXT_AttributeNodeLook"));
    }
    
    public String iconBase (Look.NodeSubstitute substitute) {
        return ICON;
    }
    
    public Node.PropertySet[] getPropertySets (Look.NodeSubstitute substitute) {
        Node.PropertySet[] sets = super.getPropertySets (substitute);
        ((Sheet.Set)sets[0]).put ( new ValueProperty ((AttributeNode)substitute.getRepresentedObject()));
        return sets;
    }
    
    public Transferable clipboardCopy (Look.NodeSubstitute substitute) throws IOException {
        AttributeNode node = (AttributeNode) substitute.getRepresentedObject ();
        String str = node.getName () + " = " + node.getValue ();    //NOI18N
        return new StringSelection (str);
    }
    
}
