/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is Forte for Java, Community Edition. The Initial
 * Developer of the Original Software is Sun Microsystems, Inc. Portions
 * Copyright 1997-2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.mdrxml.looks;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import javax.jmi.reflect.RefBaseObject;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.netbeans.api.mdr.*;
import org.netbeans.api.looks.*;
import org.netbeans.spi.looks.*;
import xmlmodel.ElementNode;
import xmlmodel.Node;
import xmlmodel.AttributeNode;
import org.netbeans.modules.mdrxml.looks.actions.GenerateXMLAction;
import org.netbeans.modules.mdrxml.looks.actions.NewAction;
import org.netbeans.modules.mdrxml.util.XMLModelEventTranslator;
import org.netbeans.modules.mdrxml.util.NodeEventTranslator;
import org.netbeans.modules.mdrxml.util.XMLGenerator;
/**
 *
 * @author  Tomas Zezula
 */
public class ElementNodeLook extends NodeLook {
    
    private static final String ICON = "org/netbeans/modules/mdrxml/resources/elementNode";
    
    /** Creates a new instance of ElementNode */
    public ElementNodeLook() {
        this(NbBundle.getMessage(ElementNodeLook.class, "TXT_ElementNodeLook"));
    }
    
    public Object attachTo(Look.NodeSubstitute substitute) {
        super.attachTo(substitute);
        NodeEventTranslator nt = null;
        try {
            RefBaseObject mdrInstance = (javax.jmi.reflect.RefObject)substitute.getRepresentedObject();
            XMLModelEventTranslator translator = XMLModelEventTranslator.getTranslator((xmlmodel.XmlmodelPackage)mdrInstance.refImmediatePackage());
            nt = new NodeEventTranslator(translator,mdrInstance,substitute);
        }catch (javax.jmi.reflect.InvalidObjectException ioe) {
        }
        return nt;
    }
    
    public ElementNodeLook(String name) {
        super(name);
    }
    
    public String iconBase(Look.NodeSubstitute substitute) {
        return ICON;
    }
    
    public javax.swing.Action[] getActions(Look.NodeSubstitute substitute) {
        javax.swing.Action[] parentActions = super.getActions(substitute);
        javax.swing.Action[] result = new javax.swing.Action [parentActions.length + 2];
        result[0] = SystemAction.get(GenerateXMLAction.class);
        result[1] = SystemAction.get(NewAction.class);
        System.arraycopy(parentActions,0,result,2,parentActions.length);
        return result;
    }
    
    public Object[] getChildObjects(Look.NodeSubstitute substitute) {
        try {
            ElementNode en = (ElementNode) substitute.getRepresentedObject();
            MDRepository repo = ((MDRObject)en).repository();
            repo.beginTrans(false);
            try {
                Collection nodes = en.getNodes();
                Object[] result = new Object [nodes.size()];
                Iterator it = nodes.iterator();
                for (int i=0; it.hasNext(); i++) {
                    result[i] = it.next();
                }
                return result;
            }finally {
                repo.endTrans();
            }
        }catch (javax.jmi.reflect.InvalidObjectException ioe) {
            return null;
        }
    }
    
    public Transferable clipboardCopy (Look.NodeSubstitute substitute) throws IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream ();
        new XMLGenerator ().generateXML (bos, (ElementNode)substitute.getRepresentedObject());
        return new StringSelection (bos.toString());
    }
    
}
