/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.jmiimpl.mof.model;

import java.util.*;

import javax.jmi.model.*;
import javax.jmi.reflect.*;

import org.netbeans.mdr.handlers.BaseObjectHandler;
import org.netbeans.mdr.handlers.AssociationHandler;
import org.netbeans.mdr.handlers.IndexSetWrapper;
import org.netbeans.mdr.storagemodel.AssociationLink;
import org.netbeans.mdr.storagemodel.StorableAssociation;
import org.netbeans.mdr.storagemodel.StorableObject;

/** Implementation of DependsOn association which is derived in the MOF model.
 *
 * @author Martin Matula
 * @version 
 */
public abstract class DependsOnImpl extends AssociationHandler implements DependsOn {

    /** Creates new DependsOnImpl */
    protected DependsOnImpl(StorableAssociation storable) {
        super(storable);
    }

    // [PENDING] should be probably live
    public Collection refAllLinks() {
        HashSet allLinks = new HashSet();
        ModelElement dependent;
        
        for (Iterator it = ((ModelPackage) refImmediatePackage()).getModelElement().refAllOfType().iterator(); it.hasNext();) {
            dependent = (ModelElement) it.next();
            for (Iterator it2 = dependent.findRequiredElements(ModelElementImpl.ALL_KINDS, true).iterator(); it2.hasNext();) {
                allLinks.add(new AssociationLink((StorableObject) ((BaseObjectHandler) dependent)._getDelegate(), (StorableObject) ((BaseObjectHandler) it2.next())._getDelegate()));
            }
        }

        return new IndexSetWrapper(_getMdrStorage(), allLinks);
    }

    public boolean exists(ModelElement dependent, ModelElement provider) {
        return dependent.isRequiredBecause(provider, new String[1]);
    }

    public Collection getProvider(ModelElement dependent) {
        return dependent.findRequiredElements(ModelElementImpl.ALL_KINDS, true);
    }

    public Collection getDependent(ModelElement provider) {
        Set result = new HashSet();
        findDependentElements(provider, result);
        return result;
    }
    
    private void findDependentElements(ModelElement provider, Set result) {
        Set temp = new HashSet();
        ModelPackage pkg = (ModelPackage) refImmediatePackage();

        if (provider instanceof Constraint) {
            temp.addAll(((Constraint) provider).getConstrainedElements());
        } else if (provider instanceof Namespace) {
            temp.addAll(((Namespace) provider).getContents());
            temp.addAll(pkg.getAliases().getImporter((Namespace) provider));
            if (provider instanceof GeneralizableElement) {
                temp.addAll(pkg.getGeneralizes().getSubtype((GeneralizableElement) provider));
                if (provider instanceof Classifier) {
                    temp.addAll(pkg.getIsOfType().getTypedElements((Classifier) provider));
                }
            } else if (provider instanceof MofException) {
                temp.addAll(pkg.getCanRaise().getOperation((MofException) provider));
            }
        } else if (provider instanceof AssociationEnd) {
            temp.addAll(pkg.getRefersTo().getReferent((AssociationEnd) provider));
            temp.addAll(pkg.getExposes().getReferrer((AssociationEnd) provider));
        }

        temp.addAll(provider.getConstraints());
        temp.add(provider.getContainer());
        temp.addAll(pkg.getAttachesTo().getTag(provider));

        for (Iterator it = temp.iterator(); it.hasNext();) {
            ModelElement me = (ModelElement) it.next();
            if (result.add(me)) {
                findDependentElements(me, result);
            }
        }
    }
}
