/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.jmiimpl.mof.model;

import java.util.*;

import javax.jmi.model.AssociationEnd;
import javax.jmi.model.Reference;
import javax.jmi.model.RefersTo;
import javax.jmi.model.Exposes;
import javax.jmi.model.ModelPackage;
import javax.jmi.reflect.*;

import org.netbeans.mdr.handlers.*;
import org.netbeans.mdr.storagemodel.*;

/** Implementation of Exposes association, which is derived in the MOF model.
 *
 * @author Martin Matula
 * @version
 */
public abstract class ExposesImpl extends AssociationHandler implements Exposes {

    /** Creates new ExposesImpl */
    protected ExposesImpl(StorableAssociation storable) {
        super(storable);
    }

    public Collection refAllLinks() {
        // [PENDING] live collection should be returned
        ModelPackage pkg = (ModelPackage) refImmediatePackage();
        RefersTo assoc = pkg.getRefersTo();
        Collection allLinks = assoc.refAllLinks();
        ArrayList result = new ArrayList(allLinks.size());
        RefAssociationLink link;

        for (Iterator it = allLinks.iterator(); it.hasNext();) {
            link = (RefAssociationLink) it.next();
            result.add(new AssociationLink((StorableObject) ((BaseObjectHandler) link.refFirstEnd())._getDelegate(), (StorableObject) ((BaseObjectHandler) link.refSecondEnd())._getDelegate()));
        }

        return new IndexSetWrapper(_getMdrStorage(), result);
    }

    public boolean exists(Reference referrer, AssociationEnd exposedEnd) {
        return exposedEnd.equals(referrer.getReferencedEnd().otherEnd());
    }

    public AssociationEnd getExposedEnd(Reference referrer) {
        return referrer.getReferencedEnd().otherEnd();
    }

    public Collection getReferrer(AssociationEnd exposedEnd) {
        return ((ModelPackage) refImmediatePackage()).getRefersTo().getReferent(exposedEnd.otherEnd());
    }

    public boolean add(Reference referrer, AssociationEnd exposedEnd) {
        ModelPackage pkg = (ModelPackage) refImmediatePackage();
        RefersTo assoc = (RefersTo) pkg.getRefersTo();
        return assoc.add(referrer, exposedEnd.otherEnd());
    }

    public boolean remove(Reference referrer, AssociationEnd exposedEnd) {
        ModelPackage pkg = (ModelPackage) refImmediatePackage();
        RefersTo assoc = (RefersTo) pkg.getRefersTo();
        return assoc.remove(referrer, exposedEnd.otherEnd());
    }
}
