/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.util;

import java.util.*;
import java.io.*;
import javax.jmi.reflect.*;

/**
 *
 * @author  Pavel Buzek
 * @version
 */
public class XmlUtils extends Object {

    static byte buf[] = new byte[4096];

    public static String readTagStart (InputStream is) throws IOException {
        while (is.read() != '<');
        int c, i=0;
        while ( (c=is.read()) != '>') {
            buf[i++]=(byte)c; // must be ascii
        }
        String result = new String(buf, 0, i);
        return result;
    }

    public static String readValue (InputStream is) throws IOException {
        int c, i=0;
        while ( (c=is.read()) != '<') {
            buf[i++]=(byte)c; // must be ascii
        }
        return decode(new String(buf, 0, i));
    }

    public static String readComplexValue (InputStream is) throws IOException {
        String start = readTagStart(is);
        int c, i=0, left=0;
        for (;;) {
            while ( ((c=is.read()) != '>') ) {
                if (c=='<') left = i;
                buf[i++]=(byte)c; // must be ascii
            }
            try {
            if (start.equals(new String(buf, left+2, i-left-2))) break;
            } catch (StringIndexOutOfBoundsException e) {
                Logger.getDefault().annotate(e, new Integer(left + 2).toString());
                Logger.getDefault().annotate(e, new Integer(i - left - 2).toString());
                Logger.getDefault().annotate(e, new String(buf, 0, i));
                throw e;
            }
            buf[i++]=(byte)'>';
        }
            
        String value = new String(buf, 0, left);
//        Logger.getDefault().log("result:"+value);
        return value;
    }
    
    public static void skipTagEnd (InputStream is) throws IOException {
        while ( is.read() != '>'); // skip the </..end..>
    }

    public static String readTextTag (InputStream is) throws IOException {
        readTagStart(is);
        String value = readValue(is);
        skipTagEnd(is);
        return value;
    }

    public static String encode(String str) {
        StringBuffer sb = new StringBuffer(str.length());
        String c;

        for (int i = 0; i < str.length(); i++) {
            c = str.substring(i, i + 1);
            if (c.equals("&")) {
                sb.append("&amp;");
            } else if (c.equals("<")) {
                sb.append("&lt;");
            } else if (c.equals(">")) {
                sb.append("&gt;");
            } else if (c.equals("\"")) {
                sb.append("&quot;");
            } else if (c.equals("'")) {
                sb.append("&apos;");
            } else {
                sb.append(c);
            }
        }

/*        if (!sb.toString().equals(str)) {
            Logger.getDefault().log("coded: " + str + " -> " + sb);
        }
*/
        return sb.toString();
    }

    public static String decode(String str) {
        StringBuffer sb = new StringBuffer(str.length());
        String c;
        int i = 0;
        int j = -1;

        while ((i = str.indexOf("&", j)) >= 0) {
            sb.append(str.substring(j + 1, i));
            j = str.indexOf(";", i + 1);
            if (j > i + 1) {
                c = str.substring(i + 1, j);
                if (c.equals("amp")) {
                    sb.append("&");
                } else if (c.equals("lt")) {
                    sb.append("<");
                } else if (c.equals("gt")) {
                    sb.append(">");
                } else if (c.equals("quot")) {
                    sb.append("\"");
                } else if (c.equals("apos")) {
                    sb.append("'");
                } else {
                    Logger.getDefault().log(Logger.WARNING, "Error! Cannot substitute character: " + c);
                    return "";
                }
            } else {
                Logger.getDefault().log(Logger.WARNING, "Error! End of substitution not found.");
                return "";
            }
        }

        sb.append(str.substring(j + 1));

/*        if (!sb.toString().equals(str)) {
            Logger.getDefault().log("decoded: " + str + " -> " + sb);
        }
*/
        return sb.toString();
    }

    public static String encodeString(String text) {
        return "<textValue>" + encode(text) + "</textValue>";
    }
    
    public static String decodeString(InputStream inputStream) throws IOException {
        return readTextTag(inputStream); //value
    }
    
    public static String encodeList(List list) {
        StringBuffer buf = new StringBuffer(100);
        buf.append("<list>");
        for (Iterator it = list.iterator(); it.hasNext();) {
            buf.append(encodeString((String) it.next()));
        }
        buf.append("</list>");
        return buf.toString();
    }
    
    public static void decodeList(InputStream inputStream, List list) throws IOException {
        readTagStart(inputStream); //<list>
        for (;;) {
            String str = readTagStart(inputStream); //<value>
            if (str.equals("/list")) break; // </list>
            list.add(readValue(inputStream));
            skipTagEnd(inputStream); // </value>
        }
    }
}
