/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.mdr.test;

import java.util.*;

import junit.extensions.*;
import junit.framework.*;

import org.openide.util.Lookup;

import org.netbeans.mdr.util.*;
import org.netbeans.mdr.NBMDRepositoryImpl;

import org.netbeans.mdr.persistence.memoryimpl.StorageFactoryImpl;
import org.netbeans.mdr.persistence.*;

import javax.jmi.reflect.*;
import javax.jmi.model.*;

/**
 * Multivalued index test case.
 */
public class UniqueIndexTest extends MDRTestCase {

    public static final String KEY_STR = "key_";
    public static final String VAL_STR = "value_";
    
    public UniqueIndexTest(String testName) {
        super (testName);
    }
    
    public static void main (String[] args) {
        junit.textui.TestRunner.run (suite ());
    }
    
    public static Test suite () {
        TestSuite suite = new TestSuite ();
        suite.addTestSuite (UniqueIndexTest.class);
        
        TestSetup setup = new TestSetup (suite) {
            public void setUp () {
//                org.netbeans.mdr.handlers.BaseObjectHandler.setDefaultClassLoader (this.getClass ().getClassLoader ());
            }
            public void tearDown () {
            }
        };
        return setup;
    }
    
    public void test () throws Exception {
        StorageFactoryImpl factory = new StorageFactoryImpl();
        Map map = new HashMap();
        map.put(StorageFactoryImpl.STORAGE_ID, "UniqueIndexTestStorage");
        Storage storage = factory.createStorage(map);
        MultivaluedIndex uniIndex = storage.createMultivaluedIndex("index_1", Storage.EntryType.STRING, Storage.EntryType.STRING, true);
        MultivaluedIndex index = storage.createMultivaluedIndex("index_2", Storage.EntryType.STRING, Storage.EntryType.STRING, false);
        MultivaluedOrderedIndex uniOrderedIndex = storage.createMultivaluedOrderedIndex("index_3", Storage.EntryType.STRING, Storage.EntryType.STRING, true);
        boolean failed;
        
        for (int x = 0; x < 10; x++) {
            String key = KEY_STR + x;
            for (int y = 0; y < 10; y++) {
                String value = VAL_STR + y;
                uniIndex.add(key, value);
                index.add(key, value);
                uniOrderedIndex.add(key, value);
            }
        }
        
        String key = KEY_STR + 2;
        String value = VAL_STR + 4;
        int pos = 4;
        
        failed = false;
        try {
            uniIndex.add(key, value);
        } catch (StorageBadRequestException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
        failed = false;
        try {
            uniOrderedIndex.add(key, value);
        } catch (StorageBadRequestException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
        // should not fail
        index.add(key, value);
        uniIndex.remove(key, value);
        uniOrderedIndex.remove(key, value);
        uniIndex.add(key, value);
        uniOrderedIndex.add(key, pos, value);
        uniOrderedIndex.replace(key, pos, value);
        
        failed = false;
        try {
            uniOrderedIndex.replace(key, 1, value);
        } catch (StorageBadRequestException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
        Collection coll_1 = uniIndex.getItems(key);
        coll_1.add(VAL_STR + 20);
        coll_1.remove(VAL_STR + 20);
        List coll_2 = uniOrderedIndex.getItemsOrdered(key);
        coll_2.add(VAL_STR + 20);
        coll_2.remove(VAL_STR + 20);
        
        failed = false;
        try {
            coll_2.set(7, value);
        } catch (RuntimeStorageException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
        ListIterator iter = coll_2.listIterator();
        iter.add(VAL_STR + 11);
        iter.next();
        iter.remove();
        iter.next();
        
        failed = false;
        try {
            iter.set(VAL_STR + 8);
        } catch (RuntimeStorageException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
        iter.next();
        failed = false;
        try {
            iter.add(VAL_STR + 11);
        } catch (RuntimeStorageException e) {
            failed = true;
        }
        if (!failed)
            fail();
        
    }

}