/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cardsui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.beans.PropertyEditor;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import org.netbeans.examples.cards.api.Card;
import org.netbeans.examples.cards.api.Game;
import org.netbeans.examples.cards.api.Rank;
import org.netbeans.examples.cards.api.Pile;
import org.netbeans.examples.cards.api.Suits;
import org.netbeans.examples.cards.propertyeditors.SuitPropertyEditor;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.NbBundle;
import org.openide.util.actions.Presenter;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Tim Boudreau
 */
public class CardNode extends AbstractNode { //BeanNode {
    //We want no icon at all, so just use a transparent image

    static final BufferedImage ICON = new BufferedImage(1, 1,
            BufferedImage.TYPE_INT_ARGB);


    static {
        Color c = new Color(255, 255, 255, 255);
        Graphics2D g = ICON.createGraphics();
        g.fillRect(0, 0, 1, 1);
        g.dispose();
    }

    public CardNode(Card card) { //throws Exception {
        super(Children.LEAF, Lookups.singleton(card));

    //If using BeanNode as superclass
    //super (card);
    }

    private Card getCard() {
        return (Card) getLookup().lookup(Card.class);
    //if using BeanNode as superclass
//        try {
//            InstanceCookie ck = (InstanceCookie) getCookie (
//                      InstanceCookie.class);
//            return (Card) ck.instanceCreate();
//        } catch (Exception e) {
//            throw new IllegalStateException (e);
//        }
    }

    public Image getIcon(int type) {
        return ICON;
    }

    public String getDisplayName() {
        Card c = getCard();
        Suits s = c.getSuit();
        if (s != null) { //will be for Joker cards
            return s.getUnicodeChar() + " " + c.getName();
        } else {
            return c.getName();
        }
    }

    public String getHtmlDisplayName() {
        Card c = getCard();
        String result = null;
        if (!c.isFaceDown()) {
            Suits s = c.getSuit();
            Color color = null;
            if (s != null) {
                color = s.getColor();
                if (Color.RED.equals(color)) {
                    result = "<font color=#FF0000>" +
                            getDisplayName();
                }
            }
        }
        return result;
    }

    public Action[] getActions(boolean popup) {
        return new Action[]{
                    new MoveToPileAction(getCard()),};
    }

    //If using BeanNode as a superclass, comment out this method and
    //the two classes below it - everything will be done automatically
    protected Sheet createSheet() {
        Sheet result = super.createSheet();
        Sheet.Set set = result.createPropertiesSet();
        set.put(new SuitProperty(getCard().getSuit()));
        set.put(new RankProperty(getCard().getRank()));
        result.put(set);
        return result;
    }

    private static final class SuitProperty extends PropertySupport.ReadOnly<Suits> {

        private final Suits suit;

        SuitProperty(Suits suit) {
            super("suit",
                    Suits.class, NbBundle.getMessage(SuitProperty.class,
                    "LBL_SUIT"), NbBundle.getMessage(SuitProperty.class,
                    "DESC_SUIT"));
            this.suit = suit;
        }

        public PropertyEditor getPropertyEditor() {
            return new SuitPropertyEditor();
        }

        @Override
        public Suits getValue() throws IllegalAccessException, InvocationTargetException {
            return suit;
        }
    }

    private static final class RankProperty extends PropertySupport.ReadOnly<String> {

        private final Rank rank;

        RankProperty(Rank rank) {
            super("rank",
                    String.class, NbBundle.getMessage(SuitProperty.class,
                    "LBL_RANK"), NbBundle.getMessage(SuitProperty.class,
                    "DESC_RANK"));
            this.rank = rank;
        }

        public String getValue() throws IllegalAccessException,
                InvocationTargetException {
            return rank.toString();
        }
    }

    private static final class MoveToPileAction extends AbstractAction
            implements Presenter.Popup {

        private final Card card;
        private static final String PROP_PILE = "pile"; //NOI18N

        MoveToPileAction(Card card) {
            this.card = card;
        }

        public JMenuItem getPopupPresenter() {
            JMenu result = new JMenu();
            result.setText(NbBundle.getMessage(MoveToPileAction.class,
                    "LBL_MOVE_TO_PILE"));

            //Get the list of all Piles that have at least one card
            //in them from the Deck
            List<Pile> piles = card.getDeck().allPiles();

            //Get the Game object (see GameChildren) that can supply
            //a String name for each pile
            Game game = card.getDeck().getGame();
            for (Pile pile : piles) {
                JMenuItem item = new JMenuItem();
                item.setText(game.nameOf(pile));
                //Store the Pile object this menu item represents,
                //so we can fetch it in actionPerformed()
                item.putClientProperty(PROP_PILE, pile);
                if (pile == card.getPile() || !pile.canAdd(card)) {
                    item.setEnabled(false);
                }
                item.addActionListener(this);
                result.add(item);
            }
            return result;
        }

        public void actionPerformed(ActionEvent ae) {
            JMenuItem item = (JMenuItem) ae.getSource();
            Pile pile = (Pile) item.getClientProperty(PROP_PILE);
            pile.add(card);
        }
    }
}
