/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cardsui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.ListView;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Utilities;

/**
 * A Detail View that shows the child nodes of the selected Node in a
 * parent panel with an explorer manager.  This component does the
 * same thing BeanTreeView or any of the standard explorer view
 * components in org.openide.explorer.view do - find an ancestor
 * which is an instance of ExplorerManager
 *
 * @author Tim Boudreau
 */
public class CardsDetailView extends JPanel implements ExplorerManager.Provider,
        PropertyChangeListener {

    private final ExplorerManager mgr = new ExplorerManager();

    public CardsDetailView() {
        setLayout(new BorderLayout());
        ListView view = new ListView();
        add(view, BorderLayout.CENTER);
        if (Utilities.isWindows()) {
            view.getViewport().getView().setFont(new Font("SansSerif",
                    Font.PLAIN, 12));
        }
    }

    public ExplorerManager getExplorerManager() {
        return mgr;
    }
    private ExplorerManager parentManager = null;

    @Override
    public void addNotify() {
        super.addNotify();
        ExplorerManager.Provider provider = (ExplorerManager.Provider) SwingUtilities.getAncestorOfClass(ExplorerManager.Provider.class,
                this);

        if (provider != null) {
            parentManager = provider.getExplorerManager();
            parentManager.addPropertyChangeListener(this);
            updateRootNode();
        }
    }

    @Override
    public void removeNotify() {
        if (parentManager != null) {
            parentManager.removePropertyChangeListener(this);
        }
        super.removeNotify();
    }

    private void updateRootNode() {
        Node[] n = parentManager.getSelectedNodes();
        if (n.length == 0) {
            mgr.setRootContext(new AbstractNode(Children.LEAF));
        } else {
            //don't worry about multiple selection
            Node node = n[0];
            Node iconFilterNode = new FilterNode(node,
                    new IconFilterChildren(node));

            mgr.setRootContext(iconFilterNode);
        }
    }

    public void propertyChange(PropertyChangeEvent e) {
        if (ExplorerManager.PROP_SELECTED_NODES.equals(e.getPropertyName())) {
            updateRootNode();
        }
    }

    private static class IconFilterChildren extends FilterNode.Children {

        public IconFilterChildren(Node original) {
            super(original);
        }

        protected Node[] createNodes(Node childNodeFromOrig) {
            Node toClone = (Node) childNodeFromOrig;
            Node result = new IconFilterNode(toClone);
            return new Node[]{result};
        }
    }
    private static final String ALTERNATE_ICON_PATH =
            "org/netbeans/examples/cardsui/alternateIcon.png";

    private static final class IconFilterNode extends FilterNode {

        IconFilterNode(Node orig) {
            super(orig);
        }

        @Override
        public Image getIcon(int beanInfoIconType) {
            return ImageUtilities.loadImage(ALTERNATE_ICON_PATH);
        }
    }
}
