/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.ahrefhyperlink;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import javax.swing.JEditorPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import org.netbeans.editor.BaseDocument;
import org.netbeans.editor.TokenItem;
import org.netbeans.editor.ext.html.HTMLSyntaxSupport;
import org.netbeans.editor.ext.html.HTMLTokenContext;
import org.netbeans.lib.editor.hyperlink.spi.HyperlinkProvider;
import org.netbeans.modules.editor.NbEditorUtilities;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.URLMapper;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.RequestProcessor;

/**
 *
 * @author Geertjan Wielenga
 */
public class AHrefHyperlinkProvider implements HyperlinkProvider {

    private static String AHREF_IDENTIFIER = "href";
    private Document lastDocument;
    private int startOffset;
    private int endOffset;
    private String identifier;

    /** Creates a new instance of HyperlingProvider */
    public AHrefHyperlinkProvider() {
        lastDocument = null;

    }

    public boolean isHyperlinkPoint(Document doc, int offset) {

        // Work only with org.netbeans.editor.BaseDocument:
        if (!(doc instanceof BaseDocument)) {
            return false;
        }

        BaseDocument bdoc = (BaseDocument) doc;
        JTextComponent target = org.netbeans.editor.Utilities.getFocusedComponent();

        // Work only with the open editor and the editor has to be the active component:
        if ((target == null) || (target.getDocument() != bdoc)) {
            return false;
        }

        // Work only if the support is HTMLSyntaxSupport:
        if (!(bdoc.getSyntaxSupport() instanceof HTMLSyntaxSupport)) {
            return false;
        }

        HTMLSyntaxSupport sup = (HTMLSyntaxSupport) bdoc.getSyntaxSupport();

        try {
            // Get the token on the offset:
            TokenItem token = sup.getTokenChain(offset, offset + 1);

            // We are interested only in the opening tag:
            if ((token != null) && (token.getTokenID().getNumericID() == HTMLTokenContext.VALUE_ID)) {
                TokenItem firstToken = token;

                // find the nearest argument or open tag token
                while ((token != null) && (token.getTokenID().getNumericID() != HTMLTokenContext.ARGUMENT_ID) && (token.getTokenID().getNumericID() != HTMLTokenContext.TAG_OPEN_ID)) {
                    token = token.getPrevious();
                }

                // if the token is argument ant it is wicket identifier, then remember all information
                // and we find it
                if ((token != null) && (token.getTokenID().getNumericID() == HTMLTokenContext.ARGUMENT_ID) && token.getImage().equals(AHREF_IDENTIFIER)) {
                    //remember the identifier and the possitions
                    identifier = firstToken.getImage();

                    if ((identifier.startsWith("nbdocs", 1) == false) && (identifier.startsWith("#seealso", 1) == false)) {
                        //remove " or '
                        identifier = identifier.substring(0, identifier.length() - 1).substring(1);
                        // set the real start and end
                        // the start of the identifier is after char " or '


                        startOffset = firstToken.getOffset() + 1;
                        endOffset = (firstToken.getOffset() + firstToken.getImage().length()) - 2;
                        // remember this document
                        lastDocument = bdoc;

                    } else {
                        return false;
                    }

                    //System.out.println("identifier: " + identifier);
                    return true;
                }
            }
        } catch (BadLocationException ex) {
            ex.printStackTrace();
        }


        return false;
    }

    public int[] getHyperlinkSpan(Document doc, int offset) {
        // first check, we need to work with BaseDocument
        if (!(doc instanceof BaseDocument)) {
            return null;
        }

        BaseDocument bdoc = (BaseDocument) doc;
        JTextComponent target = org.netbeans.editor.Utilities.getFocusedComponent();

        // work only with the open editor and the editor has to be active component and
        // the document has to be the same as was in isHyperlink method
        if ((target == null) || (lastDocument != bdoc)) {
            return null;
        }

        // return the possition, which where counted in method isHyperlink.
        return new int[]{startOffset, endOffset};
    }

    public void performClickAction(Document doc, int offset) {
        // first check, we need to work with BaseDocument
        if (!(doc instanceof BaseDocument)) {
            return;
        }

        BaseDocument bdoc = (BaseDocument) doc;
        JTextComponent target = org.netbeans.editor.Utilities.getFocusedComponent();

        // work only with the open editor and the editor has to be active component and
        // the document has to be the same as was in isHyperlink method
        if ((target == null) || (lastDocument != bdoc)) {
            return;
        }

        OpenCssThread run = new OpenCssThread(bdoc, identifier);
        RequestProcessor.getDefault().post(run);
    }

    private class OpenCssThread implements Runnable {

        private String fqn;
        private BaseDocument doc;
        private String identifier;

        public OpenCssThread(BaseDocument doc, String identifier) {
            super();
            this.doc = doc;
            this.identifier = identifier;
        }

        public void run() {

            FileObject fo = NbEditorUtilities.getFileObject(doc);
            FileObject foCss = null;

            //Here we're working out whether we're dealing with a relative link or not:
            if (identifier.contains("/")) {
                String fullPath = fo.getPath();
                try {
                    URL f = new File(fullPath).toURI().resolve(identifier).toURL();
                    foCss = URLMapper.findFileObject(f);
                } catch (MalformedURLException ex) {
                    ex.printStackTrace();
                }
            } else {
                foCss = fo.getParent().getFileObject(identifier);
            }

            DataObject dObject;

            try {
                dObject = DataObject.find(foCss);

                final EditorCookie.Observable ec = (EditorCookie.Observable) dObject.getCookie(EditorCookie.Observable.class);

                if (ec != null) {
                    org.netbeans.editor.Utilities.runInEventDispatchThread(new Runnable() {

                        public void run() {
                            final JEditorPane[] panes = ec.getOpenedPanes();

                            if ((panes != null) && (panes.length > 0)) {
                                setPosition(panes[0], identifier);
                            } else {
                                ec.addPropertyChangeListener(new PropertyChangeListener() {

                                    public void propertyChange(PropertyChangeEvent evt) {
                                        if (EditorCookie.Observable.PROP_OPENED_PANES.equals(evt.getPropertyName())) {
                                            final JEditorPane[] panes = ec.getOpenedPanes();
                                            if ((panes != null) && (panes.length > 0)) {
                                                setPosition(panes[0], identifier);
                                            }
                                            ec.removePropertyChangeListener(this);
                                        }
                                    }
                                });
                            }

                            ec.open();
                        }
                    });
                }
            } catch (DataObjectNotFoundException ex) {
                ex.printStackTrace();
            }
        }

        //Here we specify where the cursor will land:
        private void setPosition(JEditorPane pane, String identifier) {

            try {
                //The whole text:
                String text = pane.getDocument().getText(0, pane.getDocument().getLength() - 1);
                //The place where we want the cursor to be:
                int index = text.indexOf("<h2>");

                //If we can find it, we place the cursor there:
                if (index > 0) {
                    pane.setCaretPosition(index);
                } else {
                    pane.setCaretPosition(index);
                }

            } catch (BadLocationException ex) {
                ex.printStackTrace();
            }
        }
    }
}
