package org.netbeans.modules.myfirstpalette;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.geom.AffineTransform;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import org.netbeans.modules.myfirstpalette.palette.MyItemData;
import org.openide.ErrorManager;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.lookup.Lookups;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * Top component which displays something.
 */
final class MyTopComponent extends TopComponent implements DragGestureListener {
    
    private static MyTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "org/netbeans/modules/myfirstpalette/icon.gif";
    private Map displayers = new HashMap();
    private static final String PREFERRED_ID = "MyTopComponent";
    
    private MyTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(MyTopComponent.class, "CTL_MyTopComponent"));
        setToolTipText(NbBundle.getMessage(MyTopComponent.class, "HINT_MyTopComponent"));
        setIcon(Utilities.loadImage(ICON_PATH, true));
        
        associateLookup( Lookups.fixed( new Object[] { PaletteSupport.createPalette() } ) );
        
        dropPanel.setDropTarget( new DropTarget( dropPanel, new DropTargetListener() {
            public void dragEnter(DropTargetDragEvent dtde) {
                doDragOver( dtde );
            }
            public void dragExit(DropTargetEvent dte) {
            }
            public void dragOver(DropTargetDragEvent dtde) {
                doDragOver( dtde );
            }
            public void drop(DropTargetDropEvent dtde) {
                doDrop( dtde );
            }
            public void dropActionChanged(DropTargetDragEvent dtde) {
            }
        }) );
        
        
    }
    
    private void doDragOver( DropTargetDragEvent dtde ) {
        
        MyItemData data = null;
        try {
            data = (MyItemData) dtde.getTransferable().getTransferData
                    (PaletteSupport.MY_DATA_FLAVOR);
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (UnsupportedFlavorException ex) {
            ex.printStackTrace();
        }
        
        ItemDataDisplayer displayer = new ItemDataDisplayer( data );
        Image dragImage = displayer.getData().getBigImage();
        
        Graphics2D g2 = (Graphics2D) dropPanel.getGraphics();
        Rectangle visRect = dropPanel.getVisibleRect();
        dropPanel.paintImmediately(visRect.x, visRect.y, visRect.width, visRect.height);
        g2.drawImage(dragImage,
                AffineTransform.getTranslateInstance(dtde.getLocation().getX(),
                dtde.getLocation().getY()),
                null);
        
        dtde.acceptDrag( DnDConstants.ACTION_COPY_OR_MOVE );
    }
    
    private void doDrop( DropTargetDropEvent dtde ) {
        
        //first check if we support this type of data:
        if( !dtde.isDataFlavorSupported( PaletteSupport.MY_DATA_FLAVOR ) ) {
            dtde.rejectDrop();
        }
        
        //accept the drop so that we can access the Transferable:
        dtde.acceptDrop( DnDConstants.ACTION_COPY_OR_MOVE );
        MyItemData data = null;
        try {
            
            //get the dragged data from the transferable:
            data = (MyItemData) dtde.getTransferable().getTransferData
                    (PaletteSupport.MY_DATA_FLAVOR);
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (UnsupportedFlavorException ex) {
            ex.printStackTrace();
        }
        
        dtde.dropComplete( null != data );
        if( null != data ) {
            
            //find the displayer for the data,
            //for which you will need to declare private Map displayers = new HashMap()
            //at the top of the file:
            ItemDataDisplayer displayer = (ItemDataDisplayer)displayers.get( data );
            if( null == displayer ) {
                
                //its a drop from the palette and the data is not in our list yet:
                displayer = new ItemDataDisplayer( data );
                
                //This is where the drag gesture recognizer belongs:
                DragSource.getDefaultDragSource().createDefaultDragGestureRecognizer( displayer, DnDConstants.ACTION_COPY_OR_MOVE, this );
                
                dropPanel.add( displayer );
                displayer.setSize( displayer.getPreferredSize() );
                displayers.put( data, displayer );
            }
            
            //update displayer's location from the drop point:
            displayer.setLocation( dtde.getLocation() );
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        dropPanel = new javax.swing.JPanel();

        dropPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        org.jdesktop.layout.GroupLayout dropPanelLayout = new org.jdesktop.layout.GroupLayout(dropPanel);
        dropPanel.setLayout(dropPanelLayout);
        dropPanelLayout.setHorizontalGroup(
            dropPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 378, Short.MAX_VALUE)
        );
        dropPanelLayout.setVerticalGroup(
            dropPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 100, Short.MAX_VALUE)
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(dropPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(dropPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(187, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel dropPanel;
    // End of variables declaration//GEN-END:variables
    
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized MyTopComponent getDefault() {
        if (instance == null) {
            instance = new MyTopComponent();
        }
        return instance;
    }
    
    /**
     * Obtain the MyTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized MyTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find My component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof MyTopComponent) {
            return (MyTopComponent)win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }
    
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }
    
    public void componentOpened() {
        // TODO add custom code on component opening
    }
    
    public void componentClosed() {
        // TODO add custom code on component closing
    }
    
    /** replaces this in object stream */
    public Object writeReplace() {
        return new ResolvableHelper();
    }
    
    protected String preferredID() {
        return PREFERRED_ID;
    }
    
    public void dragGestureRecognized(DragGestureEvent dge) {
        //first check which object is to be dragged:
        if( dge.getComponent() instanceof ItemDataDisplayer ) {
            ItemDataDisplayer displayer = (ItemDataDisplayer)dge.getComponent();
            //get the data from the displayer and create a new transferable for dragging:
            dge.startDrag( null, new ItemDataTransferable(displayer.getData()) );
        }
    }
    
    private static class ItemDataTransferable implements Transferable {
        
        private MyItemData data;
        
        public ItemDataTransferable( MyItemData data ) {
            this.data = data;
        }
        
        public DataFlavor[] getTransferDataFlavors() {
            return new DataFlavor[] { PaletteSupport.MY_DATA_FLAVOR };
        }
        
        public boolean isDataFlavorSupported(DataFlavor flavor) {
            return PaletteSupport.MY_DATA_FLAVOR.equals( flavor );
        }
        
        public Object getTransferData(DataFlavor flavor)
        throws UnsupportedFlavorException, IOException {
            if( !isDataFlavorSupported( flavor ) ) {
                throw new UnsupportedFlavorException( flavor );
            }
            return data;
        }
        
    }
    
    final static class ResolvableHelper implements Serializable {
        private static final long serialVersionUID = 1L;
        public Object readResolve() {
            return MyTopComponent.getDefault();
        }
    }
    
}
