/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.nbtidyintegration;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.openide.cookies.LineCookie;
import org.openide.loaders.DataObject;
import org.openide.text.Line;
import org.openide.windows.OutputEvent;
import org.openide.windows.OutputListener;

/**
 * Listener class for events on the output lines.
 * With a listener provided to a line of output, it
 * will act simi lar as a hyperlink.
 */
public class TidyOutputListener implements OutputListener {

    public static String PATTERN_STRING =
            "\\s*line\\s*([0-9]+)\\s*column\\s*([0-9]+)\\s*-\\s*([^ :]+):\\s+(.+)";
    public static Pattern PATTERN = Pattern.compile(PATTERN_STRING);
    DataObject dataObject;

    public TidyOutputListener(DataObject dataObject) {
        this.dataObject = dataObject;
    }

    /** Open Line in HTML file that caused the error.*/
    public void outputLineAction(OutputEvent outputEvent) {
        String lineString = outputEvent.getLine();
        Matcher matcher = PATTERN.matcher(lineString);
        if (matcher.matches()) {
            int lineNumber = Integer.parseInt(matcher.group(1));
            int columnNumber = Integer.parseInt(matcher.group(2));
            LineCookie lc = (LineCookie) dataObject.getCookie(LineCookie.class);
            Line l = lc.getLineSet().getOriginal(lineNumber - 1);
            l.show(Line.SHOW_GOTO, columnNumber - 1);
        }
    }

    /** Nothing to do here.*/
    public void outputLineCleared(OutputEvent outputEvent) {
    }

    /** Nothing to do here.*/
    public void outputLineSelected(OutputEvent outputEvent) {
    }
}

