/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.xmlpseudonavigator;

import java.io.IOException;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ListModel;
import org.netbeans.api.pseudonavigator.ListModelProvider;
import org.openide.filesystems.FileObject;

/**
 * Provides ListModels for .xml and .html files
 *
 * @author Tim Boudreau
 */
public class XmlListModelProvider implements
        ListModelProvider<XmlListModelItem> {

    private Reference<XmlFileListModel> last;

    public ListModel getListModel(FileObject file) {
        XmlFileListModel result = null;
        //First check if this is the same file set the last time we had
        //a non null result, if it has not been garbage collected
        XmlFileListModel current = last == null ? null : last.get();
        if (current != null && file.equals(current.getFile())) {
            result = current;
        }
        if (result == null) {
            result = new XmlFileListModel(file);
            last = new WeakReference<XmlFileListModel>(result);
        }
        return result;
    }

    public void selected(XmlListModelItem item) {
        try {
            item.selected();
        } catch (IOException ex) {
            Logger.getLogger("global").log(Level.SEVERE, //NOI18N
                    ex.getMessage(), ex);
        }
    }
}
