/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.web.wicket.framework;

import java.awt.Component;
import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.netbeans.modules.web.spi.webmodule.FrameworkConfigurationPanel;

import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;

/**
 * Configuration panel for Wicket.
 * 
 * @author Radko Najman
 */
public final class WicketConfigurationPanel implements FrameworkConfigurationPanel {

    private WizardDescriptor wizardDescriptor;
    private WicketConfigurationPanelVisual component;
    private final Set/*<ChangeListener>*/ listeners = new HashSet(1);

    public Component getComponent() {
        return getTypedComponent();
    }

    private WicketConfigurationPanelVisual getTypedComponent() {
        if (component == null) {
            component = new WicketConfigurationPanelVisual(this);
        }
        return component;
    }

    public HelpCtx getHelp() {
        return new HelpCtx(WicketConfigurationPanel.class);
    }

    public boolean isValid() {
        return getTypedComponent().valid();
    }

    public final void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    public final void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Iterator it;
        synchronized (listeners) {
            it = new HashSet(listeners).iterator();
        }
        ChangeEvent ev = new ChangeEvent(this);
        while (it.hasNext()) {
            ((ChangeListener) it.next()).stateChanged(ev);
        }
    }

    public void readSettings(Object settings) {
        wizardDescriptor = (WizardDescriptor) settings;

        // XXX hack, TemplateWizard in final setTemplateImpl() forces new wizard's title
        // this name is used in NewProjectWizard to modify the title
        Object substitute = ((JComponent) component).getClientProperty("NewProjectWizard_Title"); // NOI18N
        if (substitute != null) {
            wizardDescriptor.putProperty("NewProjectWizard_Title", substitute); // NOI18N
        }
    }

    public void storeSettings(Object settings) {
        WizardDescriptor d = (WizardDescriptor) settings;
        d.putProperty("NewProjectWizard_Title", null); // NOI18N
    }

    public void enableComponents(boolean enable) {
        getTypedComponent().enableComponents(enable);
    }

    public String getServletName() {
        return getTypedComponent().getServletName();
    }

    public void setServletName(String name) {
        getTypedComponent().setServletName(name);
    }

    public String getURLPattern() {
        return getTypedComponent().getURLPattern();
    }

    public void setURLPattern(String pattern) {
        getTypedComponent().setURLPattern(pattern);
    }

    public String getAppClass() {
        return getTypedComponent().getAppClass();
    }

    public void setAppClass(String resource) {
        getTypedComponent().setAppClass(resource);
    }

    public String getHomePage() {
        return getTypedComponent().getHomePage();
    }

    public void setHomePage(String resource) {
        getTypedComponent().setHomePage(resource);
    }

    public String getDummyPojo() {
        return getTypedComponent().getDummyPojo();
    }

    public void setDummyPojo(String resource) {
        getTypedComponent().setDummyPojo(resource);
    }

    public String getMainPackage() {
        return getTypedComponent().getMainPackage();
    }

    public void setMainPackage(String resource) {
        getTypedComponent().setMainPackage(resource);
    }

    public boolean isExampleHeader() {
        return getTypedComponent().isExampleHeader();
    }

    public File getWicketFolder() {
        return getTypedComponent().getWicketFolder();
    }

    void setErrorMessage(String message) {
        // this can be called early, even before readSettings()
        if (wizardDescriptor == null) {
            return;
        }
        if (message == null) {
            message = ""; // NOI18N
        }
        wizardDescriptor.putProperty("WizardPanel_errorMessage", message);
    }
}
