/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.myorg.myeditor;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Date;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import org.myorg.myapi.APIObject;
import org.openide.ErrorManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node.Property;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.Utilities;
import org.openide.util.WeakListeners;
import org.openide.util.actions.Presenter;
import org.openide.util.lookup.Lookups;

/**
 * A simple Node subclass.
 *
 * @author Tim Boudreau
 */
public class MyNode extends AbstractNode implements PropertyChangeListener {

    public MyNode(APIObject obj) {
        super (new MyChildren(), Lookups.singleton(obj));
        setDisplayName ( "APIObject " + obj.getIndex());
        obj.addPropertyChangeListener(WeakListeners.propertyChange(this, obj));
    }
    
    public MyNode() {
        super (new MyChildren());
        setDisplayName ("Root");
    }

    //Third and final version of tutorial code
    public String getHtmlDisplayName() {
        APIObject obj = (APIObject) getLookup().lookup (APIObject.class);
        if (obj != null) {
            return "<font color='!textText'>APIObject " + obj.getIndex() + "</font>" +
                    "<font color='!controlShadow'><i>" + obj.getDate() + "</i></font>";
        } else {
            return null;
        }
    }
    
    public Image getIcon (int type) {
        return Utilities.loadImage ("org/myorg/myeditor/icon.gif");
    }

    public Image getOpenedIcon(int i) {
        return getIcon (i);
    }
    
    public Action[] getActions (boolean popup) {
        return new Action[] { new MyAction() };
    }

    //Final Action code in tutorial
    private class MyAction extends AbstractAction implements Presenter.Popup {
        public MyAction () {
            putValue (NAME, "Do Something");
        }
        
        public void actionPerformed(ActionEvent e) {
            APIObject obj = (APIObject) getLookup().lookup (APIObject.class);
            JOptionPane.showMessageDialog(null, "Hello from " + obj);
        }

        public JMenuItem getPopupPresenter() {
            JMenu result = new JMenu("Submenu");  //remember JMenu is a subclass of JMenuItem
            result.add (new JMenuItem(this));
            result.add (new JMenuItem(this));
            return result;
        }
    }

    protected Sheet createSheet() {
        Sheet sheet = Sheet.createDefault();
        Sheet.Set set = sheet.createPropertiesSet();
        APIObject obj = (APIObject) getLookup().lookup (APIObject.class);
        try {
            Property indexProp = new PropertySupport.Reflection(obj, 
                    Integer.class, "getIndex", null);
            
            PropertySupport.Reflection dateProp = 
                    new PropertySupport.Reflection(obj, Date.class, "date");
            dateProp.setPropertyEditorClass(DatePropertyEditor.class);
            
            indexProp.setName("index");
            dateProp.setName ("date");
            
            
            set.put (indexProp);
            set.put (dateProp);
            
        } catch (NoSuchMethodException ex) {
            ErrorManager.getDefault().notify (ex);
        }
        sheet.put(set);
        return sheet;
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if ("date".equals(evt.getPropertyName())) {
            this.fireDisplayNameChange(null, getDisplayName());
        }
    }
}
