/*
DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.


The contents of this file are subject to the terms of either the GNU
General Public License Version 2 only ("GPL") or the Common
Development and Distribution License("CDDL") (collectively, the
"License"). You may not use this file except in compliance with the
License. You can obtain a copy of the License at
http://www.netbeans.org/cddl-gplv2.html
or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
specific language governing permissions and limitations under the
License.  When distributing the software, include this License Header
Notice in each file and include the License file at
nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
particular file as subject to the "Classpath" exception as provided
by Sun in the GPL Version 2 section of the License file that
accompanied this code. If applicable, add the following below the
License Header, with the fields enclosed by brackets [] replaced by
your own identifying information:
"Portions Copyrighted [year] [name of copyright owner]"

Contributor(s):

The Original Software is NetBeans. The Initial Developer of the Original
Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
Microsystems, Inc. All Rights Reserved.

If you wish your version of this file to be governed by only the CDDL
or only the GPL Version 2, indicate your decision by adding
"[Contributor] elects to include this software in this distribution
under the [CDDL or GPL Version 2] license." If you do not indicate a
single choice of license, a recipient has the option to distribute
your version of this file under either the CDDL, the GPL Version 2 or
to extend the choice of license to its licensees as provided above.
However, if you add GPL Version 2 code and therefore, elected the GPL
Version 2 license, then the option applies only if the new code is
made subject to such option by the copyright holder.
*/
package org.netbeans.modules.diskexplorer;

import java.awt.BorderLayout;
import java.awt.Image;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.ActionMap;
import javax.swing.text.DefaultEditorKit;
import org.openide.ErrorManager;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.explorer.view.BeanTreeView;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Utilities;
import org.openide.windows.OutputWriter;
import org.openide.windows.IOProvider;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.Mode;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/** 
 * A simple explorer component.
 *
 * @author  Rich Unger
 */
public class DiskExplorerTopComponent extends CloneableTopComponent implements ExplorerManager.Provider
{
    private static final String MODE = "explorer"; // NOI18N

    /** A hint to the window system for generating a unique id */
    private static final String PREFERRED_ID = "disk_explorer"; // NOI18N
    
    private transient final ExplorerManager m_manager;
    private transient Node m_rootNode;
    private transient final BeanTreeView m_btv;


    public static DiskExplorerTopComponent getInstance()
    {
        // look for an open instance
        Iterator opened = TopComponent.getRegistry().getOpened().iterator();
        while (opened.hasNext())
        {
            Object tc = opened.next();
            if (tc instanceof DiskExplorerTopComponent)
            {
                return (DiskExplorerTopComponent)tc;
            }
        }
        
        // none found, make a new one
        return new DiskExplorerTopComponent();
    }

    public DiskExplorerTopComponent() 
    {
        super();
        m_manager = new ExplorerManager();
        m_btv = new BeanTreeView();
        m_btv.setDragSource(true);
        m_btv.setRootVisible(false);
        
        ActionMap map = getActionMap();
        map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(m_manager));
        map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(m_manager));
        map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(m_manager));
        map.put("delete", ExplorerUtils.actionDelete(m_manager, true)); // NOI18N

        setLayout(new BorderLayout());
        add(m_btv, BorderLayout.CENTER);
        
        associateLookup( ExplorerUtils.createLookup(m_manager, map) );
        
        m_rootNode = new RootNode();
        m_manager.setRootContext(m_rootNode);
    }
    
    public Image getIcon()
    {
        return Utilities.loadImage ("org/netbeans/modules/diskexplorer/resources/explorer16.gif");
    }
    
    public void open()
    {
        Mode m = WindowManager.getDefault().findMode(MODE);
        m.dockInto(this);
        super.open();
    }
    
    public ExplorerManager getExplorerManager() {
        return m_manager;
    }
    
    public int getPersistenceType() { 
        return TopComponent.PERSISTENCE_ALWAYS;
    } 

    protected String preferredID() { 
        return PREFERRED_ID;
    } 
    
    /** To enable cloning of the top component */
    protected CloneableTopComponent createClonedObject()
    {
        return new DiskExplorerTopComponent();
    }
    
    protected void componentActivated() {
        ExplorerUtils.activateActions(m_manager, true);
    }
    
    protected void componentDeactivated() {
        ExplorerUtils.activateActions(m_manager, false);
    }

    private void log(String str) { 
        OutputWriter out = IOProvider.getDefault().getStdOut();
        out.println(str);
        out.flush();
    }
    
    private static class RootNode extends AbstractNode
    {
        public RootNode()
        {
            super(new RootChildren());
        }
    }
    
    private static class RootChildren extends Children.Array
    {
        public RootChildren()
        {
            super();
        }

        protected java.util.Collection initCollection() {
            File[] driveRoots = File.listRoots();
            ArrayList result = new ArrayList(driveRoots.length);
            
            for (int i=0; i < driveRoots.length; i++)
            {
                File f = FileUtil.normalizeFile(driveRoots[i]);
                FileObject fo = FileUtil.toFileObject(f);
                if (fo == null) { // http://openide.netbeans.org/servlets/ReadMsg?list=dev&msgNo=17126
                    continue;
                }
                try
                {
                    DataObject dobj = DataObject.find(fo);
                    result.add(dobj.getNodeDelegate().cloneNode());
                }
                catch (DataObjectNotFoundException ex)
                {
                    ErrorManager.getDefault().notify(ex);
                }
            }
            
            return result;
        }
    }

    private static final long serialVersionUID = -4406860213735839129L;

}
