/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.editoraction;

import java.awt.Toolkit;
import javax.swing.JEditorPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.Document;
import org.openide.cookies.EditorCookie;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CookieAction;

/**
 * An example action for the NetBeans IDE that works on open text editor windows.
 * To customize, change performAction(Document,Caret); getName; and if you wish,
 * iconResource() too.
 */
public class ExampleTextAction extends CookieAction {

    /** Do something with an open document.
    * This is the part you should write according to your needs.
    * Please use javax.swing.text.* API.
    * @param d a document open in the editor pane
    * @param c the caret giving cursor position, selection, etc.
    */
    protected void performAction (Document d, Caret c) throws BadLocationException {
        int dot = c.getDot ();
        int mark = c.getMark ();
        if (dot == mark) {
            // No selection: insert [[]] at point and move cursor to center
            d.insertString (dot, "[[]]", null);
            c.setDot (dot + 2);
        } else {
            // Selection: wrap it in [[...]] and clear selection
            int start = Math.min (dot, mark);
            int end = Math.max (dot, mark);
            d.insertString (end, "]]", null);
            d.insertString (start, "[[", null);
            c.setDot (end + 4);
        }
    }

    public String getName () {
        return "Example Action";
    }

    /* Implement if you want an icon (e.g. for non-context menu item/toolbar button):
    protected String iconResource () {
      return "resource/path/to/icon.gif";
    }
    */

    // --- BOILERPLATE BELOW THIS POINT ---
    protected Class[] cookieClasses () {
        return new Class[] { EditorCookie.class };
    }
    protected int mode () {
        return MODE_EXACTLY_ONE;
    }
    protected boolean enable (Node[] nodes) {
        if (! super.enable (nodes)) return false;
        EditorCookie ed = (EditorCookie) nodes[0].getCookie (EditorCookie.class);
        return ed.getOpenedPanes () != null;
    }
    protected void performAction (Node[] nodes) {
        EditorCookie ed = (EditorCookie) nodes[0].getCookie (EditorCookie.class);
        JEditorPane pane = ed.getOpenedPanes ()[0];
        try {
            performAction (pane.getDocument (), pane.getCaret ());
        } catch (BadLocationException ble) {
            Toolkit.getDefaultToolkit ().beep ();
        }
    }
    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
    }
}
