/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import org.openide.ErrorManager;
import org.openide.execution.ExecutionEngine;
import org.openide.execution.ExecutorTask;
import org.openide.execution.NbProcessDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.MapFormat;
import org.openide.util.NbBundle;
import org.openide.windows.InputOutput;

public class ExternalPlayer implements Player {
    
    private static final String TAG_AUFILE_PLAIN = "aufile";
    public static final String TAG_AUFILE = "{" + TAG_AUFILE_PLAIN + "}";
        
    private NbProcessDescriptor playerCommand;
    private String label;
    
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    
    public ExternalPlayer(NbProcessDescriptor playerCommand) {
        this(playerCommand, null);
    }
    
    public ExternalPlayer(NbProcessDescriptor playerCommand, String label) {
        this.playerCommand = playerCommand;
        this.label = label;
    }

    public String getLabel() {
        return label;
    }
    
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }
    
    public NbProcessDescriptor getPlayerCommand() {
        return playerCommand;
    }
    
    public void setPlayerCommand(NbProcessDescriptor d) {
        NbProcessDescriptor old = playerCommand;
        playerCommand = d;
        label = null; // obsoleted
        pcs.firePropertyChange("playerCommand", old, d);
    }
    
    public ExecutorTask play(String displayName, FileObject fo, ScoreCookie s) throws IOException {
        if (fo == null) {
            IOException ioe = new IOException("No .au in " + displayName);
            ErrorManager.getDefault().annotate(ioe, NbBundle.getMessage(ExternalPlayer.class, "EXC_No_AU_file_found"));
            throw ioe;
        }
        final File f = FileUtil.toFile(fo);
        if (f == null) {
            IOException ioe = new IOException("No .au in " + displayName);
            ErrorManager.getDefault().annotate(ioe, NbBundle.getMessage(ExternalPlayer.class, "EXC_file_must_be_local"));
            throw ioe;
        }
        // XXX for now there is no other apparent way to run an NbProcessDescriptor well
        // besides hacking around with subclassing ProcessExecutor and passing it dummy args
        // Could also make a custom ExecutorTask, but then no Execution View etc.
        // This does not produce the correct return value (ProcessExecutor makes a proxy ET for this)
        final Process p = playerCommand.exec(new MyFormat(f));
        // XXX I18N
        return new ProcessTask("Playing " + displayName, p);
    }
    
    private static class MyFormat extends MapFormat {
        MyFormat(File aufile) {
            super(new HashMap(1));
            getMap().put(TAG_AUFILE_PLAIN, aufile.getAbsolutePath());
        }
    }
    
    private static final class ProcessTask extends ExecutorTask {
        
        private final Process p;
        private final ExecutorTask delegate;
        
        public ProcessTask(String label, Process p) {
            this(createDelegate(label, p), p);
        }
        
        private static ExecutorTask createDelegate(String label, final Process p) {
            Runnable r = new Runnable() {
                public void run() {
                    try {
                        p.waitFor();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            };
            return ExecutionEngine.getDefault().execute(label, r, null);
        }
        
        private ProcessTask(ExecutorTask delegate, Process p) {
            super(delegate);
            this.delegate = delegate;
            this.p = p;
        }
        
        public int result() {
            try {
                 return p.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
                return 2;
            }
        }
        
        public void stop() {
            p.destroy();
            delegate.stop();
        }
        
        public InputOutput getInputOutput() {
            return delegate.getInputOutput();
        }
        
    }
    
}
