/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 *
 * @author David Botterill
 */
public class SystemFileUtils {
    
    public static String imageLocation="/images";
    public static String nbmFileLocation="/nbms";
    
    /** Creates a new instance of FileUtils */
    public SystemFileUtils() {
    }
    
    public static String getImageLocation() {
        return SystemFileUtils.getFileRoot() + imageLocation;
    }
    
    public static String getNbmLocation() {
        return SystemFileUtils.getFileRoot() + nbmFileLocation;
    }
    
    public static String getFileRoot() {
        String appServerInstanceRoot = System.getProperty("com.sun.aas.instanceRoot");
        return appServerInstanceRoot +
                File.separator +
                "docroot" +
                File.separator +
                "nbpluginportal" +
                File.separator +
                "files";
    }
    
    public static String getImageURLString() {
        
        String imageURL = "http://" + SystemFileUtils.getServerHost();
        imageURL += File.separator +
                "nbpluginportal" +
                File.separator +
                "files" +
                SystemFileUtils.imageLocation;
        
        return imageURL;
    }
    
    public static String getNBMURLString() {
        
        String imageURL = "http://" + SystemFileUtils.getServerHost();
        imageURL += File.separator +
                "nbpluginportal" +
                File.separator +
                "files" +
                SystemFileUtils.nbmFileLocation;
        
        return imageURL;
    }
    
    /**
     * This method is a work around for not being able to get the hostname
     * for the http server actually running the web app.  This method
     * will return the server hostname that will be used for form
     * URLs for accessing files on the server.
     * @return The hostname of the server where the plugin portal is running.
     */
    public static String getServerHost() {
        String serverHost = System.getProperty("pluginportal.server");
        if(null == serverHost) {
            System.err.println("**** Plugin Portal ERROR: pluginportal.server property not set! ****");
        }
        return serverHost;
    }
    
    
    public static File [] extractNBMs(String inZipFileName, String idPrefix) throws Exception {
        
        ZipFile zipFile = new ZipFile(inZipFileName);
        ArrayList<File> returnFiles = new ArrayList();
        
        /**
         * Now extract the NBM files from the ZIP file
         */
        Enumeration entries = zipFile.entries();
        
        while(entries.hasMoreElements()) {
            ZipEntry currentEntry = (ZipEntry)entries.nextElement();
            
            if(null != currentEntry) {
                InputStream inStream = zipFile.getInputStream(currentEntry);
                File newNBMFile = new File(SystemFileUtils.getNbmLocation() + File.separatorChar + idPrefix + currentEntry.getName());
                OutputStream outStream = new BufferedOutputStream(new FileOutputStream(newNBMFile));
                
                byte[] buffer = new byte[1024];
                int len;
                
                while((len = inStream.read(buffer)) >= 0) {
                    outStream.write(buffer, 0, len);
                }
                
                outStream.flush();
                inStream.close();
                outStream.close();
                
                
                returnFiles.add(newNBMFile);
            }
        }
        
        return (File [])returnFiles.toArray(new File[0]);
    }
    
    public static File extractInfoFile(String nbmFilename) throws Exception {
        
        File nbmFile = new File(nbmFilename);
        File returnFile = null;
        /**
         * Now extract the Info.xml file
         */
        returnFile = File.createTempFile("info_","xml");
        returnFile.deleteOnExit();
        JarFile jarFile = new JarFile(nbmFile);
        Enumeration<JarEntry> jarEntries = jarFile.entries();
        while(jarEntries.hasMoreElements()) {
            JarEntry currentEntry = jarEntries.nextElement();
        }
        JarEntry infoFileEntry = jarFile.getJarEntry("Info/info.xml");
        if(null != infoFileEntry) {
            InputStream inStream = jarFile.getInputStream(infoFileEntry);
            InputStreamReader isr =  new InputStreamReader(inStream);
            BufferedReader reader = new BufferedReader(isr);
            PrintWriter writer = new PrintWriter(new FileWriter(returnFile));
            String line = null;
            while ((line = reader.readLine()) != null) {
                writer.println(line);
            }
            
            writer.flush();
            writer.close();
            reader.close();
        } else {
            return null;
        }
        
        return returnFile;
    }
    
    public static void FileCopy(File inputFile, File outputFile) throws FileNotFoundException, IOException {
 
        FileChannel ic = new FileInputStream(inputFile).getChannel();
        FileChannel oc = new FileOutputStream(outputFile).getChannel();
        ic.transferTo(0, ic.size(), oc);
        ic.close();
        oc.close();
        
    }
}
