/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;
import javax.servlet.http.HttpServletRequest;
import org.netbeans.pluginportal.data.CommentImpl;
import org.netbeans.pluginportal.data.DisplayCategoryImpl;
import org.netbeans.pluginportal.data.OtherCategoryImpl;
import org.netbeans.pluginportal.data.PluginImpl;
import org.netbeans.pluginportal.data.RatingImpl;
import org.netbeans.pluginportal.system.CategoryCount;
import org.netbeans.pluginportal.system.Comment;
import org.netbeans.pluginportal.system.DisplayCategory;
import org.netbeans.pluginportal.system.CategoryRange;
import org.netbeans.pluginportal.system.OtherCategory;
import org.netbeans.pluginportal.system.Plugin;
import org.netbeans.pluginportal.system.impl.PluginAuthentication;
import org.netbeans.pluginportal.system.PluginException;
import org.netbeans.pluginportal.system.PluginSystem;
import org.netbeans.pluginportal.system.QualityLevel;
import org.netbeans.pluginportal.system.impl.QueryPlugin;
import org.netbeans.pluginportal.system.Rating;
import org.netbeans.pluginportal.system.SystemFileUtils;
import org.netbeans.pluginportal.system.WeightedCategory;

/**
 *
 * @author David Botterill
 */
public class PluginSystemFacade implements PluginSystem, QueryPlugin, QueryCache, ModifyPlugin, PluginAuthentication {
    
    private static EntityManagerFactory entityManagerFactory;
    protected static Authenticator authenticator;
    private CategoryCountCache categoryCountCache;
    private RecentlyAddedCache recentlyAddedCache;
    private TopRatedCache topRatedCache;
    private MostPopularCache mostPopularCache;
    private UpdateCenterCache updateCenterCache;
    private UpdateManager updateManager;
    private QueryManager queryManager;
    private HttpServletRequest request; //nb.org hack
    
    
    
    public PluginSystemFacade() {
        
    }
    
    /**
     * Since
     */
    
//    @PersistenceUnit(name="jdbc/pluginportal")
//    private EntityManagerFactory emf;
    private String temp;
    
    /**
     * Creates a new instance of PluginSystemFacade
     */
    public PluginSystemFacade(EntityManagerFactory emf) {
        
        entityManagerFactory = emf;
        authenticator = new Authenticator(request);
        categoryCountCache = new CategoryCountCache(emf);
        recentlyAddedCache = new RecentlyAddedCache(emf);
        topRatedCache = new TopRatedCache(emf);
        mostPopularCache = new MostPopularCache(emf);
        File updateCenterFile = new File(SystemFileUtils.getNbmLocation() + File.separator + "pluginportal-update-center.xml");
        updateCenterCache = new UpdateCenterCache(emf,updateCenterFile);
        updateManager = new UpdateManager(emf);
        updateManager.addUpdateListener(categoryCountCache);
        updateManager.addUpdateListener(recentlyAddedCache);
        updateManager.addUpdateListener(topRatedCache);
        updateManager.addUpdateListener(mostPopularCache);
        updateManager.addUpdateListener(updateCenterCache);
        queryManager = new QueryManager(emf);
    }
    /**
     * Creates a new instance of PluginSystemFacade
     */
    public PluginSystemFacade(EntityManagerFactory emf, HttpServletRequest inRequest) {
        this(emf);
        request = inRequest;
    }
    
    protected static EntityManager getEntityManager() {
        EntityManager returnEM = null;
        /**
         *@TODO: Not sure if we can use the same entitymanager with multiple threads
         *I've seen warnings about doing that.
         *for now, just create a new instance every time.
         */
        
        if(null != entityManagerFactory) {
            returnEM = entityManagerFactory.createEntityManager();
        } else {
            System.err.println("Error! EntityManagerFactory == null.");
        }
        
        return returnEM;
    }
    
    public Plugin[] getAllPlugins() {
        return queryManager.getAllPlugins();
    }
    
    public Plugin[] getTopRatedPlugins(int numberOfRecords) {
        return(getPartialList(topRatedCache.getTopRated(),numberOfRecords));
    }
    
    private Plugin [] getPartialList(ArrayList inList, int numberOfRecords) {
        if(null == inList || inList.size() == 0) {
            return null;
        } else {
            int recordCount=0;
            if(inList.size() < numberOfRecords ) {
                recordCount = inList.size();
            } else {
                recordCount = numberOfRecords;
            }
            Plugin [] returnPlugins = (Plugin [])new ArrayList(inList.subList(0,recordCount)).toArray(new Plugin[0]);
            return returnPlugins;
        }
    }
    
    public Plugin[] getTopDownloadsPlugins(int numberOfRecords) {
        return(getPartialList(mostPopularCache.getGetMostPopular(),numberOfRecords));
    }
    
    public Plugin getPlugin(Long inPluginID) {
        return queryManager.getPlugin(inPluginID);
    }
    
    public Plugin[] getRecentlyUpdatedPlugins(int numberOfRecords) {
        return null;
    }
    
    public Plugin[] getRecentlyAddedPlugins(int numberOfRecords) {
        return(getPartialList(recentlyAddedCache.getRecentlyAdded(),numberOfRecords));
    }
    
    public String[] getCategoriesByName() {
        return queryManager.getCategoriesByName();
    }
    
    public String[] getCategoriesByCount() {
        return queryManager.getCategoriesByCount();
    }
    
    public Plugin[] getPluginsByCategory(String category, boolean showAll) {
        return queryManager.getPluginsByCategory(category, showAll);
    }
    
    public Comment[] getComments(int pluginid) {
        return queryManager.getComments(pluginid);
    }
    
    public Plugin[] getPluginsByAuthor(String userid) {
        return queryManager.getPluginsByAuthor(userid);
    }
    
    public String login(String userid, String password) {
        String authToken = authenticator.login(userid,password);
        
        return authToken;
    }
    
    public Plugin addPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        return updateManager.addPlugin(plugin,authenticationToken,userid);
    }
    
    public Plugin addRating(Plugin plugin, Rating newRating, String authenticationToken, String userid) throws PluginException {
        
        return updateManager.addRating(plugin,newRating,authenticationToken,userid);
    }
    
    public Plugin addComment(Plugin plugin, Comment newComment,String authenticationToken, String userid) throws PluginException {
        return updateManager.addComment(plugin,newComment,authenticationToken, userid);
    }
    
    public Plugin createPlugin() {
        return new PluginImpl();
    }
    
    public Comment createComment() {
        return new CommentImpl();
    }
    
    public Rating createRating()  {
        return new RatingImpl();
    }
    
    public boolean isAuthenticated(String userid, String authenticationToken) {
        return this.authenticator.isAuthenticated(userid,authenticationToken);
    }
    
    public CategoryCount[] getCategoryCounts() {
        ArrayList<CategoryCount> returnList = categoryCountCache.getCache();
        return returnList.toArray(new CategoryCount[0]);
    }
    
    public DisplayCategory[] getDisplayCategories() {
        return queryManager.getDisplayCategories();
    }
    
    public void removePlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        updateManager.removePlugin(plugin,authenticationToken,userid);
    }
    
    public Plugin modifyPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        return updateManager.modifyPlugin(plugin,authenticationToken,userid);
    }
    
    public void removeRating(Rating newRating, String authenticationToken, String userid) throws PluginException {
    }
    
    public void removeComment(Comment comment, String authenticationToken, String userid) throws PluginException {
    }
    
    public OtherCategory[] getOtherCategories() {
        return queryManager.getOtherCategories();
    }
    
    public void addOtherCategory(Plugin plugin, OtherCategory othercategory, String authenticationToken, String userid) throws PluginException {
        updateManager.addOtherCategory(plugin,othercategory,authenticationToken,userid);
    }
    
    public String[] matchOtherCategoryPrefix(String prefix) {
        return queryManager.matchOtherCategoryPrefix(prefix);
    }
    
    public OtherCategory createOtherCategory() {
        return new OtherCategoryImpl();
    }
    
    public Plugin[] getPlugins(String searchString) {
        return queryManager.getPlugins(searchString);
    }
    
    public CategoryRange getCategoryRange() {
        return queryManager.getCategoryRange();
    }
    
    public Rating getRatingForUser(Long pluginid, String userid) {
        return queryManager.getRatingForUser(pluginid,userid);
    }
    
    public Plugin getPluginByName(String pluginname) {
        return queryManager.getPluginByName(pluginname);
    }
    
    public void logoff(String userid) {
        authenticator.logoff(userid);
    }
    
    /**
     * nb.org login hack
     */
    public void refreshRequest(HttpServletRequest request) {
        this.request=request;
        authenticator.refreshRequest(request); //nb.org login hack
    }
    
    public void removeOtherCategories(Plugin plugin, String authenticationToken, String userid) throws PluginException {
        updateManager.removeOtherCategories(plugin,authenticationToken,userid);
    }
    
    public void incrementPluginDownloads(Plugin plugin) throws PluginException {
        updateManager.incrementPluginDownloads(plugin);
    }
    
    
    
}
