/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.qreporter;

/**
 *
 * @author  mk97936
 */
public class QueryCreator {
    
    public static final int QTYPE_FROM = 1;
    public static final int QTYPE_TILL = 2;
    public static final int QTYPE_CREATED = 3;
    public static final int QTYPE_RESOLUTION = 4;
    
    private String SQLQuery;
    private String [] versions;
    private String [] components;
    private String [] keywords;
    private String dateFrom;
    private String dateTill;
    
    /*
    String select_from = "SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, history.issue_status, history.short_desc, history.component, history.subcomponent, history.keyword, history.resolution " +
                         "FROM issue, history WHERE issue.issue_id = history.issue_id AND history.issue_status IN " +
                         "('NEW','REOPENED','STARTED') AND history.component IN ('ant','beans','clazz','java','javadoc','projects') " + 
                         "AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < '2002-10-08 00:00:00')";
    
    String select_till = "SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, history.issue_status, history.short_desc, history.component, history.subcomponent, history.keyword, history.resolution " +
                         "FROM issue, history WHERE  issue.issue_id = history.issue_id  AND  history.issue_status IN " +
                         "('NEW','REOPENED','STARTED') AND history.component IN ('ant','beans','clazz','java','javadoc','projects') " +
                         "AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and " +
                         "a.whn < '2002-10-14 23:59:59')";
    
    String select_created = "SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, history.issue_status, history.short_desc, history.component, history.subcomponent, history.keyword, history.resolution " +
                            "FROM issue, history WHERE issue.issue_id = history.issue_id AND history.component IN " +
                            "('ant','beans','clazz','java','javadoc','projects') AND history.whn = (select max(a.whn) " +
                            "from history a where a.issue_id = issue.issue_id and a.whn < '2002-10-14 23:59:59') " +
                            "AND creation_ts BETWEEN '2002-10-08 00:00:00' AND '2002-10-14 23:59:59'";
     
     
     SELECT issue.issue_id,history.priority,history.issue_type,history.issue_status,history.short_desc FROM issue, 
            history WHERE issue.issue_id = history.issue_id AND history.issue_type='DEFECT' AND history.issue_status 
            IN ('NEW','REOPENED','STARTED') AND  history.target_milestone='TBD' AND history.component IN 
            ('beans','clazz','java','javadoc') AND history.whn = (select max(a.whn) from history a where a.issue_id = 
            issue.issue_id and a.whn < '2003-03-06 09:59:51') AND creation_ts < '2003-03-01' ORDER BY priority
     
    */
    
    /** Creates a new instance of QueryCreator */
    public QueryCreator() {
        
    }
    
    public void setDates(String dFrom, String dTill) {
        dateFrom = dFrom;
        dateTill = dTill;
    }
    
    public void setComponents(String [] comps) {
        components = comps;
    }
    
    public void setVersions(String [] vers) {
        versions = vers;
    }
    
    public String createQuery(int queryType) {
        
        StringBuffer query = new StringBuffer();
        
        query.append("SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, " +
                       "history.issue_status, history.short_desc, history.component, history.subcomponent, " +
                       "history.keyword, history.resolution " +
                     "FROM issue, history " +
                     "WHERE issue.issue_id = history.issue_id");
        
        if (components != null) {
            query.append(" AND history.component IN (");
            for (int i = 0; i < components.length; i++) {
                query.append("\'" + components[i] + "\',");
            }
            query.deleteCharAt(query.length() - 1);
            query.append(")");
        }
        
        if (versions != null) {
            query.append(" AND history.version IN (");
            for (int i = 0; i < versions.length; i++) {
                query.append("\'" + versions[i] + "\',");
            }
            query.deleteCharAt(query.length() - 1);
            query.append(")");
        }
        
        switch (queryType) {
            
            case QTYPE_FROM: 
                
                query.append(" AND history.issue_status IN ('NEW','REOPENED','STARTED')");
                query.append(" AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < ");
                query.append("\'" + dateFrom + "\')");
                return query.toString();
                
            case QTYPE_TILL:
                
                query.append(" AND history.issue_status IN ('NEW','REOPENED','STARTED')");
                query.append(" AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < ");
                query.append("\'" + dateTill + "\')");
                return query.toString();
                
            case QTYPE_CREATED:
                
                query.append(" AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < ");
                query.append("\'" + dateTill + "\')");
                query.append(" AND creation_ts BETWEEN ");
                query.append("\'" + dateFrom + "\'");
                query.append(" AND ");
                query.append("\'" + dateTill + "\'");
                return query.toString();
                
        }
        
        return null;
        
    }
    
    public String getResolutionQuery(String issue_id) {
     
        StringBuffer query = new StringBuffer();
        query.append("SELECT issue.issue_id,history.resolution FROM issue, history WHERE issue.issue_id = history.issue_id AND history.issue_id ='");
        query.append(issue_id);
        query.append("'"); 
        query.append(" AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < '");
        query.append(dateTill);
        query.append("')");
        return query.toString();
        
    }
    
    public String getSingleIssueQuery(String issue_id) {
        StringBuffer query = new StringBuffer();
        query.append("SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, " +
                     "history.issue_status, history.short_desc, history.component, history.subcomponent, " +
                     "history.keyword, history.resolution FROM issue, history WHERE " +
                     "issue.issue_id = history.issue_id AND history.issue_id ='" + issue_id + "'" + 
                     " AND history.whn = (select max(a.whn) from history a where a.issue_id = " +
                     "issue.issue_id and a.whn < '" + dateTill + "')");
        return query.toString();
    }
    
    /**
     * Selects all non-evaluated (target milestone is TBD) bugs (P1-P3) which were created before the week
     */
    public String getNonEvaluatedQuery() {
        
        StringBuffer query = new StringBuffer();
        
        query.append("SELECT issue.issue_id, issue.assigned_to, history.priority, history.issue_type, " +
                        "history.issue_status, history.short_desc, history.component, history.subcomponent, " +
                        "history.keyword, history.resolution " +
                     "FROM issue, history " +
                     "WHERE issue.issue_id = history.issue_id " +
                       "AND history.issue_type='DEFECT' " +
                       "AND history.issue_status IN ('NEW','REOPENED','STARTED') " +
                       "AND history.target_milestone='TBD' " +
                       "AND history.priority IN ('P1','P2','P3') ");
                       
        if (components != null) {
            query.append(" AND history.component IN (");
            for (int i = 0; i < components.length; i++) {
                query.append("\'" + components[i] + "\',");
            }
            query.deleteCharAt(query.length() - 1);
            query.append(")");
        }
        
        if (versions != null) {
            query.append(" AND history.version IN (");
            for (int i = 0; i < versions.length; i++) {
                query.append("\'" + versions[i] + "\',");
            }
            query.deleteCharAt(query.length() - 1);
            query.append(")");
        }
        
        query.append(" AND history.whn = (select max(a.whn) from history a where a.issue_id = issue.issue_id and a.whn < ");
        query.append("\'" + dateTill + "\')");
        
        query.append(" AND creation_ts < \' " + dateFrom + "\'");
        
        return query.toString();
        
    }
    
}
