/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.reports;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Class representing the Query page of Issuezilla.
 *
 * @author Jan Stola
 */
public class IssueQuery {
    static final String TYPE="issue_type";
    public static final String TYPE_DEFECT="DEFECT";
    public static final String TYPE_ENHANCEMENT="ENHANCEMENT";
    public static final String TYPE_FEATURE="FEATURE";
    public static final String TYPE_TASK="TASK";
    public static final String TYPE_PATCH="PATCH";
    static final String PRODUCT="component";
    static final String COMPONENT="subcomponent";
    static final String STATUS="issue_status";
    static final String STATUS_UNCONFIRMED="UNCONFIRMED";
    public static final String STATUS_NEW="NEW";
    public static final String STATUS_ASSIGNED="STARTED";
    public static final String STATUS_REOPENED="REOPENED";
    public static final String STATUS_RESOLVED="RESOLVED";
    public static final String STATUS_VERIFIED="VERIFIED";
    public static final String STATUS_CLOSED="CLOSED";
    static final String RESOLUTION="resolution";
    public static final String RESOLUTION_FIXED="FIXED";
    public static final String RESOLUTION_INVALID="INVALID";
    public static final String RESOLUTION_WONTFIX="WONTFIX";
    public static final String RESOLUTION_LATER="LATER";
    public static final String RESOLUTION_REMIND="REMIND";
    public static final String RESOLUTION_DUPLICATE="DUPLICATE";
    public static final String RESOLUTION_WORKSFORME="WORKSFORME";
    public static final String RESOLUTION_MOVED="MOVED";
    static final String PRIORITY="priority";
    public static final String PRIORITY_P1="P1";
    public static final String PRIORITY_P2="P2";
    public static final String PRIORITY_P3="P3";
    public static final String PRIORITY_P4="P4";
    public static final String PRIORITY_P5="P5";
    static final String PLATFORM="rep_platform";
    static final String OS="op_sys";
    static final String VERSION="version";
    static final String TARGET="target_milestone";
    static final String EMAIL1="email1";
    static final String EMAIL_TYPE1="emailtype1";
    public static final int EMAIL_TYPE_REGEXP=2;
    static final String EMAIL_TYPE_REGEXP_S="regexp";
    public static final int EMAIL_TYPE_NOT_REGEXP=1;
    static final String EMAIL_TYPE_NOT_REGEXP_S="notregexp";
    public static final int EMAIL_TYPE_SUBSTRING=0;
    static final String EMAIL_TYPE_SUBSTRING_S="substring";
    public static final int EMAIL_TYPE_EXACT=3;
    static final String EMAIL_TYPE_EXACT_S="exact";
    static final String EMAIL_ASSIGNED_TO1="emailassigned_to1";
    static final String EMAIL_REPORTER1="emailreporter1";
    static final String EMAIL_QA_CONTACT1="emailqa_contact1";
    static final String EMAIL_CC1="emailcc1";
    static final String EMAIL_COMMENTS1="emaillongdesc1";
    static final String EMAIL2="email2";
    static final String EMAIL_TYPE2="emailtype2";
    static final String EMAIL_ASSIGNED_TO2="emailassigned_to2";
    static final String EMAIL_REPORTER2="emailreporter2";
    static final String EMAIL_QA_CONTACT2="emailqa_contact2";
    static final String EMAIL_CC2="emailcc2";
    static final String EMAIL_COMMENTS2="emaillongdesc2";
    static final String ISSUE_EXINCLUDE="issueidtype";
    static final String ISSUE_INCLUDE="include";
    static final String ISSUE_EXCLUDE="exclude";
    static final String ISSUE_ID="issue_id";
    static final String CHANGED_IN="changedin";
    static final String VOTES="votes";
    static final String CHANGED_FIELD="chfield";
    static final String CHANGED_FIELD_ISSUE_CREATION_S="%5BIssue+creation%5D";
    public static final long CHANGED_FIELD_ISSUE_CREATION=0x1;
    static final String CHANGED_FIELD_ASSIGNED_TO_S="assigned_to";
    public static final long CHANGED_FIELD_ASSIGNED_TO=0x2;
    static final String CHANGED_FIELD_COMPONENT_S="subcomponent";
    public static final long CHANGED_FIELD_COMPONENT=0x4;
    static final String CHANGED_FIELD_EVERCONFIRMED_S="everconfirmed";
    public static final long CHANGED_FIELD_EVERCONFIRMED=0x8;
    static final String CHANGED_FIELD_GROUPSET_S="groupset";
    public static final long CHANGED_FIELD_GROUPSET=0x10;
    static final String CHANGED_FIELD_FILE_LOC_S="issue_file_loc";
    public static final long CHANGED_FIELD_FILE_LOC=0x20;
    static final String CHANGED_FIELD_SEVERITY_S="issue_severity";
    public static final long CHANGED_FIELD_SEVERITY=0x40;
    static final String CHANGED_FIELD_STATUS_S="issue_status";
    public static final long CHANGED_FIELD_STATUS=0x80;
    static final String CHANGED_FIELD_TYPE_S="issue_type";
    public static final long CHANGED_FIELD_TYPE=0x100;
    static final String CHANGED_FIELD_KEYWORDS_S="keywords";
    public static final long CHANGED_FIELD_KEYWORDS=0x200;
    static final String CHANGED_FIELD_OS_S="op_sys";
    public static final long CHANGED_FIELD_OS=0x400;
    static final String CHANGED_FIELD_PRIORITY_S="priority";
    public static final long CHANGED_FIELD_PRIORITY=0x800;
    static final String CHANGED_FIELD_PRODUCT_S="component";
    public static final long CHANGED_FIELD_PRODUCT=0x1000;
    static final String CHANGED_FIELD_QA_CONTACT_S="qa_contact";
    public static final long CHANGED_FIELD_QA_CONTACT=0x2000;
    static final String CHANGED_FIELD_PLATFORM_S="rep_platform";
    public static final long CHANGED_FIELD_PLATFORM=0x4000;
    static final String CHANGED_FIELD_REPORTER_S="reporter";
    public static final long CHANGED_FIELD_REPORTER=0x8000;
    public static final String CHANGED_FIELD_RESOLUTION_S="resolution";
    public static final long CHANGED_FIELD_RESOLUTION=0x10000;
    static final String CHANGED_FIELD_SUMMARY_S="short_desc";
    public static final long CHANGED_FIELD_SUMMARY=0x20000;
    static final String CHANGED_FIELD_STATUS_WHITEBOARD_S="status_whiteboard";
    public static final long CHANGED_FIELD_STATUS_WHITEBOARD=0x40000;
    static final String CHANGED_FIELD_TARGET_S="target_milestone";
    public static final long CHANGED_FIELD_TARGET=0x80000;
    static final String CHANGED_FIELD_VERSION_S="version";
    public static final long CHANGED_FIELD_VERSION=0x100000;
    static final String CHANGED_FIELD_VOTES_S="votes";
    public static final long CHANGED_FIELD_VOTES=0x200000;
    static final String CHANGED_FIELD_FROM="chfieldfrom";
    static final String CHANGED_FIELD_TO="chfieldto";
    public static final String CHANGED_FIELD_TO_NOW="Now";
    static final String CHANGED_FIELD_VALUE="chfieldvalue";
    static final String SUMMARY="short_desc";
    static final String SUMMARY_TYPE="short_desc_type";
    static final String COMMENTS="long_desc";
    static final String COMMENTS_TYPE="long_desc_type";
    static final String URL="issue_file_loc";
    static final String URL_TYPE="issue_file_loc_type";
    static final String STATUS_WHITEBOARD="status_whiteboard";
    static final String STATUS_WHITEBOARD_TYPE="status_whiteboard_type";
    static final String TYPE_IGNORE_CASE_S="substring";
    public static final int TYPE_IGNORE_CASE=0;
    static final String TYPE_CASE_S="casesubstring";
    public static final int TYPE_CASE=1;
    static final String TYPE_ALL_WORDS_S="allwords";
    public static final int TYPE_ALL_WORDS=2;
    static final String TYPE_ANY_WORDS_S="anywords";
    public static final int TYPE_ANY_WORDS=3;
    static final String TYPE_REGEXP_S="regexp";
    public static final int TYPE_REGEXP=4;
    static final String TYPE_NOT_REGEXP_S="notregexp";
    public static final int TYPE_NOT_REGEXP=5;
    static final String KEYWORDS="keywords";
    static final String KEYWORDS_TYPE="keywords_type";
    public static final int KEYWORDS_ANYWORDS=0;
    public static final int KEYWORDS_ALLWORDS=1;
    public static final int KEYWORDS_NOWORDS=2;
    static final String KEYWORDS_ANYWORDS_S="anywords";
    static final String KEYWORDS_ALLWORDS_S="allwords";
    static final String KEYWORDS_NOWORDS_S="nowords";
    
    public int type;
    public Set product;
    public Set component;
    public int status;
    public int resolution;
    public int priority;
    public Set platform;
    public Set os;
    public Set version;
    public Set target;
    public String email1;
    public int emailType1;
    public boolean assignedTo1;
    public boolean reporter1;
    public boolean QAContact1;
    public boolean CC1;
    public boolean comments1;
    public String email2;
    public int emailType2;
    public boolean assignedTo2;
    public boolean reporter2;
    public boolean QAContact2;
    public boolean CC2;
    public boolean comments2;
    public boolean excludeIssues;
    public List issueNumbers;
    /* no of days */
    public int changedIn;
    public int votes;
    public long changedField;
    public String changedFrom;
    /*date or Now*/
    public String changedTo;
    public String summary;
    public int summaryType;
    public String comments;
    public int commentsType;
    public String url;
    public int urlType;
    public String statusWhiteboard;
    public int statusWhiteboardType;
    public Set keywords;
    public int keywordsType;
    
     void appendParameter(StringBuffer sb, Object key, long value) {
        sb.append(key).append('=').append(value).append('&');
    }
    
     void appendParameter(StringBuffer sb, Object key, Object value) {
        sb.append(key).append('=').append(value).append('&');
    }
    
     void appendType(StringBuffer sb, String field, int type) {
        switch (type) {
            case TYPE_IGNORE_CASE: appendParameter(sb, field, TYPE_IGNORE_CASE_S); break;
            case TYPE_CASE: appendParameter(sb, field, TYPE_CASE_S); break;
            case TYPE_REGEXP: appendParameter(sb, field, TYPE_REGEXP_S); break;
            case TYPE_NOT_REGEXP: appendParameter(sb, field, TYPE_NOT_REGEXP_S); break;
            case TYPE_ALL_WORDS: appendParameter(sb, field, TYPE_ALL_WORDS_S); break;
            case TYPE_ANY_WORDS: appendParameter(sb, field, TYPE_ANY_WORDS_S); break;
        }
    }
    
    public String getQuery() {
        Iterator iter;
        StringBuffer sb=new StringBuffer("http://www.netbeans.org/issues/buglist.cgi?");
        if (type!=0) {
            if ((type&Issue.TYPE_DEFECT)!=0) appendParameter(sb, TYPE, TYPE_DEFECT);
            if ((type&Issue.TYPE_ENHANCEMENT)!=0) appendParameter(sb, TYPE, TYPE_ENHANCEMENT);
            if ((type&Issue.TYPE_FEATURE)!=0) appendParameter(sb, TYPE, TYPE_FEATURE);
            if ((type&Issue.TYPE_TASK)!=0) appendParameter(sb, TYPE, TYPE_TASK);
            if ((type&Issue.TYPE_PATCH)!=0) appendParameter(sb, TYPE, TYPE_PATCH);
        }
        if (product!=null) {
            iter=product.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, PRODUCT, iter.next());
            }
        }
        if (component!=null) {
            iter=component.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, COMPONENT, iter.next());
            }
        }
        if (status!=0) {
            if ((status&Issue.STATUS_UNCONFIRMED)!=0) appendParameter(sb, STATUS, STATUS_UNCONFIRMED);
            if ((status&Issue.STATUS_NEW)!=0) appendParameter(sb, STATUS, STATUS_NEW);
            if ((status&Issue.STATUS_ASSIGNED)!=0) appendParameter(sb, STATUS, STATUS_ASSIGNED);
            if ((status&Issue.STATUS_REOPENED)!=0) appendParameter(sb, STATUS, STATUS_REOPENED);
            if ((status&Issue.STATUS_RESOLVED)!=0) appendParameter(sb, STATUS, STATUS_RESOLVED);
            if ((status&Issue.STATUS_VERIFIED)!=0) appendParameter(sb, STATUS, STATUS_VERIFIED);
            if ((status&Issue.STATUS_CLOSED)!=0) appendParameter(sb, STATUS, STATUS_CLOSED);
        }
        if (resolution!=0) {
            if ((resolution&Issue.RESOLUTION_FIXED)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_FIXED);
            if ((resolution&Issue.RESOLUTION_INVALID)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_INVALID);
            if ((resolution&Issue.RESOLUTION_WONTFIX)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_WONTFIX);
            if ((resolution&Issue.RESOLUTION_LATER)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_LATER);
            if ((resolution&Issue.RESOLUTION_REMIND)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_REMIND);
            if ((resolution&Issue.RESOLUTION_DUPLICATE)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_DUPLICATE);
            if ((resolution&Issue.RESOLUTION_WORKSFORME)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_WORKSFORME);
            if ((resolution&Issue.RESOLUTION_MOVED)!=0) appendParameter(sb, RESOLUTION, RESOLUTION_MOVED);
        }
        if (priority!=0) {
            if ((priority&Issue.PRIORITY_P1)!=0) appendParameter(sb, PRIORITY, PRIORITY_P1);
            if ((priority&Issue.PRIORITY_P2)!=0) appendParameter(sb, PRIORITY, PRIORITY_P2);
            if ((priority&Issue.PRIORITY_P3)!=0) appendParameter(sb, PRIORITY, PRIORITY_P3);
            if ((priority&Issue.PRIORITY_P4)!=0) appendParameter(sb, PRIORITY, PRIORITY_P4);
            if ((priority&Issue.PRIORITY_P5)!=0) appendParameter(sb, PRIORITY, PRIORITY_P5);
        }
        if (platform!=null) {
            iter=platform.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, PLATFORM, iter.next());
            }
        }
        if (os!=null) {
            iter=os.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, OS, iter.next());
            }
        }
        if (version!=null) {
            iter=version.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, VERSION, iter.next());
            }
        }
        if (target!=null) {
            iter=target.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, TARGET, iter.next());
            }
        }
        if (email1!=null) {
            appendParameter(sb, EMAIL1, email1);
            switch (emailType1) {
                case EMAIL_TYPE_SUBSTRING: appendParameter(sb, EMAIL_TYPE1, EMAIL_TYPE_SUBSTRING_S); break;
                case EMAIL_TYPE_REGEXP: appendParameter(sb, EMAIL_TYPE1, EMAIL_TYPE_REGEXP_S); break;
                case EMAIL_TYPE_NOT_REGEXP: appendParameter(sb, EMAIL_TYPE1, EMAIL_TYPE_NOT_REGEXP_S); break;
                case EMAIL_TYPE_EXACT: appendParameter(sb, EMAIL_TYPE1, EMAIL_TYPE_EXACT_S); break;
            }
            if (assignedTo1) appendParameter(sb, EMAIL_ASSIGNED_TO1, "1");
            if (reporter1) appendParameter(sb, EMAIL_REPORTER1, "1");
            if (QAContact1) appendParameter(sb, EMAIL_QA_CONTACT1, "1");
            if (CC1) appendParameter(sb, EMAIL_CC1, "1");
            if (comments1) appendParameter(sb, EMAIL_COMMENTS1, "1");
        }
        if (email2!=null) {
            appendParameter(sb, EMAIL2, email1);
            switch (emailType2) {
                case EMAIL_TYPE_SUBSTRING: appendParameter(sb, EMAIL_TYPE2, EMAIL_TYPE_SUBSTRING_S); break;
                case EMAIL_TYPE_REGEXP: appendParameter(sb, EMAIL_TYPE2, EMAIL_TYPE_REGEXP_S); break;
                case EMAIL_TYPE_NOT_REGEXP: appendParameter(sb, EMAIL_TYPE2, EMAIL_TYPE_NOT_REGEXP_S); break;
                case EMAIL_TYPE_EXACT: appendParameter(sb, EMAIL_TYPE2, EMAIL_TYPE_EXACT_S); break;
            }
            if (assignedTo2) appendParameter(sb, EMAIL_ASSIGNED_TO2, "1");
            if (reporter2) appendParameter(sb, EMAIL_REPORTER2, "1");
            if (QAContact2) appendParameter(sb, EMAIL_QA_CONTACT2, "1");
            if (CC2) appendParameter(sb, EMAIL_CC2, "1");
            if (comments2) appendParameter(sb, EMAIL_COMMENTS2, "1");
        }
        if (excludeIssues) {
            appendParameter(sb, ISSUE_EXINCLUDE, ISSUE_EXCLUDE);
        } else {
            appendParameter(sb, ISSUE_EXINCLUDE, ISSUE_INCLUDE);
        }
        if (issueNumbers!=null) {
            iter=issueNumbers.iterator();
            sb.append(ISSUE_ID).append('=');
            while (iter.hasNext()) {
                sb.append(iter.next()).append("%2C");
            }
            sb.append('&');
        }
        if (changedIn!=0) appendParameter(sb, CHANGED_IN, changedIn);
        if (votes!=0) appendParameter(sb, VOTES, votes);
        if (changedField!=0) {
            if ((changedField&CHANGED_FIELD_ISSUE_CREATION)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_ISSUE_CREATION_S);
            if ((changedField&CHANGED_FIELD_ASSIGNED_TO)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_ASSIGNED_TO_S);
            if ((changedField&CHANGED_FIELD_COMPONENT)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_COMPONENT_S);
            if ((changedField&CHANGED_FIELD_EVERCONFIRMED)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_EVERCONFIRMED_S);
            if ((changedField&CHANGED_FIELD_GROUPSET)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_GROUPSET_S);
            if ((changedField&CHANGED_FIELD_FILE_LOC)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_FILE_LOC_S);
            if ((changedField&CHANGED_FIELD_SEVERITY)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_SEVERITY_S);
            if ((changedField&CHANGED_FIELD_STATUS)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_STATUS_S);
            if ((changedField&CHANGED_FIELD_TYPE)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_TYPE_S);
            if ((changedField&CHANGED_FIELD_KEYWORDS)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_KEYWORDS_S);
            if ((changedField&CHANGED_FIELD_OS)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_OS_S);
            if ((changedField&CHANGED_FIELD_PRIORITY)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_PRIORITY_S);
            if ((changedField&CHANGED_FIELD_PRODUCT)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_PRODUCT_S);
            if ((changedField&CHANGED_FIELD_QA_CONTACT)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_QA_CONTACT_S);
            if ((changedField&CHANGED_FIELD_PLATFORM)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_PLATFORM_S);
            if ((changedField&CHANGED_FIELD_REPORTER)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_REPORTER_S);
            if ((changedField&CHANGED_FIELD_RESOLUTION)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_RESOLUTION_S);
            if ((changedField&CHANGED_FIELD_SUMMARY)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_SUMMARY_S);
            if ((changedField&CHANGED_FIELD_STATUS_WHITEBOARD)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_STATUS_WHITEBOARD_S);
            if ((changedField&CHANGED_FIELD_TARGET)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_TARGET_S);
            if ((changedField&CHANGED_FIELD_VERSION)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_VERSION_S);
            if ((changedField&CHANGED_FIELD_VOTES)!=0) appendParameter(sb, CHANGED_FIELD, CHANGED_FIELD_VOTES_S);
            if (changedFrom!=null) {
                appendParameter(sb, CHANGED_FIELD_FROM, changedFrom);
            }
            if (changedTo!=null) {
                appendParameter(sb, CHANGED_FIELD_TO, changedTo);
            } else {
                appendParameter(sb, CHANGED_FIELD_TO, CHANGED_FIELD_TO_NOW);
            }
        }
        if (summary!=null) {
            appendParameter(sb, SUMMARY, summary);
            appendType(sb, SUMMARY_TYPE, summaryType);
        }
        if (comments!=null) {
            appendParameter(sb, COMMENTS , comments);
            appendType(sb, COMMENTS_TYPE, commentsType);
        }
        if (url!=null) {
            appendParameter(sb, URL , url);
            appendType(sb, URL_TYPE, urlType);
        }
        if (statusWhiteboard!=null) {
            appendParameter(sb, STATUS_WHITEBOARD, statusWhiteboard);
            appendType(sb, STATUS_WHITEBOARD_TYPE, statusWhiteboardType);
        }
        
        if (keywords!=null) {
            iter=keywords.iterator();
            while (iter.hasNext()) {
                appendParameter(sb, KEYWORDS, iter.next());
            }
            switch (keywordsType) {
                case KEYWORDS_ANYWORDS: appendParameter(sb, KEYWORDS_TYPE, KEYWORDS_ANYWORDS_S); break;
                case KEYWORDS_ALLWORDS: appendParameter(sb, KEYWORDS_TYPE, KEYWORDS_ALLWORDS_S); break;
                case KEYWORDS_NOWORDS: appendParameter(sb, KEYWORDS_TYPE, KEYWORDS_NOWORDS_S); break;
            }
        }
        return sb.toString();
    }
    
    public List getIssueNumbers() throws IOException {
        LinkedList issueNumbs=new LinkedList();
        URL url=new URL(getQuery());
        HttpURLConnection con=(HttpURLConnection)url.openConnection();        
        try {
            BufferedReader br=new BufferedReader(new InputStreamReader(con.getInputStream()));
            try {
                String s;
                while ((s=br.readLine())!=null) {
                    int i=s.indexOf("<A HREF=\"show_bug.cgi?id=");
                    if (i!=-1) {
                        s=s.substring(i+25);
                        s=s.substring(0, s.indexOf('"'));
                        issueNumbs.add(new Integer(s));
                    }
                }
            } finally {
                br.close();
            }
        } finally {
            con.disconnect();
        }
        return issueNumbs;
    }
    
    public List getIssues() throws IOException {
        List issues=new LinkedList();
        List issueNumbs=getIssueNumbers();
        return Queries.getIssues(issueNumbs);
    }
    
    public IssueCounts getIssueCounts() throws IOException {
        IssueCounts counts=new IssueCounts();
        URL url=new URL(getQuery());
        HttpURLConnection con=(HttpURLConnection)url.openConnection();        
        try {
            BufferedReader br=new BufferedReader(new InputStreamReader(con.getInputStream()));
            try {
                String s, ss;
                while (((s=br.readLine())!=null)) {
                    if (s.startsWith("<TR VALIGN=TOP ALIGN=LEFT CLASS=")) {
                        /* Type of issue. */
                        ss=s.substring(s.indexOf("<SPAN CLASS="+TYPE+"><nobr>")+29);
                        ss=ss.substring(0, ss.indexOf("</nobr>"));
                        if (ss.equals(TYPE_DEFECT)) {
                            counts.types[0]++;
                        } else if (ss.equals("ENHANC")) {
                            counts.types[1]++;
                        } else if (ss.equals("FEATUR")) {
                            counts.types[2]++;
                        } else if (ss.equals(TYPE_TASK)) {
                            counts.types[3]++;
                        } else if (ss.equals(TYPE_PATCH)) {
                            counts.types[4]++;
                        }
                        /* Priority of issue. */
                        ss=s.substring(s.indexOf("<SPAN CLASS="+PRIORITY+"><nobr>")+27);
                        ss=ss.substring(0, ss.indexOf("</nobr>"));
                        if  (ss.equals(PRIORITY_P1)) {
                            counts.priorities[0]++;
                        } else if  (ss.equals(PRIORITY_P2)) {
                            counts.priorities[1]++;
                        } else if  (ss.equals(PRIORITY_P3)) {
                            counts.priorities[2]++;
                        } else if  (ss.equals(PRIORITY_P4)) {
                            counts.priorities[3]++;
                        } else if  (ss.equals(PRIORITY_P5)) {
                            counts.priorities[4]++;
                        }
                        /* Status of issue. */
                        ss=s.substring(s.indexOf("<SPAN CLASS=status><nobr>")+25);
                        ss=ss.substring(0, ss.indexOf("</nobr>"));
                        if (ss.equals("UNCO")) {
                            counts.statuses[0]++;
                        } else if (ss.equals(STATUS_NEW)) {
                            counts.statuses[1]++;
                        } else if (ss.equals("STAR")) {
                            counts.statuses[2]++;
                        } else if (ss.equals("REOP")) {
                            counts.statuses[3]++;
                        } else if (ss.equals("RESO")) {
                            counts.statuses[4]++;
                        } else if (ss.equals("VERI")) {
                            counts.statuses[5]++;
                        } else if (ss.equals("CLOS")) {
                            counts.statuses[6]++;
                        }
                        /** Resolution of issue. */                        
                        ss=s.substring(s.indexOf("<SPAN CLASS="+RESOLUTION+"><nobr>")+29);                        
                        ss=ss.substring(0, ss.indexOf("</nobr>"));                        
                        if (ss.equals("FIXE")) {
                            counts.resolutions[0]++;
                        } else if (ss.equals("INVA")) {
                            counts.resolutions[1]++;
                        } else if (ss.equals("LATE")) {
                            counts.resolutions[2]++;
                        } else if (ss.equals("REMI")) {
                            counts.resolutions[3]++;
                        } else if (ss.equals("DUPL")) {
                            counts.resolutions[4]++;
                        } else if (ss.equals("WORK")) {
                            counts.resolutions[5]++;
                        } else if (ss.equals("WONT")) {
                            counts.resolutions[6]++;
                        } else if (ss.equals("MOVE")) {
                            counts.resolutions[7]++;
                        }
                    }
                }
            } finally {
                br.close();
            }
        } finally {
            con.disconnect();
        }
        return counts;
    }
    
}
