/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.beans;

import java.util.ArrayList;
import org.netbeans.testspeceditor.*;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeSupport;
import org.netbeans.testspeceditor.events.Persistent;
import org.netbeans.testspeceditor.undo.UndoAnnotation;
import org.netbeans.testspeceditor.undo.UndoManager;
import org.netbeans.testspeceditor.undo.UndoMethods;
import org.netbeans.testspeceditor.utils.ParsingHelper;
import org.netbeans.testspeceditor.utils.Utils;
import org.netbeans.testspeceditor.xmlizable.XMLSource;
import org.netbeans.testspeceditor.xmlizable.XMLizableException;
import org.netbeans.testspeceditor.xmlparser.Tag;

/**
 *
 * @author ehucka
 */
public class TestSuite implements Bean, Persistent, TextData {
    
    private String title;
    private String suite_id;
    private ArrayList<TestCase> cases;
    private Purpose purpose;
    private Setup setup;
    private Comments comments;
    private Teardown teardown;
    
    transient BeanChangeSupport support = new BeanChangeSupport(this);
    transient int id=-1;
    
     
    public TestSuite() {
        cases = new ArrayList(24);
        purpose=new Purpose(this);
        setup=new Setup(this);
        comments=new Comments(this);
        teardown=new Teardown(this);
    }
    
    public TestSuite(String name) {
        this();
        UndoManager.getInstance().setLogProperties(false);
        id=Application.getInstance().getDocument().getTestSpec().caseid++;
        setTitle(name);
        UndoManager.getInstance().setLogProperties(true);
        setComments("TODO");
    }
    
    public String getTitle() {
        return title;
    }
    
    public String getSuite_id() {
        return suite_id;
    }
    
    public TestCase[] getCases() {
        return cases.toArray(new TestCase[cases.size()]);
    }
    
    public String getPurpose() {
        return purpose.getContent();
    }
    
    public Purpose getPurposeBean() {
        return purpose;
    }
    
    public String getSetup() {
        return setup.getContent();
    }
    
    public Setup getSetupBean() {
        return setup;
    }
    
    public String getComments() {
        return comments.getContent();
    }
    
    public Comments getCommentsBean() {
        return comments;
    }
    
    public String getTeardown() {
        return teardown.getContent();
    }
    
    public Teardown getTeardownBean() {
        return teardown;
    }
    
    public void setTitle(String title) {
        String old = getTitle();
        this.title = title;
        setSuite_id(Utils.getID(title, id));
        support.propertyChanged("title", old, getTitle());
    }
    
    public void setSuite_id(String suite_id) {
        String old = getSuite_id();
        this.suite_id = suite_id;
        support.propertyChanged("suite_id", old, getSuite_id());
    }
    
    public void addCase(TestCase cas) {
        this.cases.add(cas);
        support.childrenChanged(cas, -1, cases.size()-1);
    }
    
    public void removeCase(TestCase cas) {
        int old = cases.indexOf(cas);
        if (old >= 0) {
            this.cases.remove(old);
            support.childrenChanged(cas, old, -1);
        }
    }
    
    public boolean hasCase(TestCase cas) {
        return cases.contains(cas);
    }
    
    public void moveUp(TestCase cas) {
        int index=cases.indexOf(cas);
        cases.remove(index);
        index--;
        cases.add(index, cas);
        support.childrenChanged(cas, index+1, index);
    }
    
    public void moveDown(TestCase cas) {
        int index=cases.indexOf(cas);
        cases.remove(index);
        index++;
        cases.add(index, cas);
        support.childrenChanged(cas, index-1, index);
    }
    
    @UndoAnnotation(method=UndoMethods.GET)
    public TestCase get(int index) {
        return cases.get(index);
    }
    
    @UndoAnnotation(method=UndoMethods.REMOVE)
    public TestCase remove(int index) {
        TestCase cs=cases.get(index);
        removeCase(cs);
        return cs;
    }
    
    @UndoAnnotation(method=UndoMethods.ADD)
    public void add(TestCase cs, int index) {
        cases.add(index, cs);
        support.childrenChanged(cs, -1, index);
    }
    
    public void setPurpose(String purpose) {
        this.purpose.setContent(purpose);
    }
    
    public void setSetup(String setup) {
        this.setup.setContent(setup);
    }
    
    public void setComments(String comments) {
        this.comments.setContent(comments);
    }
    
    public void setTeardown(String teardown) {
        this.teardown.setContent(teardown);
    }
    
    public BeanChangeSupport getBeanSupport() {
        return support;
    }
    
    public void setObject(Tag tag) {
        title=tag.getAttribute("title");
        suite_id=tag.getAttribute("suite_id");
        
        Tag[] scs=tag.getChildren("suite-case");
        for (int i = 0; i < scs.length; i++) {
            String cid=scs[i].getAttribute("ref");
            TestCase cas=ParsingHelper.getInstance().getCases().get(cid);
            if (cas != null) {
                cases.add(cas);
            }
        }
        purpose.setContent(Utils.getValueNotNull(tag, "purpose"));
        comments.setContent(Utils.getValueNotNull(tag, "comments"));
        setup.setContent(Utils.getValueNotNull(tag, "setup"));
        teardown.setContent(Utils.getValueNotNull(tag, "teardown"));
    }
    
    public void getTag(Tag tag) {
        try {
            tag.setAttribute("title", title);
            tag.setAttribute("suite_id", suite_id);
            
            for (TestCase cas:cases) {
                SuiteCaseRef ref = new SuiteCaseRef(cas);
                XMLSource src=new XMLSource(ref);
                tag.addChild(src.getTag());
            }
            
            XMLSource src=new XMLSource(purpose);
            tag.addChild(src.getTag());
            src=new XMLSource(comments);
            tag.addChild(src.getTag());
            src=new XMLSource(setup);
            tag.addChild(src.getTag());
            src=new XMLSource(teardown);
            tag.addChild(src.getTag());
        } catch (XMLizableException ex) {
            ex.printStackTrace();
        }
    }
    
    public Bean copy() {
        TestSuite ts=new TestSuite(getTitle());
        ts.setComments(getComments());
        ts.setPurpose(getPurpose());
        ts.setSetup(getSetup());
        ts.setTeardown(getTeardown());
        for (TestCase tc:cases) {
            ts.addCase(tc);
        }
        return ts;
    }
    
    public void initObject() {
        comments.setParent(this);
        setup.setParent(this);
        teardown.setParent(this);
        purpose.setParent(this);
    }
    
    public String[] getContents() {
        String[] ret=new String[4];
        ret[0]=purpose.getContent();
        ret[1]=setup.getContent();
        ret[2]=comments.getContent();
        ret[3]=teardown.getContent();
        return ret;
    }
}
