/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.find;

import java.util.HashSet;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestResult;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.beans.Value;
import org.netbeans.testspeceditor.events.Bean;

/**
 *
 * @author ehucka
 */
public class FindEngine {
    
    static FindEngine instance = null;
    
    private FindPattern pattern;
    FoundData data;
    
    HashSet<Bean> found;
    
    public static FindEngine getInstance() {
        return instance;
    }
    
     
    public FindEngine(FindPattern pattern) {
        this.pattern=pattern;
        found=new HashSet(64);
        data=new FoundData();
        instance=this;
    }
    
    public void find() {
        find(Application.getInstance().getDocument().getTestSpec());
    }
    
    public void find(Bean bean, Bean parent, int parentIndex) {
        if (bean instanceof TestSpec) {
            find((TestSpec)bean);
        } else if (bean instanceof TestSuite) {
            find((TestSuite)bean);
        } else if (bean instanceof TestCase) {
            find((TestCase)bean);
        } else if (bean instanceof TestResult) {
            find((TestResult)bean, (TestCase)parent);
        } else if (bean instanceof TestAction) {
            find((TestAction)bean, (TestCase)parent, parentIndex);
        }
    }
    
    public void find(TestSpec ts) {
        if (found.contains(ts)) {
            return;
        } else {
            found.add(ts);
        }
        find(ts.getIntroductionBean(), ts, "Introduction");
        find(ts.getCommentsBean(), ts, "Comments");
        TestSuite[] suites=ts.getTestSuites();
        for (int i = 0; i < suites.length; i++) {
            find(suites[i]);
        }
        TestCase[] cases=ts.getTestCases();
        for (int i = 0; i < cases.length; i++) {
            find(cases[i]);
        }
    }
    
    public void find(TestSuite ts) {
        if (found.contains(ts)) {
            return;
        } else {
            found.add(ts);
        }
        find(ts.getCommentsBean(), ts, "Comments");
        find(ts.getPurposeBean(), ts, "Purpose");
        find(ts.getTeardownBean(), ts, "Teardown");
        find(ts.getSetupBean(), ts, "Setup");
    }
    
    public void find(TestCase tc) {
        if (found.contains(tc)) {
            return;
        } else {
            found.add(tc);
        }
        find(tc.getSetupBean(), tc, "Setup");
        find(tc.getTeardownBean(), tc, "Teardown");
        find(tc.getCommentsBean(), tc, "Comments");
        TestAction[] actions=tc.getActions();
        for (int j = 0; j < actions.length-1; j++) { //-1 for results
            find(actions[j], tc, j);
        }
        find(tc.getResult(), tc);
    }

    public void find(TestAction ta, TestCase tc, int index) {
        if (found.contains(ta)) {
            return;
        } else {
            found.add(ta);
        }
        find(ta, tc, "Action "+String.valueOf(index+1));
    }
    
    public void find(TestResult tr, TestCase tc) {
        if (found.contains(tr)) {
            return;
        } else {
            found.add(tr);
        }
        find(tr, tc, "Result");
    }
    
    protected void find(Value bean, Bean presented, String label) {
        FoundOccurance[] occurs=getPattern().find(bean.getContent());
        if (occurs.length > 0) {
            FoundBean fb=new FoundBean(bean, presented, label, occurs);
            data.addFoundBean(fb);
        }
    }
    
    public FoundData getFoundData() {
        return data;
    }

    public FindPattern getPattern() {
        return pattern;
    }
}
