/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui;


import java.awt.Cursor;
import java.awt.event.KeyEvent;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.DocumentListener;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.options.Options;
import org.netbeans.testspeceditor.ui.actions.*;
import org.netbeans.testspeceditor.ui.errors.ErrorsDialog;
import org.netbeans.testspeceditor.ui.status.StatusManager;
import org.netbeans.testspeceditor.ui.status.StatusTarget;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.ui.tree.TestSpecNode;
import org.netbeans.testspeceditor.ui.tree.menu.PopupSupport;
import org.netbeans.testspeceditor.ui.tree.TSTreeModel;
import org.netbeans.testspeceditor.ui.tree.TreeRenderer;
import org.netbeans.testspeceditor.utils.Utils;
/**
 *
 * @author  ehucka
 */
public class MainFrame extends javax.swing.JFrame implements StatusTarget, DocumentListener {
    
    private static final Cursor WAIT_CURSOR = Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR );
    
    
    private TSTreeModel treeModel;
    private PanelCard cards;
    private ErrorsDialog errors;
    
    private TitleChanger titleChanger;
    
    /**
     * Creates new form MainFrame
     */
    public MainFrame() {
        Application.getInstance().setUi(this);
        treeModel=new TSTreeModel();
        initComponents();
        treeModel.setSelectionModel(tree.getSelectionModel());
        
        setSize(Options.getInstance().getWidth(), Options.getInstance().getHeight());
        setLocation(Options.getInstance().getX(), Options.getInstance().getY());
        jSplitPane1.setDividerLocation((int)(Options.getInstance().getDividerLocations().getMain()*getWidth()));
        cards=new PanelCard(itemPanel);
        
        errors=new ErrorsDialog(this);
        System.setErr(errors.getPrintStream());
        titleChanger=new TitleChanger(this);
        
        UIInitializer.addTreeUpdateListener(tree);
        UIInitializer.initActions(this);
        tree.addTreeSelectionListener(cards);
        PopupSupport.getDefault().registerJTree(tree);
        PopupSupport.getDefault().registerMenu(editMenu, new Action[] {new UndoAction(), new RedoAction(),
        null, new CopyAction(), new CutAction(), new PasteAction(), new DeleteAction(), null,
        new FindAction(), new ReplaceAction()});
        PopupSupport.getDefault().registerMenu(fileMenu, new Action[] {new NewDocumentAction(),
        new OpenDocumentAction(), new SaveDocumentAction(), new SaveAsDocumentAction(),
        null, new ExitApplicationAction()});
        PopupSupport.getDefault().registerMenu(toolsMenu, new Action[] {new ErrorsAction(errors),
        new ImportAction(), new TransformAction()});//run tests, transform
        PopupSupport.getDefault().registerMenu(helpMenu, new Action[] {new HelpAction()});
        StatusManager.getInstance().registerStaturTarget(this);
    }
    
    public TSTreeModel getTreeModel() {
        return treeModel;
    }
    
    public void setStatusLineText(String text) {
        statusL.setText(text);
    }
    
    public void setProgressValue(double percent) {
    }
    
    public void documentChanged() {
        getTreeModel().setRoot(Application.getInstance().getDocument().getTestSpec());
        cards.documentChanged();
    }
    
    public Node[] getSelectedNodes() {
        return Utils.getSelectedNodes(tree);
    }
    
    public ErrorsDialog getErrors() {
        return errors;
    }
    
    public JTree getTree() {
        return tree;
    }
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        tree = new JTree(getTreeModel());
        itemPanel = new javax.swing.JPanel();
        statusL = new javax.swing.JLabel();
        mainMenu = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        editMenu = new javax.swing.JMenu();
        toolsMenu = new javax.swing.JMenu();
        helpMenu = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Test Specifications Editor 2005");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jPanel2.setLayout(new java.awt.BorderLayout());

        jSplitPane1.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        jSplitPane1.setDividerSize(6);
        jSplitPane1.setResizeWeight(0.4);

        jScrollPane1.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));

        tree.setFont(new java.awt.Font("Dialog", 0, 10));
        tree.setAutoscrolls(true);
        tree.setCellRenderer(new TreeRenderer());
        tree.setDragEnabled(true);
        tree.setInvokesStopCellEditing(true);
        tree.setLargeModel(true);
        tree.setShowsRootHandles(true);
        tree.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                treeKeyReleased(evt);
            }
        });
        jScrollPane1.setViewportView(tree);

        jSplitPane1.setLeftComponent(jScrollPane1);

        itemPanel.setLayout(new java.awt.CardLayout());
        jSplitPane1.setRightComponent(itemPanel);

        jPanel2.add(jSplitPane1, java.awt.BorderLayout.CENTER);

        statusL.setFont(new java.awt.Font("Dialog", 0, 11));
        statusL.setText(" ");

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 615, Short.MAX_VALUE)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(statusL, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 591, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 332, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(statusL, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        mainMenu.add(fileMenu);

        editMenu.setMnemonic('e');
        editMenu.setText("Edit");
        mainMenu.add(editMenu);

        toolsMenu.setMnemonic('t');
        toolsMenu.setText("Tools");
        mainMenu.add(toolsMenu);

        helpMenu.setMnemonic('h');
        helpMenu.setText("Help");
        mainMenu.add(helpMenu);

        setJMenuBar(mainMenu);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void treeKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_treeKeyReleased
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            cards.invokeDefaultAction(tree);
        }
    }//GEN-LAST:event_treeKeyReleased
    
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        finish();
    }//GEN-LAST:event_formWindowClosing
    
    public void selectNodeFor(Bean path) {
        TreePath tpath = ((TestSpecNode)getTreeModel().getRoot()).findNode(path);
        if (tpath != null) {
            tree.setSelectionPath(tpath);
            tree.makeVisible(tpath);
            tree.scrollPathToVisible(tpath);
        }
    }
    
    public void finish() {
        Options.getInstance().setWidth(getWidth());
        Options.getInstance().setHeight(getHeight());
        Options.getInstance().setX(getX());
        Options.getInstance().setY(getY());
        Options.getInstance().getDividerLocations().setMain(jSplitPane1.getDividerLocation()*1.0/getWidth());
        cards.finish();
        Application.getInstance().finish();
    }
    
    public void setWaitCursor(boolean wait) {
        if (wait) {
            setCursor(WAIT_CURSOR);
        } else {
            setCursor(Cursor.getDefaultCursor());
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JMenu editMenu;
    public javax.swing.JMenu fileMenu;
    public javax.swing.JMenu helpMenu;
    public javax.swing.JPanel itemPanel;
    public javax.swing.JPanel jPanel1;
    public javax.swing.JPanel jPanel2;
    public javax.swing.JScrollPane jScrollPane1;
    public javax.swing.JSplitPane jSplitPane1;
    public javax.swing.JMenuBar mainMenu;
    public javax.swing.JLabel statusL;
    public javax.swing.JMenu toolsMenu;
    public javax.swing.JTree tree;
    // End of variables declaration//GEN-END:variables
    
}
