/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;

import java.util.ArrayList;
import java.util.Collections;
import org.netbeans.testspeceditor.options.CompletionTag;
import org.netbeans.testspeceditor.options.Options;

/**
 *
 * @author ehucka
 */
public class FormatSupport {
    
    static FormatSupport instance = null;
    
    ArrayList<FormatTag> tags;
    
    public static FormatSupport getInstance() {
        if (instance == null) {
            instance=new FormatSupport();
        }
        return instance;
    }
    
     
    public FormatSupport() {
        CompletionTag[] custom=Options.getInstance().getCompletionTags();
        tags=new ArrayList(HTMLTAGS.length*3+custom.length+ENTITIES.length);
        
        for (String html: HTMLTAGS) {
            FormatTag.generateTags(html, tags);
        }
        for (CompletionTag ct:custom) {
            tags.add(new InsertionFormatTag(ct.getName(), ct.getValue()));
        }
        for (String en:ENTITIES) {
            tags.add(new ContextFormatTag(en));
        }
    }
    
    public boolean isHTMLTag(String tagName) {
        for (String t:HTMLTAGS) { //for all formated tags
            if (t.equals(tagName.toLowerCase())) {
                return true;
            }
        }
        return false;
    }
    
    public ArrayList<FormatTag> getTagForContext(String context) {
        ArrayList<FormatTag> ret=new ArrayList(tags.size());
        for (FormatTag tag:tags) {
            if (tag.matchContext(context)) {
                ret.add(tag);
            }
        }
        Collections.sort(ret);
        return ret;
    }
    
    public boolean isEscape(String t) {
        for (String s:ENTITIES) {
            if (s.equals(t)) {
                return true;
            }
        }
        return false;
    }
    
    static String[] HTMLTAGS={"a","applet","b","big","body","br","caption","cite",  //HTML
    "col","colgroup", "code", "dd","div","dl","dt","em","font","form","frame","frameset",
    "head","h1","h2","h3","h4","h5","h6","hr","html","i","iframe","img","link",
    "li","map","meta","noframes","ol","p","param","pre","s","script","small","span",
    "strong","style","sub","sup","td","th","title","tr","tt","u","ul","var","table"};
    
    static String[] ENTITIES = {"&amp;", "&brvbar;", "&copy;", "&frasl;", "&gt;",
    "&lt;", "&nbsp;", "&plusmn;", "&quot;", "&reg;", "&sect;"};
}
