/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;


import java.lang.reflect.Method;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;
import org.netbeans.testspeceditor.TextData;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeListener;
import org.netbeans.testspeceditor.utils.Timer;
import org.netbeans.testspeceditor.utils.Utils;

/**
 *
 * @author ehucka
 */
public class TextAdapter implements DocumentListener, Runnable, BeanChangeListener {
    
    String propertyName;
    Method getter, setter=null;
    JTextComponent component;
    boolean update=true;
    Bean bean = null;
    
     
    public TextAdapter(JTextComponent component, Class clazz, String propertyName) throws Exception {
        this.component = component;
        this.propertyName = propertyName;
        this.setter = clazz.getMethod(Utils.getSetterName(propertyName), String.class);
        this.getter = clazz.getMethod(Utils.getGetterName(propertyName));
        component.getDocument().addDocumentListener(this);
    }
    
    public void load(Bean object) {
        if (object != bean) {
            if (bean != null) {
                bean.getBeanSupport().removeBeanChangeListener(this);
            }
            if (object != null) {
                object.getBeanSupport().addBeanChangeListener(this);
            }
        }
        this.bean = object;
        load();
    }
    
    protected void load() {
        if (update) {
            update=false;
            if (bean != null) {
                try {
                    String t=(String)(getter.invoke(bean));
                    component.setText(t);
                } catch (Exception ex) {
                    System.err.println("getter "+getter.getName());
                    ex.printStackTrace();
                }
            } else {
                component.setText("");
            }
            update=true;
        }
    }
    
    protected void set() {
        if (update) {
            Timer.getInstance().delayOnce(this, 333);
            if (bean instanceof TextData) {
                MatchingWords.getInstance().updateWords((TextData)bean);
            }
        }
    }
    
    public void run() {
        if (update && bean != null) {
            try {
                update=false;
                setter.invoke(bean, component.getText());
                update=true;
            } catch (Exception ex) {
                System.err.println("object "+bean+" component text "+component.getText());
                ex.printStackTrace();
            }
        }
    }
    
    public void removeUpdate(javax.swing.event.DocumentEvent e) {
        set();
    }
    
    public void insertUpdate(javax.swing.event.DocumentEvent e) {
        set();
    }
    
    public void changedUpdate(javax.swing.event.DocumentEvent e) {
//        set();
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (this.propertyName.equals(propertyName)) {
            load();
        }
    }
    
}
