/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;


import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.NodesClipboard;
import org.netbeans.testspeceditor.ui.actions.ActionAnnotation;
import org.netbeans.testspeceditor.ui.actions.ActionsManager;
import org.netbeans.testspeceditor.ui.actions.NewAction;
import org.netbeans.testspeceditor.ui.actions.NewTestSuiteAction;
import org.netbeans.testspeceditor.ui.actions.PasteAction;
/**
 *
 * @author ehucka
 */
public class SuitesNode extends Node {
    
    static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(SuitesNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/suites.png"));
        ActionsManager.getInstance().registerActions(SuitesNode.class,
                new NewAction(new NewTestSuiteAction()),  null, new PasteAction());
    }
    
    
    public SuitesNode(TestSpecNode root) {
        super(root, true);
        init();
        root.getTestSpec().getBeanSupport().addBeanChangeListener(this);
    }
    
    private void init() {
        TestSuite[] suites=((TestSpecNode)getParent()).getTestSpec().getTestSuites();
        for (TestSuite ts:suites) {
            TestSuiteNode<TestSuite> node=new TestSuiteNode(this, ts);
            addChild(node);
        }
    }
    
    @ActionAnnotation(actionClass=PasteAction.class)
    public boolean canPaste() {
        Node[] nds=NodesClipboard.getInstance().content();
        for (Node n:nds) {
            if (n.getClass() != TestSuiteNode.class) {
                return false;
            }
        }
        return true;
    }
    
    public String toString() {
        return "suites";
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
        if (source == ((TestSpecNode)getParent()).getTestSpec() && child instanceof TestSuite) {
            if (oldIndex == -1) { //new
                if (child instanceof TestSuite) {
                    TestSuiteNode<TestSuite> node=new TestSuiteNode(this, (TestSuite)child);
                    addChild(node, newIndex);
                    treeNodesInserted(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{newIndex}, new Object[] {node}));
                }
            } else if (newIndex == -1) { //old
                TestSuiteNode<TestSuite> node=(TestSuiteNode)getChildAt(oldIndex);
                if (node != null && node.getTestSuite() == child) {
                    int inx=remove(node);
                    treeNodesRemoved(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{inx}, new Object[] {node}));
                }
            } else {  //moved
                Node n=(Node)(children.remove(oldIndex));
                children.add(newIndex, n);
                treeStructureChanged(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{newIndex}, new Object[] {n}));
            }
        }
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public TreePath findNode(Bean bean) {
        for (int i = 0; i < getChildCount(); i++) {
            if (bean instanceof TestSuite) {
                if (((TestSuiteNode)getChildAt(i)).getTestSuite() == bean) {
                    return ((TestSuiteNode)getChildAt(i)).getTreePath();
                }
            } else {
                TreePath path = ((TestSuiteNode)getChildAt(i)).findNode(bean);
                if (path != null) {
                    return path;
                }
            }
        }
        return null;
    }
}
