/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.utils;

import java.util.HashMap;
import java.util.LinkedList;

/**
 *
 * @author ehucka
 */
public class Timer extends Thread {
    
    HashMap<Runnable, Event> runnables;
    LinkedList<Event> events;
    
    static Timer instance = null;
    
    boolean debug = false;
    
    public static Timer getInstance() {
        if (instance == null) {
            instance=new Timer();
            instance.start();
        }
        return instance;
    }
    
     
    protected Timer() {
        super("Event timer");
        runnables=new HashMap(64);
        events=new LinkedList();
    }
    
    /**
     * delay runnable, if the runnable is planned already it schedules it to (current time + delay)
     */
    public void delayOnce(Runnable runnable, long delay) {
        Event ev = runnables.get(runnable);
        if (ev != null) {
            synchronized(events) {
                events.remove(ev);
            }
        } else {
            ev = new OnceEvent();
            runnables.put(runnable, ev);
        }
        ev.runnable=runnable;
        ev.time=System.currentTimeMillis()+delay;
        int add=0;
        for (int i = 0; i < events.size(); i++) {
            if (events.get(i).time > ev.time) {
                add=i+1;
            } else {
                break;
            }
        }
        synchronized(events) {
            events.add(add, ev);
            if (debug) System.out.println("add once event "+ev+" delay "+delay+" to "+add+" events "+events);
            events.notify();
        }
    }
    
    /**
     * delay runnable, if the runnable is planned already it schedules it again to previous event time + delay;
     * where event time is time of calling this method + delay - each event will be invoked but in period of delay
     */
    public void delayAfter(Runnable runnable, long delay) {
        Event ev=runnables.get(runnable);
        long time=System.currentTimeMillis();
        
        if (ev != null && ev.time > time+delay) return;
        
        if (ev != null && ev.time+delay > time+delay) {
            time=ev.time+delay;
        } else {
            time+=delay;
        }
        ev=new Event();
        ev.runnable=runnable;
        ev.time=time;
        if (!events.contains(ev)) {
            int add=0;
            for (int i = 0; i < events.size(); i++) {
                if (events.get(i).time > ev.time) {
                    add=i+1;
                } else {
                    break;
                }
            }
            runnables.put(runnable, ev);
            synchronized(events) {
                events.add(add, ev);
                if (debug) System.out.println("delay after add new event "+ev+" delay "+delay+" to "+add+" events "+events);
                events.notify();
            }
        }
    }
    
    public void run() {
        Event ev;
        while (true) {
            try {
                if (events.isEmpty()) {
                    synchronized (events) {
                        if (events.isEmpty())
                            try {
                                events.wait();
                            } catch (InterruptedException ex) {
                            }
                    }
                }
                ev=events.getLast();
                if (debug) System.out.println("getLast event "+ev+"     "+events);
                long wt=ev.time-System.currentTimeMillis();
                if (wt > 0) {
                    if (debug) System.out.println("wait "+wt+"\n");
                    synchronized (events) {
                        try {
                            events.wait(wt);
                            if (debug) System.out.println("after wait "+wt);
                        } catch (InterruptedException ex) {
                        }
                    }
                    if (ev.time-System.currentTimeMillis() < 5) {
                        invoke(ev);
                    } else {
                        if (debug) System.out.println("event will not be invoked - postponed "+(ev.time-System.currentTimeMillis()));
                    }
                } else {
                    if (debug) System.out.println("not wait");
                    invoke(ev);
                }
            } catch (Exception ex) {   //sicher ist sicher
                ex.printStackTrace();
            }
        }
    }
    
    public void invoke(Event ev) {
        if (debug) System.out.println("invoked "+ev);
        if (events.remove(ev)) {
            try {
                ev.runnable.run();
            } catch (Exception ex) {   //if something throws from run of event - it mustn't break timer
                ex.printStackTrace();
            }
            if (runnables.get(ev.runnable) == ev) {
                runnables.remove(ev.runnable);
            }
        }
    }
    
    class Event {
        public long time;
        public Runnable runnable;
        
        public boolean equals(Object obj) {
            return (((Event)obj).time==time && ((Event)obj).runnable == runnable);
        }
        
        public String toString() {
            String t=runnable.toString();
            if (t.indexOf(".") > -1)
                t=t.substring(t.lastIndexOf(".")+1);
            return t+" "+(time-System.currentTimeMillis());
        }
    }
    
    class OnceEvent extends Event {
        
        public boolean equals(Object obj) {
            return ((Event)obj).runnable == runnable;
        }
        
        public String toString() {
            return "*"+super.toString();
        }
    }
}
