/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlizable;

import org.netbeans.testspeceditor.xmlparser.Tag;
import org.netbeans.testspeceditor.xmlparser.TagFactory;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;



/**
 * Generate XML from a XMlizable object. Store all single objects and all Collection objects.
 * It doesn't know to store Map yet.
 *
 * @author ehucka
 */
public class XMLSource {
    
    HashMap<Object, Sign> storedobjects=new HashMap(128);
    HashMap<String, Object> objectsids=new HashMap(128);
    
    ArrayList<XMLizable> queue=new ArrayList(128);
    ArrayList<XMLizable> stack=new ArrayList(128);
    
    XMLizable object;
    XMLUtils utils;
    int deep=0;
    boolean init=false;
    
    
    public XMLSource(XMLizable object) {
        this(object, XMLUtils.getUtils());
    }
    
     
    public XMLSource(XMLizable object, XMLUtils utils) {
        this.object=object;
        this.utils=utils;
        this.utils.initIdGenerator();
    }
    
    public Tag getTag() throws XMLizableException {
        try {
            Tag tag=getTag(object);
            storedobjects.clear();
            return tag;
        } catch (Exception ex) {
            throw new XMLizableException(ex);
        }
    }
    
    public XMLUtils getUtils() {
        return utils;
    }
    
    protected Tag getTag(XMLizable obj) throws Exception {
        Tag main=utils.getTagForObject(obj);
        //has own xmlization method
        if (utils.callGetTag(obj, main)) {
            return main;
        }
        if (!init) {
            queue.add(object);
            prestoreObject(object, false);
            solveQueue();
            init=true;
        }
        ObjectSign sign=(ObjectSign)(storedobjects.get(obj));
        for (int i=0;i < sign.fields.size();i++) {
            Sign s=(Sign)(sign.fields.get(i));
            if (s instanceof ObjectSign) { //full stored objects
                ObjectSign os=(ObjectSign)s;
                Tag t=getTag((XMLizable)os.object);
                //if the object is linked and doesn't have name - there should be add a linked attribute
                if (os.linked && !utils.isAggregateOnly()) {// && XMLUtils.hasName(os.object.getClass()) == null) {
                    t.setAttribute(XMLUtils.ID_ATTRIBUTE, (String)(os.id));
                }
                //test if the field tag should be created
                /*ClassUsageSign cs=(ClassUsageSign)(storedobjects.get(f));
                if (cs == null || !cs.once) { //more objcols of this class has the same type - create field tag
                    Tag obf=TagFactory.createTag(f.getName());
                    main.addChild(obf);
                    obf.addChild(t);
                } else { //single object - can be without field tag
                    main.addChild(t);
                }*/
                main.addChild(t);/////////////////////////////////////////
            } else if (s instanceof LinkObjectSign) { //links
                linkTag(main, s);
            } else if (s instanceof CollectionSign) { //collections
                CollectionSign colsign=(CollectionSign)s;
                
                for (int j=0;j < colsign.objects.size();j++) {
                    Object o=colsign.objects.get(j);
                    if (o instanceof LinkObjectSign || o instanceof PrimitiveSign) {
                        colsign.links++;
                    }
                }
                if (colsign.links == colsign.objects.size()) { //all are as links
                    linkTag(main, colsign);
                } else {
                    Tag asst=null;
                    for (int j=0;j < colsign.objects.size();j++) {
                        Sign sgn=(Sign)(colsign.objects.get(j));
                        if (asst == null && (colsign.links > 0 || colsign.links == 0 && !colsign.alone)) {
                            asst=TagFactory.createTag(colsign.fieldname);
                        }
                        if (sgn instanceof ObjectSign) { //store whole objects
                            Tag ret=getTag(((ObjectSign)sgn).object);
                            //if the object is linked and doesn't have name - there should be add a linked attribute
                            if (((ObjectSign)sgn).linked && !utils.isAggregateOnly()) {// && XMLUtils.hasName(((ObjectSign)sgn).object.getClass()) == null) {
                                ret.setAttribute(XMLUtils.ID_ATTRIBUTE, (String)(((ObjectSign)sgn).id));
                            }
                            if (asst != null)
                                asst.addChild(ret);
                            else
                                main.addChild(ret);
                        } else { //store links of collection
                            linkTag(XMLUtils.LINKS_ATTRIBUTE, asst, sgn);
                        }
                    }
                    if (asst != null) {
                        main.addChild(asst);
                    }
                }
            } else {  //collect of primitive types
                linkTag(main, s);
            }
        }
        
        return main;
    }
    
    
    protected void linkTag(Tag mainTag, Sign object) {
        linkTag(object.fieldname, mainTag, object);
    }
    
    protected void linkTag(String fieldName, Tag mainTag, Sign object) {
        String idd=null;
        if (object instanceof LinkObjectSign) {
            idd=(String)(((LinkObjectSign)object).sign.id);
        } else if (object instanceof PrimitiveSign) {
            idd=((PrimitiveSign)object).object;
        } else if (object instanceof CollectionSign) {
            CollectionSign cs=(CollectionSign)object;
            StringBuilder sb=new StringBuilder();
            for (int i=0;i < cs.objects.size();i++) {
                Sign sg=(Sign)(cs.objects.get(i));
                if (sg instanceof LinkObjectSign) {
                    sb.append(((LinkObjectSign)sg).sign.id);
                } else if (sg instanceof PrimitiveSign) {
                    sb.append(((PrimitiveSign)sg).object);
                }
                if (i < cs.objects.size()-1) {
                    sb.append(", ");
                }
            }
            idd=sb.toString();
        }
        String v=mainTag.getAttribute(fieldName);
        if (v != null && v.length() > 0) { //multi link
            v+=", "+idd;
        } else {
            v=idd;
        }
        mainTag.setAttribute(fieldName, v);
    }
    
    
    private void solveQueue() throws Exception {
        while (queue.size() > 0) {
            for (int remove=queue.size(); remove > 0;remove--) {
                preprocess((XMLizable)(queue.remove(remove-1)));
            }
        }
    }
    /**
     * Watch into the storing bean's structure and set which associtaion will be stored and which will be linked and its linking style.
     *
     */
    protected void preprocess(XMLizable object) throws Exception {
        if (stack.contains(object)) return;
        stack.add(object);
        deep++;
        
        ObjectSign sign=(ObjectSign)(storedobjects.get(object));
        //collect objcols - mulit associations * to N
        ArrayList<Field> colfields=utils.getAllCollectionFields(object);
        for (Field f:colfields) {
            Collection collect=(Collection)(f.get(object));
            if (collect != null && collect.size() > 0) {
                CollectionSign lsgn=new CollectionSign();
                sign.fields.add(lsgn);
                lsgn.fieldname=f.getName();
                //TODO - better logic for non-field tags - find all with possible same names
                lsgn.alone=(colfields.size() == 1 && utils.getAllTypeFields(object, XMLizable.class).size() == 0);
                for (Object o:collect) {
                    Sign os=prestoreObject(o, true);
                    lsgn.objects.add(os);
                    if (o instanceof XMLizable) {
                        queue.add((XMLizable)o);
                    }
                }
            }
        }
        
        //single associtaion - * to 1
        ArrayList<Field> singfields=utils.getAllSingleFields(object);
        
        HashMap<Class, String> types=new HashMap(32);
        //separate how often is there used such field type
        for (Field f:singfields) {
            if (types.get(f.getType()) == null)
                types.put(f.getType(), "once");
            else
                types.put(f.getType(), "manytimes");
        }
        //single objcols
        for (Field f: singfields) {
            Object v=f.get(object);
            if (v != null) {
                Sign os=prestoreObject(v, false);
                sign.fields.add(os);
                os.fieldname=f.getName();
                if (v instanceof XMLizable) {
                    queue.add((XMLizable)v);
                    ClassUsageSign cs=(ClassUsageSign)(storedobjects.get(f));
                    if (cs == null) {
                        cs=new ClassUsageSign(); //if the type of objects is only once in objcols set - it can be stored without field tag
                        if (types.get(f.getType()).equals("once")) {
                            cs.once=true;
                        }
                        storedobjects.put(f, cs);
                    }
                }
            }
        }
        deep--;
    }
    
    private Sign prestoreObject(Object obj, boolean agressivePrimitives) throws Exception {
        if (obj instanceof XMLizable) {
            //get obj id
            Field nameField=utils.hasName(obj.getClass());
            String id=null;
            if (nameField != null) {
                try {
                    id=(String)(nameField.get(obj));//map by name
                    if (objectsids.get(id) != null && objectsids.get(id) != obj) {
                        id=null;//the name has already another obj
                    }
                } catch (Exception ex) {}
            }
            boolean linked=false;
            if (id == null) {
                linked=true;
                id=findID();
            }
            objectsids.put(id, obj);
            ObjectSign os=null;
            if (storedobjects.get(obj) != null) {
                os=(ObjectSign)storedobjects.get(obj);
                if (linked && !os.linked) {
                    os.linked=linked;
                }
                os.id=id;
                return new LinkObjectSign(os);
            } else {
                os=new ObjectSign();
                os.id=id;
                os.linked=linked;
                os.object=(XMLizable)obj;
                storedobjects.put(obj, os);
            }
            return os;
        } else { //primitives
            String s=XMLUtils.convertToString(obj);
            if (agressivePrimitives) {
                Object oid=objectsids.get(s);
                if (oid == null) {
                    objectsids.put(s, obj);
                } else if (oid instanceof XMLizable) {
                    objectsids.put(s, obj);//string should be always mapped by name
                    prestoreObject(oid, false);
                }
            }
            return new PrimitiveSign(s);
        }
    }
    
    private String findID() {
        String id=String.valueOf(utils.getNextID());
        while (objectsids.get(id) != null) {
            id=String.valueOf(utils.getNextID());
        }
        return id;
    }
    
    abstract class Sign {
        String fieldname=null;
    }
    
    class PrimitiveSign extends Sign {
        public String object=null;
        public PrimitiveSign(String obj) {
            object=obj;
        }
    }
    
    class CollectionSign extends Sign {
        public ArrayList objects=new ArrayList(32);
        public int size=0;
        public int deepness=0;
        public double rate;
        public int links=0;
        public boolean alone=false;
        
        public void setRate(int maxsize, int mindeepness) {
            rate=0.4*(mindeepness/deepness)+0.6*(size/maxsize);
        }
        
        public String toString() {
            return "<size="+String.valueOf(size)+", deep="+String.valueOf(deepness)+">";
        }
    }
    
    class ObjectSign extends Sign {
        public ArrayList<Sign> fields=new ArrayList(32);
        boolean linked=false;
        public Object id;
        public XMLizable object=null;
        
        public boolean isLinked() {
            return linked;
        }
        
        public String toString() {
            return "<id="+id.toString()+", linked="+String.valueOf(linked)+">";
        }
    }
    
    class LinkObjectSign extends Sign {
        public ObjectSign sign;
        
        public LinkObjectSign(ObjectSign sign) {
            this.sign=sign;
        }
    }
    
    class ClassUsageSign extends Sign {
        public boolean once=false;
        public String toString() {
            return "<once="+String.valueOf(once)+">";
        }
    }
}
