/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Query;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.QuizQuestion;
import org.netbeans.modules.quiz.entities.Response;
import org.netbeans.modules.quiz.entities.ResponseItem;
import org.netbeans.modules.quiz.entities.UserdirLogging;
import static org.netbeans.modules.quiz.AcceptingServlet.ErrorStatus.*;

/**
 *
 * @author  Jindrich Sedek
 */
public class AcceptingServlet extends HttpServlet {

    private static final Logger LOG = Logger.getLogger(AcceptingServlet.class.getName());
    private static PersistenceUtils pUtils = PersistenceUtils.getInstance();
    private ErrorStatus errorStatus;
    
    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        LOG.info("Connection accepted");
        NbUser user = processUser(request);
        if (user == null) {
            missingAttributeRedirect(request, response);
            return;
        }
        String quizIdStr = request.getParameter("quizid");
        Quiz quiz = null;
        EntityManager em = pUtils.createEntityManager();
        em.getTransaction().begin();
        if (quizIdStr != null){
            Integer quizId = Integer.parseInt(quizIdStr);
            quiz = Quiz.findById(em, quizId);
        }
        if (quiz == null){
            quiz = pUtils.getActualQuiz();
        }
        if (!Response.isAlreadySubmitted(user, quiz)) {
            Response userResp = new Response(user, quiz);
            ArrayList<ResponseItem> items = new ArrayList();
            for (int i = 1; i <= quiz.getQuestionCount(); i++) {
                String qResponse = request.getParameter("question" + i);
                if ((qResponse == null)||(qResponse.length() == 0)){
                    errorStatus = MISSING_RESPONSE;
                    missingAttributeRedirect(request, response);
                    em.getTransaction().rollback();
                    em.close();
                    return;
                }
                QuizQuestion corrQuest = getCorresponding(quiz, i);
                boolean isCorrect = QuizUtils.isCorrectResponse(qResponse, 
                        corrQuest.getResponseText(), corrQuest.getQuestionType());
                items.add(new ResponseItem(userResp, i, qResponse, isCorrect));
            }
            userResp.setResponseItemCollection(items);
            em.persist(userResp);
            Mailer.sendResponseConfiramtion(quiz, user);
        }else{
            LOG.severe("User has submitted the quiz twice!");
        }
        em.getTransaction().commit();
        em.close();
        PersistenceUtils.addOpenQuizes(user, request);
        request.setAttribute("userid", user.getNbId());
        
        LOG.info("Accepting finished");
        request.getRequestDispatcher("accepted.jsp").forward(request, response);
    }

    private QuizQuestion getCorresponding(Quiz quiz, int order) {
         Collection<QuizQuestion> questions = quiz.getQuizQuestionCollection();
         for (QuizQuestion quizQuestion : questions) {
            if (quizQuestion.getQuizQuestionPK().getRecord() == order){
                return quizQuestion;
            }
        }
        return null;
    }
    
    private void missingAttributeRedirect(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException{
            LOG.warning("Missing fields " + errorStatus);
            request.setAttribute("email", request.getParameter("email"));
            request.setAttribute("username", request.getParameter("username"));
            request.setAttribute("university", request.getParameter("university"));
            request.setAttribute("userid", request.getParameter("userid"));
            request.setAttribute("quizid", request.getParameter("quizid"));
            switch(errorStatus){
                case INCORRECT_EMAIL:
                    request.setAttribute("incorrect_email", true);
                    break;
                case MISSING_REQUIRED:
                    request.setAttribute("missing_required", true);
                    break;
                case MISSING_RESPONSE:
                    request.setAttribute("missing_response", true);
                    break;
            }
            request.getRequestDispatcher("/quiz").forward(request, response);
    }
    
    protected String getIP(HttpServletRequest request) {
        String headerIP = request.getHeader("x-forwarded-for");
        if (headerIP == null) {
            return request.getRemoteHost();// no Lighttpd

        }
        if (headerIP != null) {
            String[] array = headerIP.split(",");
            if (array.length > 1) {
                headerIP = array[array.length - 1];
            }
        }
        return headerIP;
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Short description";
    }

    // </editor-fold>
    NbUser processUser(HttpServletRequest request) {
        String userId = request.getParameter("userid");
        String email = request.getParameter("email");
        String name = request.getParameter("username");
        String address = request.getParameter("country");
        String univerzity = request.getParameter("university");
        String notification = request.getParameter("notification");

        if ((email == null) || (name == null) || (address == null) ||
                (email.length() == 0) || (name.length() == 0) || (address.length() == 0)) {
            errorStatus = MISSING_REQUIRED;
            return null;
        }
        if (email.indexOf('@') == -1){
            errorStatus = INCORRECT_EMAIL;
            return null;
        }
        EntityManager em = pUtils.createEntityManager();
        em.getTransaction().begin();
        NbUser user = NbUser.findById(em, userId);
        if (user == null) {
            Query q = em.createNamedQuery("NbUser.findByEmail").setParameter("email", email);
            try {
                user = (NbUser) q.getSingleResult();
                //find existing E-mail -> merge users!
                LOG.info("Changing user id according to e-mail " + email);
                em.persist(new UserdirLogging(user.getNbId(), new Date(), email));
            } catch (NoResultException exc) {
                // no email found
            }
        }        
        if (user == null) {
            LOG.info("Adding new user " + userId);
            String ip = getIP(request);
            user = new NbUser(userId, name, email, address, univerzity, ip);
            em.persist(user);
        }else{
            //update data
            user.setAddress(address);
            user.setNbId(userId);
            user.setEmail(email);
            user.setUniversity(univerzity);
            user.setUsername(name);
        }
        if (notification != null) {
            user.setSendNotification(1);
        } else {
            user.setSendNotification(0);
        }
        em.merge(user);
        em.getTransaction().commit();
        em.close();
        return user;
    }
    
    ErrorStatus getErrorStatus(){
        return errorStatus;
    }

    enum ErrorStatus{
        INCORRECT_EMAIL, MISSING_REQUIRED, MISSING_RESPONSE
    }
}
