/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.quiz;

import java.io.FileOutputStream;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.Winners;

/**
 *
 * @author ms113234
 */
public class RssGenerator {

    /**
     * 0 - currentDate
     */
    private static final MessageFormat HEADER_FRMT = new MessageFormat(
            "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
            "\n" +
            "<!DOCTYPE rss PUBLIC \"-//Netscape Communications//DTD RSS 0.91//EN\" " +
            "\"http://my.netscape.com/publish/formats/rss-0.91.dtd\">\n" +
            "<rss version=\"0.91\">\n" +
            "    <channel>\n" +
            "        <title>NetBeans Quiz</title>\n" +
            "        <description>NetBeans Quiz</description>\n" +
            "        <link>http://www.netbeans.org/index.html</link>\n" +
            "        <language>en</language>\n" +
            "        <webMaster>webmaster@netbeans.org (NetBeans webmaster)</webMaster>\n" +
            "        <image>\n" +
            "            <title>NetBeans Quiz</title>\n" +
            "            <url>http://www.netbeans.org/images/v5/nb-logo.gif</url>\n" +
            "            <link>http://www.netbeans.org/index.html</link>\n" +
            "            <width>110</width>\n" +
            "            <height>30</height>\n" +
            "        </image>\n" +
            "        <pubDate>{0,date,full}</pubDate>\n");
    /**
     * 0 - context URL
     * 1 - quiz id
     */
    private static final MessageFormat ACTUAL_FRMT = new MessageFormat(
            "        <item>\n" +
            "            <title>New Quiz Available</title>\n" +
            "            <link>{0}/quiz?quizId={1}</link>\n" +
            "            <description>New weekly NetBeans quiz available.</description>\n" +
            "        </item>\n");
    /**
     * 0 - context URL
     * 1 - quiz id
     * 2 - quiz start date
     * 3 - quiz end date
     * 4 - quiz winner name
     */
    private static final MessageFormat WINNER_FRMT = new MessageFormat(
            "            <item>\n" +
            "                <title>{2,date,long} - {3,date,long}\n" +
            "                week''s Quiz answers and winner.</title>\n" +
            "                <link>{0}/quiz?id={1}</link>\n" +
            "                <description>The winner of the quiz is {4}.\n" +
            "                We are congratulate and sending the prize.</description>\n" +
            "            </item>\n");
    private static final String FOOTER =
            "    </channel>\n" +
            "</rss>\n";

    private RssGenerator() {
    }

    public static void refreshRss() throws IOException {
        String contextUrl = System.getProperty("quiz.rss.pages");
        String rssFile = System.getProperty("quiz.rss.location");
        StringBuffer sb = new StringBuffer();
        HEADER_FRMT.format(new Object[]{new Date()}, sb, null);
        // add actual quiz item
        Quiz actual = PersistenceUtils.getInstance().getActualQuiz();
        if (actual != null) {
            ACTUAL_FRMT.format(new Object[]{contextUrl, actual.getId()}, sb, null);
        }
        // add (max 5) previous winners
        List<Winners> winners = PersistenceUtils.getAll(Winners.class);
        Collections.sort(winners);
        Collections.reverse(winners);
        if (winners.size() > 5) {
            winners = winners.subList(0, 5);
        }
        for (Winners winner : winners) {
            Quiz quiz = winner.getQuiz();
            WINNER_FRMT.format(new Object[]{contextUrl, quiz.getId(), quiz.getStartdate(),
                        quiz.getEnddate(), winner.getNbuser().getUsername()
                    }, sb, null);
        }
        sb.append(FOOTER);
        synchronized (RssGenerator.class) {
            FileOutputStream fos = new FileOutputStream(rssFile);
            fos.write(sb.toString().getBytes("UTF-8"));
            fos.close();
        }
    }
}