/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.quiz.entities;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import org.netbeans.modules.quiz.QuestionType;

/**
 *
 * @author Jindrich Sedek
 */
@Entity
@Table(name = "QUIZ_QUESTION")
@NamedQueries({@NamedQuery(name = "QuizQuestion.findByQuizId", query = "SELECT q FROM QuizQuestion q WHERE q.quizQuestionPK.quizId = :quizId"),
@NamedQuery(name = "QuizQuestion.findByRecord", query = "SELECT q FROM QuizQuestion q WHERE q.quizQuestionPK.record = :record"),
@NamedQuery(name = "QuizQuestion.findByQuestion", query = "SELECT q FROM QuizQuestion q WHERE q.question = :question"),
@NamedQuery(name = "QuizQuestion.findByQuestionType", query = "SELECT q FROM QuizQuestion q WHERE q.questionType = :questionType"),
@NamedQuery(name = "QuizQuestion.findByResponseText", query = "SELECT q FROM QuizQuestion q WHERE q.responseText = :responseText"),
@NamedQuery(name = "QuizQuestion.count", query = "SELECT count(q) FROM QuizQuestion q"),
@NamedQuery(name = "QuizQuestion.countByCategoryId", query = "SELECT count(q) FROM QuizQuestion q WHERE q.category.id = :category")
})
public class QuizQuestion implements Serializable {
    private static final long serialVersionUID = 1L;
    @EmbeddedId
    protected QuizQuestionPK quizQuestionPK;
    @Column(name = "QUESTION")
    private String question;
    @Column(name = "QUESTION_TYPE")
    private Integer questionType;
    @Column(name = "RESPONSE_TEXT")
    private String responseText;
    @JoinColumn(name = "QUIZ_ID", referencedColumnName = "ID", insertable = false, updatable = false)
    @ManyToOne
    private Quiz quiz;
    @JoinColumn(name = "CATEGORY", referencedColumnName = "ID")
    @ManyToOne
    private Category category;


    public QuizQuestion() {
    }

    public QuizQuestion(Quiz quiz, int record, QuestionType type, String responseText) {
        this.quizQuestionPK = new QuizQuestionPK(quiz.getId(), record);
        setQuestionType(type);
        this.responseText = responseText;
        this.quiz = quiz;
    }

    public QuizQuestion(Quiz quiz, int record, QuestionType type, String responseText, String questionText) {
        this.quizQuestionPK = new QuizQuestionPK(quiz.getId(), record);
        setQuestionType(type);
        this.responseText = responseText;
        this.quiz = quiz;
        this.question = questionText;
    }
    
    public QuizQuestion(Quiz quiz, int record, QuestionType type, String responseText, String questionText, Category category) {
        this.quizQuestionPK = new QuizQuestionPK(quiz.getId(), record);
        setQuestionType(type);
        this.responseText = responseText;
        this.quiz = quiz;
        this.question = questionText;
        this.category = category;
    }

    public QuizQuestionPK getQuizQuestionPK() {
        return quizQuestionPK;
    }

    public void setQuizQuestionPK(QuizQuestionPK quizQuestionPK) {
        this.quizQuestionPK = quizQuestionPK;
    }

    public String getQuestion() {
        return question;
    }

    public void setQuestion(String question) {
        this.question = question;
    }

    public QuestionType getQuestionType() {
        return QuestionType.values()[questionType];
    }

    public void setQuestionType(QuestionType type) {
        this.questionType = type.ordinal();
    }

    public Category getCategory() {
        return category;
    }

    public void setCategory(Category category) {
        this.category = category;
    }

    public String getResponseText() {
        return responseText;
    }

    public void setResponseText(String responseText) {
        this.responseText = responseText;
    }

    public Quiz getQuiz() {
        return quiz;
    }

    public void setQuiz(Quiz quiz) {
        this.quiz = quiz;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (quizQuestionPK != null ? quizQuestionPK.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof QuizQuestion)) {
            return false;
        }
        QuizQuestion other = (QuizQuestion) object;
        if ((this.quizQuestionPK == null && other.quizQuestionPK != null) || (this.quizQuestionPK != null && !this.quizQuestionPK.equals(other.quizQuestionPK))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.quiz.entities.QuizQuestion[quizQuestionPK=" + quizQuestionPK + "]";
    }

}
