/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.Response;
import static org.junit.Assert.*;

/**
 *
 * @author Jindrich Sedek
 */
public class TestAddOpenQuizes {
    NbUser user;
    
    public TestAddOpenQuizes() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        PersistenceUtils.setDefaultPu(PersistenceUtils.TEST_PU);
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    @Test
    public void testAddOpenQuizes() throws Exception {
        user = new NbUser("usrId", "Tonda", "Tonda@omacka", "Indi", "Dilli", "10.10.10.2");
        PersistenceUtils.persist(user);
        Quiz quiz = null;
        for (int i = 1; i < 10; i++) {
            Date start = new Date(107, i, 10);
            Date end = new Date(107, i + 1, 10);
            quiz = new Quiz(Quiz.suppressId(), start, end, "IPod");
            PersistenceUtils.persist(quiz);
        }
        verifySize(9);//all quizes
        
        quiz.setClosed(1);
        PersistenceUtils.merge(quiz);
        verifySize(8);//one is already closed id = 9
        
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        quiz = em.find(Quiz.class, 2);
        quiz.setStartdate(new Date(120, 1, 2));
        PersistenceUtils.merge(quiz);
        verifySize(7);// one is closed, on is in the future

        quiz = em.find(Quiz.class, 3);
        Response userResp = new Response(user, quiz);
        PersistenceUtils.persist(userResp);
        verifySize(6);// one is closed, one is in the future, one was responsed
        
    }

    private void verifySize(int size) {
        RequestWrapper request = new RequestWrapper();
        PersistenceUtils.addOpenQuizes(user, request);
        List<Integer> openQuizes = (List<Integer>) request.getAttribute("openQuizes");
        assertEquals(size, openQuizes.size());

    }
}