/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted;

import java.awt.Point;
import java.util.ArrayList;
import javax.swing.event.*;
import javax.swing.undo.*;
import java.util.Iterator;
import java.awt.Dimension;
import java.util.Set;
import java.util.HashSet;
import org.netbeans.modules.regsup.debug.RexBreakpoint;
import java.awt.Rectangle;
import java.util.Collections;

/**
 *
 * @author  Petr Slegr
 * @version 25.10.01/1
 */
public class AutomatonEditor implements UndoableEditListener {
    
    private Point currentPosition;
    
    /**
     */
    private ArrayList selectedElements;

    private Set errorElements = new HashSet ();
    /**
     */
    private Set activeElements;
    
    /**
     */
    private Set activeInContainers = new HashSet ();
    
    private UndoManagerPlus undoManager;
    
    private VisualAutomatonContainer container;
    
    // !!!???
    private AutomatonPanel automatonPanel;
    
    private PositionListenPoint positionListener;
    
    /**
     */
    private Set breakpoints;
    
    /** Utility field holding list of AutomatonEditorListeners. */
    private transient java.util.ArrayList automatonEditorListenerList;
    
    /**
     */
    AutomatonEditor (VisualAutomatonContainer container) {
        init ();
        this.container = container;
        ContainerListenPoint listener = new ContainerListenPoint ();
        /* x3
        container.addAutomatonDocumentListener (listener);
         */
        container.addContainerListener (listener);
        container.addUndoableEditListener (this);
        // !!!!!!!!!!!! pochybeny
        if (null != automatonPanel) {
            // checks the positions of elements
            Iterator it = container.elements ().iterator ();
            Point pos;
            while (it.hasNext ()) {
                VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
                pos = elem.getPosition ();
                if (exceedsAutomatonPanel (pos)) {
                    /* x2
                    window.enlargeAutomatonPanel (pos);
                     */
                }
            }
        }
    }
    
    /**
     */
    private void init () {
        //window.setEditor (this); // registers editor for callback
        selectedElements = new ArrayList ();
        activeElements = new HashSet ();
        breakpoints = new HashSet ();
        undoManager = new UndoManagerPlus ();
        positionListener = new PositionListenPoint ();
    }
    
    /** Checks if this editor is for root VisualAutomatonContainer
     */
    public boolean isForRoot () {
        return container.isRoot ();
    }
    
    private boolean exceedsAutomatonPanel (Point position) {
        Dimension dim = automatonPanel.getSize ();
        return true ? position.getX () > dim.getWidth () || position.getY () > dim.getHeight () : false;
    }
    
    /**
     */
    /*
    public void makeElementVisible (VisualAutomatonElement element) {
        if (! container.contains (element)) {
            throw new IllegalArgumentException ("this editor does not contain element: " + element);
        }
        if (null == automatonPanel) {
            return;
        }
        automatonPanel.getBasePanel ().scrollRectToVisible (new Rectangle ((int) element.getPosition ().getX (),
        (int) element.getPosition ().getY (), (int) element.getPosition ().getX () + 50,
        (int) element.getPosition ().getY () + 50));
        // !! const
    }
     */
    
    
    /**
     */
    public VisualAutomatonContainer getContent () {
        return container;
    }
    
    /** Method to be called when set of active elements get changed
     */
    private void activeElementsChanged () {
        // !!! radsi firovat eventu a nedrzet v editoru panel
        if (automatonPanel != null) {
            automatonPanel.repaint ();
        }
        //System.out.println ("actives changed (hack)");
    }
    
    private void activeInContainersChanged () {
        // !!! radsi firovat eventu a nedrzet v editoru panel
        // !!!!!???? bacha hack
        if (automatonPanel != null) {
            automatonPanel.repaint ();
        }
    }
    
    /**
     */
    void addSelectedElement (VisualAutomatonElement elem) {
        if (! selectedElements.contains (elem)) {
            selectedElements.add (elem);
        }
        selectedElementsChanged ();
        //System.out.println ("selected elements: " + selectedElements);
    }
    
    /**
     */
    public void removeSelectedElement (VisualAutomatonElement elem) {
        boolean res = selectedElements.remove (elem);
        if (! res) {
            throw new IllegalStateException ("selectedElements does not contain: " + elem);
        }
        selectedElementsChanged ();
        //System.out.println ("selected elements: " + selectedElements);
    }
    
    /**
     */
    public boolean containsSelectedElement (VisualAutomatonElement elem) {
        return selectedElements.contains (elem);
    }
    
    /**
     */
    public void resetSelectedElements () {
        selectedElements.removeAll (selectedElements);
        //System.out.println ("selected elements: " + selectedElements);
        selectedElementsChanged ();
    }
    
    /**
     */
    public VisualAutomatonElement[] getSelectedElements () {
        VisualAutomatonElement[] type = new VisualAutomatonElement[0];
        return (VisualAutomatonElement []) selectedElements.toArray (type);
    }
    
    // !!!! name collision
    public Set getSelected () {
        return new HashSet (selectedElements);
    }
    
    /**
     * @param do not use null, use empty set instead
     */
    public void setErrorElements (Set elements) {
        errorElements = elements;
        errorElementsChanged ();
    }
    
    public void resetErrorElements () {
        errorElements = Collections.EMPTY_SET;
        errorElementsChanged ();
    }
    
    public Set errorElements () {
        return new HashSet (errorElements);
    }
    
    /**
     */
    public void setActiveElements (Set activeElements) {
        this.activeElements = activeElements;
        activeElementsChanged ();
    }
    
    /**
     */
    public void addActiveElement (VisualAutomatonElement element) {
        // !!! check duplicity
        boolean res = activeElements.add (element);
        //System.out.println ("active element added into editor: " + this);
        activeElementsChanged ();
    }
    
    /**
     */
    public void addActiveInContainer (VisualAutomatonContainer container) {
        // !!! check duplicity
        boolean res = activeInContainers.add (container);
        //System.out.println("active element added into editor: " + this);
        activeInContainersChanged ();
    }
    
    /**
     */
    public void resetActiveInContainers () {
        activeInContainers = new HashSet ();
        activeInContainersChanged ();
    }
    
    /**
     */
    public void resetActiveElements () {
        activeElements = new HashSet ();
        //System.out.println ("active elements reset in editor: " + this);
        activeElementsChanged ();
    }
    
    public boolean containsActiveElement (VisualAutomatonElement element) {
        return activeElements.contains (element);
    }
    
    /**
     */
    // ! note element is of VisualAutomatonContainer type
    public boolean containsActiveInContainer (VisualAutomatonElement element) {
        return activeInContainers.contains (element);
    }
    
    /**
     */
    public void addBreakpoint (RexBreakpoint brk) {
        // !! ma smysl pouze u root editoru
        boolean res = breakpoints.add (brk);
        if (! res) {
            throw new IllegalStateException ("Breakpoint already added: " + brk);
        }
    }
    
    /**
     */
    public void removeBreakpoint (RexBreakpoint brk) {
        // !! ma smysl pouze u root editoru
        boolean res = breakpoints.remove (brk);
        if (! res) {
            throw new IllegalStateException ("Breakpoint not added: " + brk);
        }
    }
    
    public Set getBreakpoints () {
        return new HashSet (breakpoints);
    }
    
    // !!!!!!???
    public UndoManager getUndoManager () {
        return undoManager;
    }
    
    public Point getCurrentPosition () {
        return currentPosition;
    }
    
    void setCurrentPosition (Point currentPosition) {
        this.currentPosition = currentPosition;
    }
    
    /**
     * An undoable edit happened
     */
    // !! predelat listener na inner tridu
    public void undoableEditHappened (UndoableEditEvent event) {
        //System.out.println("undoEdit class: " + event.getEdit ().getClass ());
        UndoableEdit edit = event.getEdit ();
        //System.out.println("can undo: " + edit.canUndo ());
        undoManager.addEdit (event.getEdit ());
    }
    
    /** If this editor is for root VisualAutomatonContainer saves it to file else
     * throws IllegalStateException.
     * @throw IllegalStateException
     */
    public void save () {
        if (! isForRoot ()) {
            throw new IllegalStateException ("Container is not root. Cannot be saved");
        }
        getContent ().notifySave ();
    }
    
    private void selectedElementsChanged () {
        fireSelectedElementsChanged (new AutomatonEditorEvent (this));
    }
    
    private void errorElementsChanged () {
        fireErrorElementsChanged (new AutomatonEditorEvent (this));
    }
    
    /** Registers AutomatonEditorListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addAutomatonEditorListener (AutomatonEditorListener listener) {
        if (automatonEditorListenerList == null ) {
            automatonEditorListenerList = new java.util.ArrayList ();
        }
        automatonEditorListenerList.add (listener);
    }
    
    /** Removes AutomatonEditorListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeAutomatonEditorListener (AutomatonEditorListener listener) {
        if (automatonEditorListenerList != null ) {
            automatonEditorListenerList.remove (listener);
        }
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireSelectedElementsChanged (AutomatonEditorEvent event) {
        java.util.ArrayList list;
        synchronized (this) {
            if (automatonEditorListenerList == null) return;
            list = (java.util.ArrayList)automatonEditorListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((AutomatonEditorListener)list.get (i)).selectedElementsChanged (event);
        }
    }

    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireErrorElementsChanged (AutomatonEditorEvent event) {
        java.util.ArrayList list;
        synchronized (this) {
            if (automatonEditorListenerList == null) return;
            list = (java.util.ArrayList)automatonEditorListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((AutomatonEditorListener)list.get (i)).errorElementsChanged (event);
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class ContainerListenPoint implements /* x3 AutomatonDocumentListener, */
    VisualAutomatonContainerListener {
        
        
        public void elementAdded (VisualAutomatonContainerEvent event) {
            /*
            event.getElement ().addPropertyChangeListener (VisualAutomatonElement.PROP_POSITION,
            positionListener);
             */
            /* x3
            VisualAutomatonElement elem = event.getElement ();
            if (exceedsAutomatonPanel (elem.getPosition ())) {
                window.enlargeAutomatonPanel (elem.getPosition ());
            }
             */
        }
        
        public void elementRemoved (VisualAutomatonContainerEvent event) {
            /*
            event.getElement ().removePropertyChangeListener (VisualAutomatonElement.PROP_POSITION,
            positionListener);
             */
        }
        
        public void containerRemoved (VisualAutomatonContainerEvent event) {
        }
        
        public void containerAdded (VisualAutomatonContainerEvent event) {
        }
        
        public void contentChanged (VisualAutomatonContainerEvent event) {
            // !!!!???? docasne
            //automatonPanel.repaint (); // was bug: was missing
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     */
    private class PositionListenPoint implements java.beans.PropertyChangeListener {
        
        public void propertyChange (java.beans.PropertyChangeEvent evt) {
            /*
            VisualAutomatonElement elem = (VisualAutomatonElement) evt.getSource ();
            if (! isPositionWithin (elem.getPosition ())) {
                window.enlargeAutomatonPanel (elem.getPosition ());
            }
             */
        }
        
    }
    
}

