/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.util.Util;
import java.util.Enumeration;
import java.awt.Frame;
import javax.swing.JFrame;

/**
 *
 * @author  Petr Slegr
 * @version 26.11.01/2
 */
public class EdgeValueDialog extends javax.swing.JDialog {
    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;
    
    //private String edgeValue;
    
    private RexEdgeElement edgeValue;
    
    private String UPPER_CASE_OPTION = "upper case";
    
    private String LOWER_CASE_OPTION = "lower case";
    
    private String BOTH_CASE_OPTION = "upper and lower";
    
    private RexEdgeElement upperLetter;
    
    private RexEdgeElement lowerLetter;
    
    private RexEdgeElement bothLetter;
    
    private RexEdgeElement alfaNum;
    
    // !! trapnej hack, prozkoumat ButtonGroup
    private int lastSelected;
    
    private static final int SELECTED_INTERVAL = 1;
    
    private static final int SELECTED_COMPLEMENT = 2;
    
    /** Creates new EdgeValueDialog
     */
    /*
    EdgeValueDialog (Frame parent, boolean modal, RexEdgeElement initValue) {
    }
     */
    
    /**
     * @param eps whether option with epsilon edge should be enabled
     */
    // !!! param modal not necessary, always modal
    EdgeValueDialog (Frame parent, boolean modal, RexEdgeElement initValue, boolean eps) {
        super (parent, true);
        // ! this inits could be lazy and also static, probably
        upperLetter = RexEdgeFactory.createLetter (RexEdgeFactory.UPPER_CASE);
        lowerLetter = RexEdgeFactory.createLetter (RexEdgeFactory.LOWER_CASE);
        bothLetter = RexEdgeFactory.createLetter (RexEdgeFactory.BOTH_CASE);
        alfaNum = RexEdgeFactory.createAlfaNum ();
        // bug - the inits above should be before initComponents ()
        initComponents ();
        //emptyRadioButton.setText("empty string " + (char)0x03B5); 
        // ! "epsilon", but needs greek locale
        emptyRadioButton.setText("empty string (" + (char)0x220A + ") [NFA only]");
        // ! "is element sign"
        //
        letterCaseCombo.addItem (UPPER_CASE_OPTION);
        letterCaseCombo.addItem (LOWER_CASE_OPTION);
        letterCaseCombo.addItem (BOTH_CASE_OPTION);
        buttonGroup.add (jRadioButton1);
        buttonGroup.add (jRadioButton2);
        buttonGroup.add (jRadioButton3);
        buttonGroup.add (exceptRadioButton);
        buttonGroup.add (fromRadioButton);
        buttonGroup.add (intervalRadioButton);
        buttonGroup.add (emptyRadioButton);
        //
        valueTextField.setRequestFocusEnabled (true);
        edgeValue = initValue;
        if (null != edgeValue) {
            valueTextField.setText (edgeValue.toString ());
        }
    }
    
    /**
     * @return the return status of this dialog - one of RET_OK or RET_CANCEL
     */
    public int getReturnStatus () {
        return returnStatus;
    }
    
    /**
     */
    public RexEdgeElement getValue () {
        System.out.println("edgeValue: " + edgeValue);
        return edgeValue;
    }
    
    private void handleSelectionChange (RexEdgeElement value) {
        edgeValue = value;
        valueTextField.setText (value.toString ());
    }
    
    private void handleSelectionBeforeExit () {
        if (intervalRadioButton.getModel ().equals (buttonGroup.getSelection ())) {
            handleIntervalRadioButton ();
        }
        else if (exceptRadioButton.getModel ().equals (buttonGroup.getSelection ())) {
            handleExceptRadioButton ();
        }
        else if (fromRadioButton.getModel ().equals (buttonGroup.getSelection ())) {
            handleFromRadioButton ();
        }
    }
    
    private void handleFromRadioButton () {
        String str = fromTextField.getText ();
        if (str.length () < 1) {
            return;
        }
        handleSelectionChange (RexEdgeFactory.createFrom (str));
    }
    
    private void handleExceptRadioButton () {
        String except = exceptTextField.getText ();
        if (except.length () < 1) {
            return;
        }
        handleSelectionChange (RexEdgeFactory.createExcept (except));
    }
    
    private void handleIntervalRadioButton () {
        String fromStr = jTextField4.getText ();
        String toStr = jTextField5.getText ();
        char from;
        char to;
        if (fromStr.length () != 1) {
            return;
        } else {
            from = fromStr.toCharArray ()[0];
        }
        if (toStr.length () != 1) {
            return;
        } else {
            to = toStr.toCharArray ()[0];
        }
        // !!! testovat jestli je from <= to
        ComplexRexEdgeElement elem = new ComplexRexEdgeElement (false);
        elem.addInterval (from, to);
        handleSelectionChange (elem);
    }
    
    private void handleCaseComboChanged (Object item) {
        if (item.equals (UPPER_CASE_OPTION)) {
            handleSelectionChange (upperLetter);
        }
        else if (item.equals (LOWER_CASE_OPTION)) {
            handleSelectionChange (lowerLetter);
        }
        else if (item.equals (BOTH_CASE_OPTION)) {
            handleSelectionChange (bothLetter);
        } else {
            throw new IllegalStateException ("unknown option of combo: " + item);
        }
    }
    
 
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroup = new javax.swing.ButtonGroup();
        jPanel6 = new javax.swing.JPanel();
        valueTextField = new javax.swing.JTextField();
        valueLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jRadioButton1 = new javax.swing.JRadioButton();
        letterCaseCombo = new javax.swing.JComboBox();
        jRadioButton2 = new javax.swing.JRadioButton();
        jRadioButton3 = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        exceptRadioButton = new javax.swing.JRadioButton();
        exceptTextField = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        fromRadioButton = new javax.swing.JRadioButton();
        fromTextField = new javax.swing.JTextField();
        jPanel5 = new javax.swing.JPanel();
        intervalRadioButton = new javax.swing.JRadioButton();
        jTextField4 = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jTextField5 = new javax.swing.JTextField();
        emptyRadioButton = new javax.swing.JRadioButton();
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("Edit edge value");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel6.setLayout(new java.awt.GridBagLayout());

        valueTextField.setPreferredSize(new java.awt.Dimension(200, 21));
        valueTextField.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        jPanel6.add(valueTextField, gridBagConstraints);

        valueLabel.setText("Value");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        jPanel6.add(valueLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        getContentPane().add(jPanel6, gridBagConstraints);

        jPanel2.setLayout(new java.awt.GridBagLayout());

        jPanel2.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        jPanel1.setLayout(new java.awt.GridBagLayout());

        jRadioButton1.setText("any letter");
        jRadioButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton1ActionPerformed(evt);
            }
        });

        jRadioButton1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jRadioButton1StateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(jRadioButton1, gridBagConstraints);

        letterCaseCombo.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                letterCaseComboItemStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(letterCaseCombo, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 10);
        jPanel2.add(jPanel1, gridBagConstraints);

        jRadioButton2.setText("any digit");
        jRadioButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(jRadioButton2, gridBagConstraints);

        jRadioButton3.setText("any alphanumeric char");
        jRadioButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton3ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(jRadioButton3, gridBagConstraints);

        jPanel3.setLayout(new java.awt.GridBagLayout());

        exceptRadioButton.setText("any character except from");
        exceptRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exceptRadioButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(exceptRadioButton, gridBagConstraints);

        exceptTextField.setPreferredSize(new java.awt.Dimension(60, 21));
        jPanel3.add(exceptTextField, new java.awt.GridBagConstraints());

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(jPanel3, gridBagConstraints);

        jPanel4.setLayout(new java.awt.GridBagLayout());

        fromRadioButton.setText("any character from");
        fromRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fromRadioButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel4.add(fromRadioButton, gridBagConstraints);

        fromTextField.setPreferredSize(new java.awt.Dimension(100, 21));
        fromTextField.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                fromTextFieldInputMethodTextChanged(evt);
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
                fromTextFieldCaretPositionChanged(evt);
            }
        });

        jPanel4.add(fromTextField, new java.awt.GridBagConstraints());

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(jPanel4, gridBagConstraints);

        jPanel5.setLayout(new java.awt.GridBagLayout());

        intervalRadioButton.setText("any character between");
        intervalRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                intervalRadioButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(intervalRadioButton, gridBagConstraints);

        jTextField4.setPreferredSize(new java.awt.Dimension(30, 21));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        jPanel5.add(jTextField4, gridBagConstraints);

        jLabel1.setBackground(java.awt.Color.black);
        jLabel1.setForeground(java.awt.Color.black);
        jLabel1.setText("and");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        jPanel5.add(jLabel1, gridBagConstraints);

        jTextField5.setPreferredSize(new java.awt.Dimension(30, 21));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        jPanel5.add(jTextField5, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(jPanel5, gridBagConstraints);

        emptyRadioButton.setText("empty string (this label is DEBUG ONLY)");
        emptyRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                emptyRadioButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 10);
        jPanel2.add(emptyRadioButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jPanel2, gridBagConstraints);

        buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        okButton.setText("OK");
        okButton.setPreferredSize(new java.awt.Dimension(73, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(cancelButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(buttonPanel, gridBagConstraints);

        pack();
    }//GEN-END:initComponents

    private void emptyRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_emptyRadioButtonActionPerformed
        handleSelectionChange (RexEmptyElement.create ());
    }//GEN-LAST:event_emptyRadioButtonActionPerformed
    
    private void exceptRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exceptRadioButtonActionPerformed
        handleExceptRadioButton ();
    }//GEN-LAST:event_exceptRadioButtonActionPerformed
    
    private void intervalRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_intervalRadioButtonActionPerformed
        handleIntervalRadioButton ();
    }//GEN-LAST:event_intervalRadioButtonActionPerformed
    
    private void jRadioButton3ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButton3ActionPerformed
        handleSelectionChange (alfaNum);
    }//GEN-LAST:event_jRadioButton3ActionPerformed
    
    private void fromTextFieldCaretPositionChanged (java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_fromTextFieldCaretPositionChanged

    }//GEN-LAST:event_fromTextFieldCaretPositionChanged
    
    private void fromRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fromRadioButtonActionPerformed
        handleFromRadioButton ();
    }//GEN-LAST:event_fromRadioButtonActionPerformed
    
    private void fromTextFieldInputMethodTextChanged (java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_fromTextFieldInputMethodTextChanged

    }//GEN-LAST:event_fromTextFieldInputMethodTextChanged
    
    private void jRadioButton1StateChanged (javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jRadioButton1StateChanged
        letterCaseCombo.setEnabled (jRadioButton1.isSelected ());
    }//GEN-LAST:event_jRadioButton1StateChanged
    
    private void letterCaseComboItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_letterCaseComboItemStateChanged
        if (evt.getStateChange () == evt.SELECTED) {
            handleCaseComboChanged (evt.getItem ());
        }
        
    }//GEN-LAST:event_letterCaseComboItemStateChanged
    
    private void jRadioButton2ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButton2ActionPerformed
        handleSelectionChange (new ComplexRexEdgeElement ('0', '9'));
    }//GEN-LAST:event_jRadioButton2ActionPerformed
    
    private void jRadioButton1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButton1ActionPerformed
        handleCaseComboChanged (letterCaseCombo.getSelectedItem ());
    }//GEN-LAST:event_jRadioButton1ActionPerformed
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        doClose (RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        doClose (RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose (RET_CANCEL);
    }//GEN-LAST:event_closeDialog
    
    private void doClose (int returnStatus) {
        this.returnStatus = returnStatus;
        if (returnStatus == RET_OK) {
            handleSelectionBeforeExit ();
        }
        setVisible (false);
        dispose ();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        EdgeValueDialog dialog = new EdgeValueDialog (new JFrame (),
        true, RexEdgeFactory.createAlfaNum (), true);
        dialog.show ();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JComboBox letterCaseCombo;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JTextField exceptTextField;
    private javax.swing.JRadioButton exceptRadioButton;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton okButton;
    private javax.swing.ButtonGroup buttonGroup;
    private javax.swing.JTextField valueTextField;
    private javax.swing.JTextField jTextField5;
    private javax.swing.JTextField jTextField4;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JButton cancelButton;
    private javax.swing.JRadioButton intervalRadioButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JTextField fromTextField;
    private javax.swing.JRadioButton fromRadioButton;
    private javax.swing.JLabel valueLabel;
    private javax.swing.JRadioButton emptyRadioButton;
    // End of variables declaration//GEN-END:variables
    
    private int returnStatus = RET_CANCEL;
}
