/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.logic.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.beans.*;
import javax.swing.SwingUtilities;

/**
 *
 * @author  Petr Slegr
 * @version 21.6.01/2
 */
public class VisualAutomatonNode extends VisualAutomatonElement {
    
    // !!!!! presunout do NodeGeometry
    public static final int DEFAULT_NODE_SIZE = 30; // was 12
    
    public static final String PROP_FINAL_NODE = new String ("finalNode");
    
    public static final String PROP_STARTING = new String ("starting");
    
    // !!??
    public static final String PROP_SIZE = new String ("size");
    
    protected LogicAutomatonNode logicNode;
    
    // !!!!! presunout do NodeGeometry
    protected static int size = DEFAULT_NODE_SIZE;

    protected int id;
    
    private static int idCounter;
    
    /** For creation of VisualAutomatonContainers from templates.
     */
    protected VisualAutomatonNode (LogicAutomatonNode logicNode) {
        super ();
        this.logicNode = logicNode;
        id = idCounter++;
    }

    /**
     */
    VisualAutomatonNode (Point position, LogicAutomatonNode logicNode) {
        super ();
        this.logicNode = logicNode;
        this.position = new PointPlus (position);
        setHumanName ("Node");  // !! bundle
        geometry = prepareGeometry ();
        id = idCounter++;
    }
    
    /**
     */
    // !!!! zrusit - misto uzivatelskych id bude humanName (obecnejsi)
    public void setID (int id) {
        this.id = id;
        geometry = prepareGeometry ();
    }
    
    public int getID () {
        return id;
    }
    
    /**
     */
    public String toString () {
        StringBuffer str = new StringBuffer ();
        str.append ("vnode: ");
        str.append (logicNode.getID ());
        str.append (" ");
        str.append (hashCode ());
        return str.toString ();
    }
    
    /** Test whether specified edge is from/to (output/input) this node.
     * @return true if so
     */
    public boolean containsEdge (VisualAutomatonEdge edge) {
        return logicNode.containsEdge (edge.getLogicEdge ());
    }
    
    /** Return Set of all edges (both input and output)
     */
    public Set getEdges () {
        Set res = new HashSet ();
        Set logicEdges = logicNode.getEdges ();
        for (Iterator it = logicEdges.iterator (); it.hasNext (); ) {
            res.add (container.find ((LogicAutomatonEdge) it.next ()));
        }
        return res;
    }
    
    /**
     * @return true when edge with the same to node
     * was found.
     */
    /*
    private boolean checkFromEdge (VisualAutomatonEdge edge) {
        VisualAutomatonNode to = edge.getToNode ();
        Iterator it = outputEdges.iterator ();
        boolean match = false;
        while (it.hasNext ()) {
            VisualAutomatonEdge e = (VisualAutomatonEdge) it.next ();
            if (e.getToNode ().equals (to)) {
                match = true;
                System.out.println("match: " + match);
                return match;
            }
        }
        return false;
    }
     */
    
    /**
     * @return true when edge with the same to node
     * was found.
     */
    /*
    private boolean checkToEdge (VisualAutomatonEdge edge) {
        VisualAutomatonNode from = edge.getFromNode ();
        Iterator it = inputEdges.iterator ();
        boolean match = false;
        while (it.hasNext ()) {
            VisualAutomatonEdge e = (VisualAutomatonEdge) it.next ();
            if (e.getFromNode ().equals (from)) {
                match = true;
                System.out.println("match: " + match);
                return match;
            }
        }
        return false;
    }
     */
    
    public void addPropertyChangeListener (String name, PropertyChangeListener pcl) {
        super.addPropertyChangeListener (name, pcl);
        propChangeSup.addPropertyChangeListener (name, pcl);
    }
    
    public void removePropertyChangeListener (String name, PropertyChangeListener pcl) {
        super.removePropertyChangeListener (name, pcl);
        propChangeSup.removePropertyChangeListener (name, pcl);
    }
    
    /**
     */
    public boolean isOn (Point p) {
        //throw new UnsupportedOperationException ();
        return geometry.isOn (p);
    }
    
    /**
     */
    public boolean isDragable () {
        return true;
    }
    
    public LogicAutomatonNode getLogicNode () {
        return logicNode;
    }
    
    public boolean isOnForSelection (Point p) {
        return isOn (p);
    }
    
    /** Getter for property starting.
     * @return Value of property starting.
     */
    public boolean isStarting () {
        return logicNode.isStarting ();
    }
    
    /** Setter for property starting.
     * @param starting New value of property starting.
     */
    public void setStarting (boolean starting) {
        boolean oldValue = logicNode.isStarting ();
        boolean newValue = starting;
        if (oldValue == newValue) {
            // new value is the same as the current one
            return;
        }
        logicNode.setStarting (newValue);
        geometry = prepareGeometry ();
        //
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_STARTING,
        new Boolean (oldValue), new Boolean (newValue));
        VisualAutomatonChange edit = new VisualAutomatonChange (container, pce);
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
        propChangeSup.firePropertyChange (pce);
    }
    
    /**
     */
    public void setStartingAsSystem (boolean starting) {
        boolean oldStarting = logicNode.isStarting ();
        logicNode.setStarting (starting);
        geometry = prepareGeometry ();
        propChangeSup.firePropertyChange (PROP_POSITION, oldStarting, starting);
    }
    
    public boolean isFinalNode () {
        return logicNode.isFinalNode ();
    }
    
    public void setFinalNode (boolean finalNode) {
        boolean oldValue = logicNode.isFinalNode ();
        boolean newValue = finalNode;
        if (oldValue == newValue) {
            return;
        }
        logicNode.setFinalNode (newValue);
        geometry = prepareGeometry ();
        //
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_FINAL_NODE,
        new Boolean (oldValue), new Boolean (newValue));
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
        propChangeSup.firePropertyChange (pce);
    }
    
    void setFinalNodeAsSystem (boolean finalNode) {
        boolean oldValue = logicNode.isFinalNode ();
        logicNode.setFinalNode (finalNode);
        geometry = prepareGeometry ();
        propChangeSup.firePropertyChange (PROP_FINAL_NODE, oldValue, finalNode);
    }
    
    public boolean isIsolated () {
        return logicNode.isIsolated ();
    }
    
    public boolean hasOutputEdgeTo (VisualAutomatonNode to) {
        return logicNode.containsEdgeByOutputNode (to.getLogicNode ());
    }
    
    public void handleDeleteOn (Point position) {
        //super.handleDeleteOn (position);
        container.removeElement (this);
    }
    
    public void addPropertyChangeListener (PropertyChangeListener pcl) {
        super.addPropertyChangeListener (pcl);
        propChangeSup.addPropertyChangeListener (pcl);
    }
    
    public void removePropertyChangeListener (PropertyChangeListener pcl) {
        super.removePropertyChangeListener (pcl);
        propChangeSup.removePropertyChangeListener (pcl);
    }
    
}
