/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted.actions;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.netbeans.modules.regsup.auted.*;
import org.netbeans.modules.regsup.logic.*;
import java.awt.Point;

/** Creates from selected node to itself (i.e. "self-edge").
 *
 * @author  Petr Slegr
 */
public class CreateSelfEdgeAction extends ElementAction {

    public void performAction (VisualAutomatonElement[] selected) {
        VisualAutomatonNode node = (VisualAutomatonNode) selected [0];
        VisualAutomatonContainer container = node.getContainer ();
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        Point position = editor.getCurrentPosition ();
        // secure because of checking in enabled
        //VisualAutomatonNode node = (VisualAutomatonNode) container.findElementOn (position);
        position.translate (40, 40); // !! const
        RexEdgeElement rexValue = UserSupport.letUserEditEdgeValue (null, position);
        // !!!!! nahradit null
        container.createSelfEdge (node, rexValue);
        //editor.resetSelectedElements ();
    } 

    public String getName () {
        //return NbBundle.getMessage (CreateEdgeAction.class, "CTL_CREATE_EDGE_ACTION");
        return "Create Self Edge";
    }

    protected String iconResource () {
        return "ActionIcon.gif";
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (AddNodeActionAction.class);
    }

    /** This action is enabled only on VisualAutomatonNode, not on its subclasses.
     * @return <code>true</code> if so
     * @return false if node has already self edge
     */
    public boolean isEnabled (VisualAutomatonElement[] selected) {
        if (selected.length != 1) {
            return false;
        }
        if (! (selected[0].getClass ().equals (VisualAutomatonNode.class))) {
            return false;
        }
        VisualAutomatonNode node = (VisualAutomatonNode) selected[0];
        if (node.getLogicNode ().hasSelfEdge ()) {
            return false;
        }
        return true;
    }
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
    protected void initialize () {
	super.initialize ();
     * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (AddNodeActionAction.class, "HINT_Action"));
    }
    */

}
