/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.debug;

/** Ancestor of all breakpoints. API user should override at least one from
 * applies methods. By default they return false.
 *
 * @author  Petr Slegr
 * @version 28.12.01
 */
public abstract class RexBreakpoint {

    private boolean enabled = true;
    
    /** Holds value of property hidden. */
    private boolean hidden;
    
    /** Holds value of property suspend. */
    private boolean suspend = true;
    
    /** Property whether all applies method should called even if some of them
     * returned true.
     * By default false.
     */
    protected boolean evaluateAll = false;
    
    /** Creates new RexBreakpoint */
    public RexBreakpoint () {
    }

    /** 
     * @return true if breakpoint applies for specified debugger state.
     * @return By default return false.
     */
    public boolean applies (RexDebuggerState state) {
        return false;
    }
    
    /**
     * @return true if breakpoint applies for debugger in some state
     * @return By default return false.
     */
    public boolean applies (RexDebugger debugger) {
        return false;
    }
    
    /**
     */
    public boolean isEnabled () {
        return enabled;
    }
    
    /**
     */
    public void setEnabled (boolean enabled) {
        this.enabled = enabled;
    }
    
    /** Getter for property hidden.
     * @return Value of property hidden.
     */
    public boolean isHidden () {
        return this.hidden;
    }    
    
    /** Setter for property hidden.
     * @param hidden New value of property hidden.
     */
    public void setHidden (boolean hidden) {
        this.hidden = hidden;
    }
    
    /** Getter for property suspend.
     * @return Value of property suspend.
     */
    public boolean isSuspend () {
        return this.suspend;
    }
    
    /** Setter for property suspend.
     * @param suspend New value of property suspend.
     */
    public void setSuspend (boolean suspend) {
        this.suspend = suspend;
    }
    
    boolean isEvaluateAll () {
        return evaluateAll;
    }
    
    public String toString () {
        return super.toString () + " suspend: " + suspend + " hidden: " + hidden;
    }
    
}
