/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.ide;

import org.xml.sax.*;
import java.net.URL;
import org.netbeans.modules.regsup.auted.VisualAutomatonContainer;

/**
 * The class reads XML documents according to specified DTD and
 * translates all related events into AutomatonHandler events.
 * <p>Usage sample:
 * <pre>
 *    AutomatonParser parser = new AutomatonParser(...);
 *    parser.parse(new InputSource("..."));
 * </pre>
 * <p><b>Warning:</b> the class is machine generated. DO NOT MODIFY</p>
 */
public class AutomatonParser implements DocumentHandler {
    
    private java.lang.StringBuffer buffer;
    
    private AutomatonHandler handler;
    
    private java.util.Stack context;
    
    public AutomatonParser (final AutomatonHandler handler) {
        this.handler = handler;
        buffer = new StringBuffer (111);
        context = new java.util.Stack ();
    }
    
    /*
    public VisualAutomatonContainer getResult () {
        return handler.getResult ();
    }
     */
    
    public static void main (String[] args) {
        try {
            URL url = AutomatonParser.class.getResource ("automaton.xml");
            System.out.println("url: " + url);
            parse (url, new AutomatonHandlerImpl ());
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    public void characters (char[] chars, int start, int len) throws SAXException {
        buffer.append (chars, start, len);
    }
    
    private void dispatch (final boolean fireOnlyIfMixed) throws SAXException {
        if (fireOnlyIfMixed && buffer.length () == 0) return; //skip it
        
        Object[] ctx = (Object[]) context.peek ();
        String here = (String) ctx[0];
        AttributeList attrs = (AttributeList) ctx[1];
        if ("final".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_final (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("nodeId".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_nodeId (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("longHumanName".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_longHumanName (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("humanName".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_humanName (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("y".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_y (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("x".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_x (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("value".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_value (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("starting".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_starting (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else if ("templateName".equals (here)) {
            if (fireOnlyIfMixed) throw new IllegalStateException ("Unexpected characters() event! (Missing DTD?)");
            handler.handle_templateName (buffer.length () == 0 ? null : buffer.toString (), attrs);
        } else {
            //do not care
        }
        buffer.delete (0, buffer.length ());
    }
    
    public void endDocument () throws SAXException {
    }
    
    public void endElement (java.lang.String name) throws SAXException {
        dispatch (false);
        context.pop ();
        if ("source".equals (name)) {
            handler.end_source ();
        } else if ("container".equals (name)) {
            handler.end_container ();
        } else if ("edge".equals (name)) {
            handler.end_edge ();
        } else if ("node".equals (name)) {
            handler.end_node ();
        } else if ("position".equals (name)) {
            handler.end_position ();
        } else if ("target".equals (name)) {
            handler.end_target ();
        } else if ("content".equals (name)) {
            handler.end_content ();
        } else if ("automaton".equals (name)) {
            handler.end_automaton ();
        }
    }
    
    private ErrorHandler getDefaultErrorHandler () {
        return new ErrorHandler () {
            public void error (SAXParseException ex) throws SAXException  {
                if (context.isEmpty ()) System.err.println ("Missing DOCTYPE.");
                throw ex;
            }
            
            public void fatalError (SAXParseException ex) throws SAXException {
                throw ex;
            }
            
            public void warning (SAXParseException ex) throws SAXException {
                // ignore
            }
        };
        
    }
    
    public void ignorableWhitespace (char[] chars, int start, int len) throws SAXException {
    }
    
    /**
     * The recognizer entry method taking an InputSource.
     * @param input InputSource to be parsed.
     * @throws java.io.IOException on I/O error.
     * @throws SAXException propagated exception thrown by a DocumentHandler.
     * @throws javax.xml.parsers.ParserConfigurationException a parser satisfining requested configuration can not be created.
     * @throws javax.xml.parsers.FactoryConfigurationRrror if the implementation can not be instantiated.
     */
    public void parse (final InputSource input) throws SAXException, javax.xml.parsers.ParserConfigurationException, java.io.IOException {
        parse (input, this);
    }
    
    /**
     * The recognizer entry method taking a URL.
     * @param url URL source to be parsed.
     * @throws java.io.IOException on I/O error.
     * @throws SAXException propagated exception thrown by a DocumentHandler.
     * @throws javax.xml.parsers.ParserConfigurationException a parser satisfining requested configuration can not be created.
     * @throws javax.xml.parsers.FactoryConfigurationRrror if the implementation can not be instantiated.
     */
    public void parse (final java.net.URL url) throws SAXException, javax.xml.parsers.ParserConfigurationException, java.io.IOException {
        parse (new InputSource (url.toExternalForm ()), this);
    }
    
    /**
     * The recognizer entry method taking an Inputsource.
     * @param input InputSource to be parsed.
     * @throws java.io.IOException on I/O error.
     * @throws SAXException propagated exception thrown by a DocumentHandler.
     * @throws javax.xml.parsers.ParserConfigurationException a parser satisfining requested configuration can not be created.
     * @throws javax.xml.parsers.FactoryConfigurationRrror if the implementation can not be instantiated.
     */
    public static void parse (final InputSource input, final AutomatonHandler handler) throws SAXException, javax.xml.parsers.ParserConfigurationException, java.io.IOException {
        parse (input, new AutomatonParser (handler));
    }
    
    /**
     * The recognizer entry method taking a URL.
     * @param url URL source to be parsed.
     * @throws java.io.IOException on I/O error.
     * @throws SAXException propagated exception thrown by a DocumentHandler.
     * @throws javax.xml.parsers.ParserConfigurationException a parser satisfining requested configuration can not be created.
     * @throws javax.xml.parsers.FactoryConfigurationRrror if the implementation can not be instantiated.
     */
    public static void parse (final java.net.URL url, final AutomatonHandler handler) throws SAXException, javax.xml.parsers.ParserConfigurationException, java.io.IOException {
        parse (new InputSource (url.toExternalForm ()), handler);
    }
    
    private static void parse (final InputSource input, final AutomatonParser recognizer) throws SAXException, javax.xml.parsers.ParserConfigurationException, java.io.IOException {
        javax.xml.parsers.SAXParserFactory factory = javax.xml.parsers.SAXParserFactory.newInstance ();
        //factory.setValidating (true);  //the code was generated according DTD
        factory.setNamespaceAware (false);  //the code was generated according DTD
        Parser parser = factory.newSAXParser ().getParser ();
        parser.setDocumentHandler (recognizer);
        parser.setErrorHandler (recognizer.getDefaultErrorHandler ());
        parser.parse (input);
    }
    
    public void processingInstruction (java.lang.String target, java.lang.String data) throws SAXException {
    }
    
    public void setDocumentLocator (Locator locator) {
    }
    
    public void startDocument () throws SAXException {
    }
    
    public void startElement (java.lang.String name, AttributeList attrs) throws SAXException {
        dispatch (true);
        context.push (new Object[] {name, new org.xml.sax.helpers.AttributeListImpl (attrs)});
        if ("source".equals (name)) {
            handler.start_source (attrs);
        } else if ("container".equals (name)) {
            handler.start_container (attrs);
        } else if ("edge".equals (name)) {
            handler.start_edge (attrs);
        } else if ("node".equals (name)) {
            handler.start_node (attrs);
        } else if ("position".equals (name)) {
            handler.start_position (attrs);
        } else if ("target".equals (name)) {
            handler.start_target (attrs);
        } else if ("content".equals (name)) {
            handler.start_content (attrs);
        } else if ("automaton".equals (name)) {
            handler.start_automaton (attrs);
        }
    }
    
}

