/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.logic;

import java.util.ArrayList;
import java.util.Set;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Collections;
import java.util.List;
import java.util.HashSet;

/**
 *
 * @author  Petr Slegr
 * @version 01.05.02
 */
public class ComplexRexEdgeElement extends RexEdgeElement {
    
    //protected ArrayList intervals;
    protected Set intervals;
    
    protected ArrayList enumeration;
    
    protected boolean complemented;
    
    protected ComplexRexEdgeElement innerForm;
    
    /** Creates simple positive edge
     */
    public ComplexRexEdgeElement (char ch) {
        this (false);
        addSingle (ch);
    }
    
    public ComplexRexEdgeElement (boolean complemented) {
        //intervals = new ArrayList ();
        intervals = new TreeSet ();
        enumeration = new ArrayList ();
        this.complemented = complemented;
    }
    
    public ComplexRexEdgeElement (char from, char to, boolean complemented) {
        this (complemented);
        addInterval (from, to);
    }
    
    public ComplexRexEdgeElement (char from, char to) {
        this (from, to, false);
    }
    
    public boolean isEmpty () {
        return intervals.size () == 0 && enumeration.size () == 0;
    }
    
    /** Modifies list of intervals
     */
    private void tryAddInterval (Interval interv) {
        int size = intervals.size ();
        if (size == 0) {
            intervals.add (interv);
            return;
        }
        //
        Iterator it = intervals.iterator ();
        Interval toAdd = null;
        //Enumeration en = Collections.enumeration (intervals);
        while (it.hasNext ()) {
            Interval in = (Interval) it.next ();
            if (in.contains (interv)) {
                return;
            }
            if (interv.contains (in)) {
                it.remove ();
                //intervals.add (interv);
                toAdd = interv;
            } else {
                //intervals.add (interv);
                toAdd = interv;
            }
        }
        if (null != toAdd) {
            intervals.add (toAdd);
        }
    }
    
    /**
     */
    public void addInterval (char from, char to) {
        Interval temp = new Interval (from, to);
        tryAddInterval (temp);
        innerForm = null;
    }
    
    /**
     */
    public void addInterval (Interval in) {
        tryAddInterval (in);
        innerForm = null;
    }
    
    /** Checks whether ch is not within some intervals
     * @return true if some interval contains ch
     * @return true if some intervals starts or ends with char greater or less than ch resp.
     * In such case modify the interval to contain ch.
     * @sidef
     */
    private boolean checkSingle (char ch) {
        /* NOT NECESSARY
        if (intervals.size () == 0) {
            // empty set - nothing to check
            return false;
        }
         */
        Character cha = new Character (ch);
        Iterator it = intervals.iterator ();
        Interval in = null;
        Interval repl = null;
        int neib = 0;
        while (it.hasNext ()) {
            in = (Interval) it.next ();
            if (in.contains (cha)) {
                return true;
            }
            neib = in.neibs (cha);
            if (neib != 0) {
                if (neib < 0) {
                    repl = new Interval (prevChar (in.from), in.to);
                }
                if (neib > 0) {
                    repl = new Interval (in.from, nextChar (in.to));
                }
                break;
            }
        }
        if (neib != 0) {
            intervals.remove (in);
            intervals.add (repl);
            //System.out.println ("ok");
            return true;
        }
        return false;
    }
    
    /**
     */
    public void addSingle (char ch) {
        if (checkSingle (ch)) {
            //System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
            return;
        }
        // !! checkovat duplicitu ??
        //RexCharElement elem = new RexCharElement (ch);
        //boolean res = enumeration.add (elem);
        boolean res = enumeration.add (new Character (ch));
        //
        eliminateSingles ();
        //
        innerForm = null;
    }
    
    /** Creates new ComplexRexEdgeElement object without single chars, i.e.
     * [^abe-k] replaces by [^a-ab-be-k].
     */
    public ComplexRexEdgeElement eliminateSingles () {
        ComplexRexEdgeElement res = (ComplexRexEdgeElement) clone ();
        for (int i = 0; i < enumeration.size (); i++) {
            //RexCharElement elem = (RexCharElement) enumeration.get (i);
            Character cha = (Character) enumeration.get (i);
            //System.out.println("eliminating char: " + cha);
            //char ch = elem.getChar ();
            char ch = cha.charValue ();
            res.addInterval (ch, ch);
            //res.enumeration.remove (elem);
            res.enumeration.remove (cha);
        }
        return res;
    }
    
    private static Character prevChar (Character cha) {
        Character res = new Character ((char) (cha.charValue () - 1));
        //System.out.println("prev of: " + cha + " is: " + res);
        return res;
    }
    
    private static Character nextChar (Character cha) {
        Character res = new Character ((char) (cha.charValue () + 1));
        //System.out.println("next of: " + cha + " is: " + res);
        return res;
    }
    
    /** Creates the equivalent ComplexRexEdgeElement without using "^", i.e.
     * not complemented.
     * @return this object (without any modification) if this object is not complemented
     */
    public ComplexRexEdgeElement createPositive () {
        /* bylo misto x1, bug fix
        if (! complemented) {
            return (ComplexRexEdgeElement) clone ();
        }
        ComplexRexEdgeElement source;
        System.out.println("enumeration: " + enumeration);
        if (this.enumeration.size () > 0) {
            source = eliminateSingles ();
        } else {
            source = this;
        }
         */
        // x1
        ComplexRexEdgeElement source;
        if (enumeration.size () > 0) {
            source = eliminateSingles ();
        } else {
            source = (ComplexRexEdgeElement) this.clone ();
        }
        if (! complemented) {
            return source;
        }
        // x1
        //System.out.println("enumeration: " + enumeration);
        ComplexRexEdgeElement elem = new ComplexRexEdgeElement (false);
        Iterator it = source.intervals.iterator ();
        int num = 0;
        Character last = null;
        Character MIN = new Character (Character.MIN_VALUE);
        Character MAX = new Character (Character.MAX_VALUE);
        while (it.hasNext ()) {
            Interval in = (Interval) it.next ();
            Interval repl;
            if (num == 0) {
                if (in.from.compareTo (MIN) > 0) {
                    repl = new Interval (MIN, prevChar (in.from));
                    elem.addInterval (repl);
                    //System.out.println ("step: " + num+ " added interval: " + repl);
                }
            } else {
                repl = new Interval (nextChar (last), prevChar (in.from));
                elem.addInterval (repl);
                //System.out.println ("step: " + num+ " added interval: " + repl);
                
            }
            last = in.to;
            if (num == source.intervals.size () - 1) {
                if (in.to.compareTo (MAX) < 0) {
                    repl = new Interval (nextChar (last), MAX);
                    elem.addInterval (repl);
                    //System.out.println ("step: " + num+ " added interval: " + repl);
                }
            }
            num++;
        }
        return elem;
    }
    
    /**
     */
    public String toString () {
        // whether print "[]"
        boolean bracket = true;
        if (! complemented && enumeration.size () <= 1 && intervals.size () == 0) {
            bracket = false;
        }
        //
        StringBuffer buf = new StringBuffer ();
        if (bracket) {
            buf.append ("[");
        }
        if (complemented) {
            buf.append ('^');   // !! char
        }
        Iterator it = intervals.iterator ();
        while (it.hasNext ()) {
            Interval in = (Interval) it.next ();
            buf.append (in.toString ());
        }
        for (int i=0; i<enumeration.size (); i++) {
            buf.append (enumeration.get (i).toString ());
        }
        if (bracket) {
            buf.append ("]");
        }
        return buf.toString ();
    }
    
    public Object clone () {
        // !!!! prosetrit vsechn klonovani a tohle zvlast !!!
        ComplexRexEdgeElement res = new ComplexRexEdgeElement (complemented);
        res.intervals = (Set) ((TreeSet) intervals).clone ();
        res.enumeration = (ArrayList) enumeration.clone ();
        // !!?? mohla by se klonovat i innerforma?
        return res;
    }
    
    public boolean isComplemented () {
        return complemented;
    }
    
    public int size () {
        return 1;
    }
    
    /**
     */
    public boolean intersects (RexEdgeElement element) {
        if (null == innerForm) {
            innerForm = createPositive ();
        }
        Set sec = element.getIntervals ();
        //System.out.println("this: " + innerForm.intervals);
        //System.out.println("compared: " + sec);
        Iterator first = innerForm.intervals.iterator ();
        //Iterator second = sec.iterator ();
        for (; first.hasNext (); ) {
            Iterator second = sec.iterator ();
            Interval i1 = (Interval) first.next ();
            while (second.hasNext ()) {
                Interval i2 = (Interval) second.next ();
                //System.out.println(i1 + " intersects? " + i2 + " " + i1.intersects (i2));
                if (i1.intersects (i2)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     */
    public boolean includes (char ch) {
        //System.out.println("");
        if (null == innerForm) {
            //System.out.println("CREE: intervals, before crtPositive: " + innerForm);
            innerForm = createPositive ();
            //System.out.println("CREE: intervals, after crtPositive: " + innerForm);
        }
        Iterator it = innerForm.intervals.iterator ();
        while (it.hasNext ()) {
            Interval in = (Interval) it.next ();
            if (in.contains (ch)) {
                //System.out.println(this + " includes: " + ch);
                return true;
            }
        }
        //System.out.println(this + " NOT includes: " + ch);
        return false;
    }
    
    List getSingles () {
        return new ArrayList (enumeration);
    }
    
    Set getIntervals () {
        if (innerForm == null) {
            innerForm = createPositive ();
        }
        return new HashSet (innerForm.intervals);
    }
    
    ////////////////////////////////////////////////////////////////////////////
    /**
     */
    static class Interval implements Comparable {
        
        /** Empty interval
         */
        public static final Interval EMPTY = new Interval ();
        
        /** Interval including all characters
         */
        public static final Interval FULL = new Interval ();
        
        static {
            EMPTY.from = new Character (Character.MAX_VALUE);
            EMPTY.to = new Character (Character.MIN_VALUE);
            FULL.from = new Character (Character.MIN_VALUE);
            FULL.to = new Character (Character.MAX_VALUE);
        }
        
        Character from;
        Character to;
        
        private Interval () {
        }
        
        Interval (char from, char to) {
            this.from = new Character (from);
            this.to = new Character (to);
        }
        
        Interval (Character from, Character to) {
            this.from = from;
            this.to = to;
        }
        
        public String toString () {
            if (equals (EMPTY)) {
                return "000";
            } else {
                return from + "-" + to;
            }
        }
        
        public boolean equals (Object o) {
            if (! (o instanceof Interval)) {
                return false;
            }
            Interval in = (Interval) o;
            if (in.from.equals (from) && in.to.equals (to)) {
                return true;
            }
            return false;
        }
        
        /** Interval contains itself.
         */
        public boolean contains (Interval in) {
            return in.from.compareTo (from) >= 0 && in.to.compareTo (to) <= 0;
        }
        
        public boolean contains (char ch) {
            Character cha = new Character (ch);
            return contains (cha);
        }
        
        public boolean contains (Character cha) {
            return from.compareTo (cha) <= 0 && to.compareTo (cha) >= 0;
        }
        
        /** Returns non-zero if cha is lower or upper neighbour of this interval.
         * E.g. (b, d).neibs (a) returns -1
         * E.g. (b, d).neibs (e) returns 1
         * E.g. (b, d).neibs (f) returns 0
         * E.g. (b, d).neibs (') returns 0
         * E.g. (b, d).neibs (b) returns 0
         * E.g. (b, d).neibs (d) returns 0
         * E.g. (b, d).neibs (c) returns 0
         */
        public int neibs (Character cha) {
            char ch = cha.charValue ();
            char prev = (char) (from.charValue () - 1);
            if (ch == prev) {
                return -1;
            }
            char next = (char) (to.charValue () + 1);
            if (ch == next) {
                return 1;
            }
            return 0;
        }
        
        public boolean intersects (Interval in) {
            //return in.from.compareTo (to) <= 0; // BUG!!
            return inter (in) != EMPTY;
        }
        
        public Interval inter (Interval in) {
            if (in == EMPTY || this == EMPTY) {
                return EMPTY;
            }
            Character nf, nt;
            if (from.compareTo (in.from) >= 0) {
                nf = from;
            } else {
                nf = in.from;
            }
            if (to.compareTo (in.to) <= 0) {
                nt = to;
            } else {
                nt = in.to;
            }
            //
            if (nf.compareTo (nt) > 0) {
                return EMPTY;
            } else {
                return new Interval (nf, nt);
            }
        }
        
        /**
         * (a, b) < (c, d) iff a < c && b < c (is assumed that a < b)
         * (a, b) == (c, d) iff a == c && b == d
         * (a, b) > (c, d) iff a > c && a > d (is assumed that a < b)
         * Otherwise throws IllegalArgumentException.
         */
        public int compareTo (Object o) {
            Interval in = (Interval) o;
            if (from.compareTo (in.from) < 0 && to.compareTo (in.from) < 0) {
                //System.out.println(this + " < " + in);
                return -1;
            }
            if (from.compareTo (in.from) > 0 && from.compareTo (in.to) > 0) {
                //System.out.println(this + " > " + in);
                return 1;
            }
            if (from.equals (in.from) && to.equals (in.to)) {
                //System.out.println(this + " = " + in);
                return 0;
            }
            throw new IllegalArgumentException ("Improper interval object: " + o);
        }
        
    }
    
    private static void eliminateSingleTest () {
        ComplexRexEdgeElement e1 = new ComplexRexEdgeElement ('a');
        System.out.println ("before elim: " + e1);
        ComplexRexEdgeElement e1e = e1.eliminateSingles ();
        System.out.println ("after elim: " + e1e);
        System.out.println ("includes: " + e1e + " a?" + e1e.includes ('a'));
        ComplexRexEdgeElement e2 = new ComplexRexEdgeElement (true);
        e2.addSingle ('b');
        System.out.println ("e2: " + e2);
        System.out.println ("e2.elimSingles: " + e2.eliminateSingles ());
        System.out.println ("e2.createPositive: " + e2.createPositive ());
    }
    
    private static void intersectsTest () {
        RexEdgeElement empty = RexEmptyElement.create ();
        RexEmptyElement e;
        boolean b;
        ComplexRexEdgeElement c;
        ComplexRexEdgeElement d;
        // one char & empty
        //
        c = new ComplexRexEdgeElement ('c');
        b = empty.intersects (c);
        if (! b) {
            throw new RuntimeException ();
        }
        //
        b = c.intersects (empty);
        if (! b) {
            throw new RuntimeException ();
        }
        // ^interval & empty
        // 
        c = new ComplexRexEdgeElement (true);
        c.addInterval ('b', 'd');
        b = empty.intersects (c);
        if (! b) {
            throw new RuntimeException ();
        }
        //
        b = c.intersects (empty);
        if (! b) {
            throw new RuntimeException ();
        }
        // ^ interval & ^ interval (not)
        //
        c = new ComplexRexEdgeElement (true);
        c.addInterval ('b', 'd');
        d = new ComplexRexEdgeElement (true);
        d.addInterval (Character.MIN_VALUE, 'a');
        d.addInterval ('e', Character.MAX_VALUE);
        System.out.println("c: " + c.getIntervals ());
        System.out.println("d: " + d.getIntervals ());
        b = c.intersects (d);
        if (b) {
            throw new RuntimeException ();
        }
        //
        b = d.intersects (c);
        if (b) {
            throw new RuntimeException ();
        }
        // ^ interval & ^ interval
        //
        c = new ComplexRexEdgeElement (true);
        c.addInterval ('b', 'd');
        d = new ComplexRexEdgeElement (true);
        d.addInterval (Character.MIN_VALUE, 'a');
        d.addInterval ('f', Character.MAX_VALUE);
        System.out.println("c: " + c.getIntervals ());
        System.out.println("d: " + d.getIntervals ());
        b = c.intersects (d);
        if (! b) {
            throw new RuntimeException ();
        }
        //
        b = d.intersects (c);
        if (! b) {
            throw new RuntimeException ();
        }
        // (2 x interval) & ^ interval
        //
        c = new ComplexRexEdgeElement (false);
        c.addInterval ('b', 'e');
        c.addInterval ('g', 'k');
        d = new ComplexRexEdgeElement (true);
        d.addInterval ('f', 'f');
        System.out.println("c: " + c.getIntervals ());
        System.out.println("d: " + d.getIntervals ());
        b = c.intersects (d);
        if (! b) {
            throw new RuntimeException ();
        }
        //
        b = d.intersects (c);
        if (! b) {
            throw new RuntimeException ();
        }
        // ^(2 x interval) & interval
        //
        c = new ComplexRexEdgeElement (true);
        c.addInterval ('b', 'e');
        c.addInterval ('g', 'k');
        d = new ComplexRexEdgeElement (false);
        d.addInterval ('f', 'f');
        System.out.println("c: " + c.getIntervals ());
        System.out.println("d: " + d.getIntervals ());
        b = c.intersects (d);
        if (! b) {
            throw new RuntimeException ();
        }
        //
        b = d.intersects (c);
        if (! b) {
            throw new RuntimeException ();
        }
    }
    
    /**
     */
    private static boolean toStringTest () {
        ComplexRexEdgeElement e;
        String s;
        boolean b;
        e = new ComplexRexEdgeElement ('a');
        s = e.toString ();
        b = s.equals ("a");
        if (! b) {
            throw new RuntimeException ("got: " + s + " expected: " + "a");
        }
        e = new ComplexRexEdgeElement ('a', 'c');
        s = e.toString ();
        b = s.equals ("[a-c]");
        if (! b) {
            throw new RuntimeException ("got: " + s + " expected: " + "[a-c]");
        }
        e = new ComplexRexEdgeElement (true);
        e.addSingle ('a');
        s = e.toString ();
        b = s.equals ("[^a]");
        if (! b) {
            throw new RuntimeException ("got: " + s + " expected: " + "[^a]");
        }
        e = new ComplexRexEdgeElement (true);
        e.addInterval ('a', 'c');
        s = e.toString ();
        b = s.equals ("[^a-c]");
        if (! b) {
            throw new RuntimeException ("got: " + s + " expected: " + "[^a-c]");
        }
        e = new ComplexRexEdgeElement (true);
        e.addInterval ('a', 'c');
        e.addSingle ('e');
        e.addInterval ('g','i');
        s = e.toString ();
        b = s.equals ("[^a-cg-ie]"); // form of string is due to impl
        if (! b) {
            throw new RuntimeException ("got: " + s + " expected: " + "[^a-cg-ie]");
        }
        System.out.println ("ok");
        return true;
    }
    
    public static void main (String[] args) {
        //eliminateSingleTest ();
        //toStringTest ();
        //intersectsTest ();
        ComplexRexEdgeElement e = new ComplexRexEdgeElement (false);
        e.addSingle ('a');
        System.out.println("a: " + e.includes ('a'));
        System.out.println("b: " + e.includes ('b'));
        System.out.println ("OK");
        /*
        RexIntervalElement elem = new RexIntervalElement (false);
        elem.add (new RexCharElement ('_'));
        elem.addInterval ('a', 'c');
        System.out.println("elem: " + elem);
         */
        /*
        Interval in = new Interval ('d', 'k');
        Interval i2 = new Interval ('d', 'h');
        Interval i3 = new Interval ('e', 'h');
        Interval i4 = new Interval ('e', 'k');
        Interval i5 = new Interval ('d', 'k');
        Interval i6 = new Interval ('a', 'e');
        Interval i7 = new Interval ('a', 'z');
        Interval i8 = new Interval ('g', 'z');
        Interval i9 = new Interval ('b', 'g');
        Interval i10 = new Interval ('a', 'd');
        Interval i11 = new Interval ('d', 'd');
        Interval i12 = new Interval ('a', 'b');
        Interval i13 = new Interval ('c', 'd');
        // commutativity
        System.out.println ("true: " + i2.intersects (i4));
        System.out.println ("true: " + i4.intersects (i2));
        // subinterval
        System.out.println ("true: " + in.intersects (i2));
        System.out.println ("true: " + i2.intersects (in));
        // overlaping in margin
        System.out.println ("true: " + i8.intersects (i9));
        System.out.println ("true: " + i9.intersects (i8));
        //
        System.out.println ("false: " + i10.intersects (i3));
        System.out.println ("false: " + i10.intersects (i3));
        //
        System.out.println ("false: " + i12.intersects (i13));
        System.out.println ("false: " + i13.intersects (i12));
         
        //
        System.out.println ("b-d " + i10.inter (i9));
        System.out.println ("b-d " + i9.inter (i10));
        //
        System.out.println ("000 " + i9.inter (Interval.EMPTY));
        System.out.println ("000 " + Interval.EMPTY.inter (i9));
        //
        System.out.println ("d-d " + i10.inter (i11));
        System.out.println ("d-d " + i11.inter (i10));
         */
        /*
        System.out.println (i2 + " contained in: " + in + " " + in.contains (i2));
        System.out.println (i3 + " contained in: " + in + " " + in.contains (i3));
        System.out.println (i4 + " contained in: " + in + " " + in.contains (i4));
        System.out.println (i5 + " contained in: " + in + " " + in.contains (i5));
        System.out.println (i6 + " contained in: " + in + " " + in.contains (i6));
        System.out.println (i7 + " contained in: " + in + " " + in.contains (i7));
        System.out.println (i8 + " contained in: " + in + " " + in.contains (i8));
         */
        /*
        System.out.println("a : " + in.isWithin ('a'));
        System.out.println("e : " + in.isWithin ('e'));
        System.out.println("p : " + in.isWithin ('p'));
        System.out.println(" : " + in.isWithin ('p'));
         */
        /*
        ComplexRexEdgeElement elem = new ComplexRexEdgeElement (false);
        elem.addInterval (i2);
        System.out.println ("d-h should: " + elem);
        elem.addInterval (in); // in >
        System.out.println ("d-k should: " + elem);
        elem.addInterval (i3); // i3 <
        System.out.println ("d-k should: " + elem);
        elem.addInterval (i7); // i7 >
        System.out.println ("a-z should: " + elem);
        //
        ComplexRexEdgeElement e2 = new ComplexRexEdgeElement (false);
        e2.addInterval ('a', 'c');
        e2.addInterval ('x', 'z');
        e2.addInterval ('d', 'e');
        e2.addInterval ('k', 'k');
        System.out.println ("a-c, d-e, k-k, x-z: " + e2);
        //
        ComplexRexEdgeElement e3 = new ComplexRexEdgeElement (true);
        e3.addInterval ('a', 'b');
        e3.addSingle ('e');
        System.out.println ("^a-b,d: " + e3);
        ComplexRexEdgeElement e4  = e3.eliminateSingles ();
        System.out.println ("^a-b,d-d: " + e4);
        //
        System.out.println ("MIN-(a-1),c-d,f-MAX: " + e3.createPositive ());
        ComplexRexEdgeElement e5 = new ComplexRexEdgeElement (true);
        e5.addInterval ('a', 'b');
        e5.addSingle ('c');
        System.out.println ("^a-c: " + e5);
        System.out.println ("MIN-(a-1),d-MAX: " + e5.createPositive ());
         */
        /*
        System.out.println("pokus: " + new Character (Character.MIN_VALUE));
        System.out.println("pokus: " + new Character ((char) ('a' - 1)));
        System.out.println("pokus: " + new Character (Character.MAX_VALUE));
         */
        /*
        System.out.println("Intersection of CREEs:");
        ComplexRexEdgeElement e6 = new ComplexRexEdgeElement (false);
        e6.addInterval ('d', 'g');
        ComplexRexEdgeElement e7 = new ComplexRexEdgeElement (true);
        e7.addInterval ('a', 'e');
        ComplexRexEdgeElement e8 = new ComplexRexEdgeElement (true);
        e8.addSingle ('e');
        System.out.println(e6 + " INT " + e7 + " -> true: " + e6.intersects (e7));
        System.out.println(e7 + " INT " + e6 + " -> true: " + e7.intersects (e6));
        System.out.println(e6 + " INT " + e8 + " -> true: " + e6.intersects (e8));
        System.out.println(e8 + " INT " + e6 + " -> true: " + e8.intersects (e6));
        System.out.println(e7 + " INT " + e8 + " -> true: " + e7.intersects (e8));
        System.out.println(e8 + " INT " + e7 + " -> true: " + e8.intersects (e7));
        ComplexRexEdgeElement e9 = new ComplexRexEdgeElement (true);
        e9.addSingle ('e');
        ComplexRexEdgeElement ea = new ComplexRexEdgeElement (false);
        ea.addSingle ('e');
        System.out.println(e9 + " INT " + ea + " -> false: " + e9.intersects (ea));
        System.out.println(ea + " INT " + e9 + " -> false: " + ea.intersects (e9));
         */
        /*
        ArrayList list = new ArrayList ();
        list.add (new Character ('a'));
        System.out.println(list);
        list.add (new Character ('a'));
        System.out.println("contains a: " + list.contains (new Character ('a')));
        System.out.println("contains b: " + list.contains (new Character ('b')));
         */
    }
}
