/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 *
 * @author  Petr Slegr
 * @version
 */
public class RexAlterElement extends RexElement {
    
    protected Set alters;
    
    public RexAlterElement () {
        alters = new HashSet ();
    }
    
    public RexAlterElement (RexElement el) {
        this ();
        add (el);
    }
    
    public RexAlterElement (RexElement e1, RexElement e2) {
        this (e1);
        add (e2);
    }
    
    public void add (RexElement el) {
        if (null == el)
            throw new IllegalArgumentException ("element cannot be null");
        alters.add (el);
    }
    
    /**
     * @return number of TOP LEVEL elements
     */
    public int size () {
        return alters.size ();
    }
    
    public String toString () {
        if (size () == 0)
            // !!??
            return new String ();
        boolean par = true;
        if (size () == 1)
            par = false;
        StringBuffer buf = new StringBuffer ();
        if (par)
            buf.append ('(');   // !! const
        buf.append (toStringWithoutParenthesis ());
        if (par)
            buf.append (')');   // !! const
        return buf.toString ();
    }
    
    public String toStringWithoutParenthesis () {
        StringBuffer buf = new StringBuffer ();
        Iterator it = alters.iterator ();
        while (it.hasNext ()) {
            Object el = it.next ();
            buf.append (el.toString ());
            if (it.hasNext ())
                buf.append ('|');   // !! const
        }
        return buf.toString ();
    }
    
    /** Makes a deep copy
     */
    public Object clone () {
        RexAlterElement elem = new RexAlterElement ();
        Iterator it = alters.iterator ();
        while (it.hasNext ()) {
            RexElement next = (RexElement) it.next ();
            RexElement clone = (RexElement) next.clone ();
            elem.add (clone);
        }
        return elem;
    }
}
