/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic.aut2expr;

import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.logic.LogicAutomatonNode.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 *
 * @author  Petr Slegr
 * @version 26.07.01/1
 */
public class RegEquation implements Cloneable{
    
    private NodeID state;
    
    Hashtable members;
    
    /** Inner RexElement of the node. For simple node value will be null and for
     * compound (container) node its value.
     */
    private RexElement innnerElement;
    
    private boolean emptyMember;
    
    private boolean finalNode;
    
    private boolean starting;
    
    /** Creates new RegEquation */
    public RegEquation (NodeID state, RexElement innnerElement, boolean starting, boolean finalNode) {
        this ();
        if (null == state) {
            throw new IllegalArgumentException ("state cannot be null");
        }
        this.state = state;
        this.innnerElement = innnerElement;
        //members = new Hashtable ();
        this.finalNode = finalNode;
        this.starting = starting;
        if (starting)
            addEmptyMember ();
    }
    
    /** Base constructor
     */
    private RegEquation () {
        members = new Hashtable ();
    }
    
    /** Cloning constructor
     */
    private RegEquation (RegEquation equ) {
        this ();
        this.state = equ.state;
        //this.members = new Hashtable ();
        Enumeration enum = equ.members.keys ();
        while (enum.hasMoreElements ()) {
            Object next = enum.nextElement ();
            // !!?? clonovat next asi neni potreba
            this.members.put (next, ((RexElement) equ.members.get (next)).clone ());
        }
    }
    
    /**
     */
    public boolean isFin () {
        return finalNode;
    }
    
    /**
     */
    public RexElement getSolutionPart () {
        if (members.size () != 1) {
            //System.out.println("eq: " + this);
            //System.out.println("eq.members.size: " + members.size ());
        }
        RexElement res = (RexElement) members.get (RegEquation.EmptyNodeID.getDefault ());
        if (res == null)
            throw new IllegalStateException ("has not solution: " + this);
        return res;
    }
    
    /** 
     * FOR TESTING ONLY
     */
    /*
    static RegEquation create (int first, int num) {
        NodeID state = new NodeID (first);
        RegEquation res = new RegEquation (state, false, false);
        for (int i=first; i<first+num; i++) {
            res.addMember (new NodeID (i), new RexCharElement ((char) ('a' + i)));
        }
        return res;
    }
     */
    
    public NodeID getState () {
        return state;
    }
    
    public int getOrder () {
        return emptyMember ? members.size () - 1 : members.size ();
    }
    
    /**
     * @param cons should not contain some NodeID elem inside
     */
    public void addMember (NodeID var, RexElement cons) {
        RexElement elem;
        if (null != innnerElement) {
            elem = new RexConcatElement (cons, innnerElement);
        } else {
            elem = cons;
        }
        Object res = members.put (var, elem);
        if (null != res)
            throw new IllegalStateException ("member already inserted");
    }
    
    /**
     * FOR TESTING ONLY
     */
    /*
    void addMember (int id, char ch) {
        addMember (new NodeID (id), new RexCharElement (ch));
    }
     */
    
    public void addEmptyMember () {
        addMember (EmptyNodeID.getDefault (), RexEmptyElement.create ());
        emptyMember = true;
    }
    
    /**
     */
    RexElement removeMember (NodeID var) {
        Object res = members.remove (var);
        if (null == res) {
            //throw new IllegalStateException ("member not inserted");
            return RexEmptyElement.create ();
        }
        return (RexElement) res;
    }
    
    /** Eliminates the state on the left side of this equation from it.
     */
    public void eliminate () {
        //System.out.println("before elim: " + this);
        RexElement clos = new RexClosureElement (removeMember (state));
        rightMultiply (clos);
        //System.out.println("eliminated: " + this);
    }
    
    /** Adds equation equ to this equation
     */
    void add (RegEquation equ) {
        //System.out.println("before add: " + this);
        Enumeration enum = equ.members.keys ();
        while (enum.hasMoreElements ()) {
            NodeID var = (NodeID) enum.nextElement ();
            RexElement elem = (RexElement) equ.members.get (var);
            Object current = this.members.get (var);
            if (null == current)
                this.members.put (var, elem);
            else {
                this.members.remove (var);
                RexElement old = (RexElement) current;
                this.members.put (var, new RexAlterElement (old, elem));
            }
        }
        //System.out.println("added: " + this + " with: " + equ);
    }
    
    /** Multiplies this equation by the elem form right
     */
    void rightMultiply (RexElement elem) {
        //System.out.println("before rightMulti: " + this);
        Enumeration enum = members.keys ();
        while (enum.hasMoreElements ()) {
            Object next = enum.nextElement ();
            RexElement e = (RexElement) members.get (next);
            if (e instanceof RexConcatElement)
                ((RexConcatElement) e).add (elem);
            else {
                members.remove (next);
                members.put (next, new RexConcatElement (e, elem));
            }
        }
        //System.out.println("rightMultiplied: " +  this + " by: " + elem);
    }
    
    /**
     */
    public void substitute (RegEquation subst) {
        //System.out.println("before substi: " +  this);
        if (null != subst.members.get (subst.state))
            throw new IllegalArgumentException ("illegal substitution: contains state as member");
        Object o = members.get (subst.state);
        if (null == o) {
            //throw new IllegalArgumentException ("substituted variable not present in this eq.");
            return;
        }
        RegEquation eq = (RegEquation) subst.clone ();
        eq.rightMultiply ((RexElement) o);
        this.add (eq);
        removeMember (subst.state);
        //System.out.println("substituted: " + this + " by: " + subst);
    }
    
    
    
    public Object clone () {
        return new RegEquation (this);
    }
    
    public String toString () {
        StringBuffer buf = new StringBuffer ();
        buf.append (state.toString ());
        buf.append (" = ");
        Enumeration enum = members.keys ();
        while (enum.hasMoreElements ()) {
            Object var = enum.nextElement ();
            Object cons = members.get (var);
            buf.append (var.toString ());
            buf.append (cons.toString ());
            if (enum.hasMoreElements ())
                buf.append ("+");   // !! const
        }
        
        return buf.toString ();
    }
    
    /**
     */
    static class EmptyNodeID extends NodeID {
        
        private EmptyNodeID () {
            super.id = -1;
        }
        private static EmptyNodeID inst = new EmptyNodeID ();
        
        static EmptyNodeID getDefault () {
            return inst;
        }
        
        public String toString () {
            return new String ();
        }
        
    }
    
    public static void main (String[] args) {
        /*
        RegEquation eq1 = create (0, 4);
        eq1.addEmptyMember ();
        System.out.println ("creation: " + eq1);
        eq1.eliminate ();
        System.out.println ("elimination: " + eq1);
        RegEquation eq2 = create (0, 5);
        System.out.println ("eq2: " + eq2);
        eq2.substitute (eq1);
        System.out.println ("elim (eq1) -> eq2: " + eq2);
         */
        /* OLDER
        NodeID self;
        RegEquation eq = new RegEquation (self = new NodeID ());
        NodeID sec = new NodeID ();
        eq.addMember (self, new RexCharElement ('b'));
        eq.addMember (sec, new RexCharElement ('a'));
        System.out.println ("created: " + eq);
        RegEquation eq2 = (RegEquation) eq.clone ();
        eq2.addMember (new NodeID (), new RexCharElement ('c'));
        eq.add (eq2);
        System.out.println ("added to itself: " + eq);
        eq.eliminate ();
        System.out.println("eliminated: " + eq);
         */
    }
    
}
